@@ -7,7 +7,7 @@ if [[ $# -ne 1 ]]; then
   exit 1
 fi
 
-validcommentors="coffeegoddd andy-wm-arthur bheni Hydrocharged reltuk tbantle22 timsehn zachmu max-hoffman"
+validcommentors="coffeegoddd andy-wm-arthur bheni Hydrocharged reltuk tbantle22 timsehn zachmu max-hoffman nicktobey fulghum"
 
 contains() {
     [[ $1 =~ (^|[[:space:]])$2($|[[:space:]]) ]] && echo "valid=true" >> $GITHUB_OUTPUT || exit 0
