@@ -112,6 +112,13 @@ func (cmd BackupCmd) Exec(ctx context.Context, commandStr string, args []string,
 
 	var verr errhand.VerboseError
 
+	// All the sub commands except `restore` require a valid environment
+	if apr.NArg() == 0 || apr.Arg(0) != cli.RestoreBackupId {
+		if !cli.CheckEnvIsValid(dEnv) {
+			return 2
+		}
+	}
+
 	switch {
 	case apr.NArg() == 0:
 		verr = printBackups(dEnv, apr)
