@@ -20,6 +20,7 @@ import (
 	"path"
 	"strings"
 
+	"github.com/dolthub/dolt/go/libraries/doltcore/dconfig"
 	"github.com/dolthub/dolt/go/store/types"
 
 	"github.com/dolthub/dolt/go/cmd/dolt/cli"
@@ -237,15 +238,15 @@ func validateAndParseDolthubUrl(urlStr string) (string, bool) {
 }
 
 func getRemoteUserAndPassConfig(apr *argparser.ArgParseResults) (*creds.DoltCredsForPass, errhand.VerboseError) {
-	if !apr.Contains(cli.UserParam) {
+	if !apr.Contains(cli.UserFlag) {
 		return nil, nil
 	}
-	pass, found := os.LookupEnv("DOLT_REMOTE_PASSWORD")
+	pass, found := os.LookupEnv(dconfig.EnvDoltRemotePassword)
 	if !found {
 		return nil, errhand.BuildDError("error: must set DOLT_REMOTE_PASSWORD environment variable to use --user param").Build()
 	}
 	return &creds.DoltCredsForPass{
-		Username: apr.GetValueOrDefault(cli.UserParam, ""),
+		Username: apr.GetValueOrDefault(cli.UserFlag, ""),
 		Password: pass,
 	}, nil
 }
