@@ -29,9 +29,9 @@ import (
 	goisatty "github.com/mattn/go-isatty"
 
 	"github.com/dolthub/dolt/go/cmd/dolt/cli"
-	"github.com/dolthub/dolt/go/cmd/dolt/commands/engine"
 	"github.com/dolthub/dolt/go/cmd/dolt/errhand"
 	"github.com/dolthub/dolt/go/libraries/doltcore/branch_control"
+	"github.com/dolthub/dolt/go/libraries/doltcore/dconfig"
 	"github.com/dolthub/dolt/go/libraries/doltcore/diff"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
@@ -41,6 +41,7 @@ import (
 	"github.com/dolthub/dolt/go/libraries/utils/iohelp"
 	"github.com/dolthub/dolt/go/libraries/utils/set"
 	"github.com/dolthub/dolt/go/store/datas"
+	"github.com/dolthub/dolt/go/store/util/outputpager"
 )
 
 var commitDocs = cli.CommandDocumentationContent{
@@ -85,26 +86,22 @@ func (cmd CommitCmd) RequiresRepo() bool {
 
 // Exec executes the command
 func (cmd CommitCmd) Exec(ctx context.Context, commandStr string, args []string, dEnv *env.DoltEnv, cliCtx cli.CliContext) int {
-	res, skipped := performCommit(ctx, commandStr, args, cliCtx)
-	if res == 1 {
+	res, skipped := performCommit(ctx, commandStr, args, cliCtx, dEnv)
+	if res != 0 {
 		return res
 	}
 
 	if skipped {
 		iohelp.WriteLine(cli.CliOut, "Skipping empty commit")
-		return res
 	}
 
-	// TODO: switch to using the log command once dolt log is migrated to use sql queries
-	// if the commit was successful, print it out using the log command
-	// return LogCmd{}.Exec(ctx, "log", []string{"-n=1"}, dEnv, nil)
 	return 0
 }
 
 // performCommit creates a new Dolt commit using the specified |commandStr| and |args|. The response is an integer
 // status code indicating success or failure, as well as a boolean that indicates if the commit was skipped
 // (e.g. because --skip-empty was specified as an argument).
-func performCommit(ctx context.Context, commandStr string, args []string, cliCtx cli.CliContext) (int, bool) {
+func performCommit(ctx context.Context, commandStr string, args []string, cliCtx cli.CliContext, temporaryDEnv *env.DoltEnv) (int, bool) {
 	ap := cli.CreateCommitArgParser()
 	help, usage := cli.HelpAndUsagePrinters(cli.CommandDocsForCommandString(commandStr, commitDocs, ap))
 	apr := cli.ParseArgsOrDie(ap, args, help)
@@ -122,6 +119,7 @@ func performCommit(ctx context.Context, commandStr string, args []string, cliCtx
 		defer closeFunc()
 	}
 
+	// dolt_commit performs this check as well. This nips the problem in the bud early, but is not required.
 	err = branch_control.CheckAccess(sqlCtx, branch_control.Permissions_Write)
 	if err != nil {
 		cli.Println(err.Error())
@@ -153,7 +151,7 @@ func performCommit(ctx context.Context, commandStr string, args []string, cliCtx
 	// process query through prepared statement to prevent sql injection
 	query, params, err := constructParametrizedDoltCommitQuery(msg, apr, cliCtx)
 	if err != nil {
-		return 1, false
+		return handleCommitErr(sqlCtx, queryist, err, usage), false
 	}
 	interpolatedQuery, err := dbr.InterpolateForDialect(query, params, dialect.MySQL)
 	if err != nil {
@@ -167,22 +165,21 @@ func performCommit(ctx context.Context, commandStr string, args []string, cliCtx
 	}
 	resultRow, err := sql.RowIterToRows(sqlCtx, schema, rowIter)
 	if err != nil {
+		cli.Println(err.Error())
 		return 1, false
 	}
 	if resultRow == nil {
 		return 0, true
 	}
 
-	// TODO: when dolt log is migrated, remove this block printing out the commit and print with a dolt log call in Exec()
-	schema, rowIter, err = queryist.Query(sqlCtx, "select * from dolt_log() limit 1")
-	if err != nil {
-		cli.Println(err.Error())
-		return 1, false
-	}
-	err = engine.PrettyPrintResults(sqlCtx, engine.FormatTabular, schema, rowIter)
-	if err != nil {
-		cli.Println(err.Error())
-		return 1, false
+	commit, err := getCommitInfo(queryist, sqlCtx, "HEAD")
+	if cli.ExecuteWithStdioRestored != nil {
+		cli.ExecuteWithStdioRestored(func() {
+			pager := outputpager.Start()
+			defer pager.Stop()
+
+			PrintCommitInfo(pager, 0, false, "auto", commit)
+		})
 	}
 
 	return 0, false
@@ -354,7 +351,7 @@ func getCommitMessageFromEditor(sqlCtx *sql.Context, queryist cli.Queryist, sugg
 
 	backupEd := "vim"
 	// try getting default editor on the user system
-	if ed, edSet := os.LookupEnv("EDITOR"); edSet {
+	if ed, edSet := os.LookupEnv(dconfig.EnvEditor); edSet {
 		backupEd = ed
 	}
 	// try getting Dolt config core.editor
@@ -378,7 +375,7 @@ func getCommitMessageFromEditor(sqlCtx *sql.Context, queryist cli.Queryist, sugg
 func checkIsTerminal() bool {
 	isTerminal := false
 	cli.ExecuteWithStdioRestored(func() {
-		if goisatty.IsTerminal(os.Stdout.Fd()) || os.Getenv("DOLT_TEST_FORCE_OPEN_EDITOR") == "1" {
+		if goisatty.IsTerminal(os.Stdout.Fd()) || os.Getenv(dconfig.EnvTestForceOpenEditor) == "1" {
 			isTerminal = true
 		}
 	})
