@@ -23,8 +23,8 @@ import (
 	"strings"
 
 	"github.com/dolthub/go-mysql-server/sql"
-	"github.com/dolthub/go-mysql-server/sql/parse"
 	"github.com/dolthub/go-mysql-server/sql/plan"
+	"github.com/dolthub/go-mysql-server/sql/planbuilder"
 	"github.com/fatih/color"
 
 	"github.com/dolthub/dolt/go/cmd/dolt/cli"
@@ -137,7 +137,7 @@ func (cmd DumpCmd) Exec(ctx context.Context, commandStr string, args []string, d
 	resFormat, _ := apr.GetValue(FormatFlag)
 	resFormat = strings.TrimPrefix(resFormat, ".")
 
-	outputFileOrDirName, vErr := validateArgs(apr)
+	outputFileOrDirName, vErr := validateDumpArgs(apr)
 	if vErr != nil {
 		return HandleVErrAndExitCode(vErr, usage)
 	}
@@ -146,9 +146,9 @@ func (cmd DumpCmd) Exec(ctx context.Context, commandStr string, args []string, d
 	case emptyFileExt, sqlFileExt:
 		var defaultName string
 		if schemaOnly {
-			defaultName = fmt.Sprintf("doltdump_schema_only.sql")
+			defaultName = "doltdump_schema_only.sql"
 		} else {
-			defaultName = fmt.Sprintf("doltdump.sql")
+			defaultName = "doltdump.sql"
 		}
 
 		if outputFileOrDirName == emptyStr {
@@ -269,6 +269,7 @@ func dumpProcedures(sqlCtx *sql.Context, engine *engine.SqlEngine, root *doltdb.
 	}
 
 	stmtColIdx := sch.IndexOfColName(doltdb.ProceduresTableCreateStmtCol)
+	sqlModeIdx := sch.IndexOfColName(doltdb.ProceduresTableSqlModeCol)
 
 	defer func(iter sql.RowIter, context *sql.Context) {
 		err := iter.Close(context)
@@ -285,7 +286,19 @@ func dumpProcedures(sqlCtx *sql.Context, engine *engine.SqlEngine, root *doltdb.
 			return err
 		}
 
-		err = iohelp.WriteLine(writer, fmt.Sprintf("delimiter END_PROCEDURE"))
+		sqlMode := ""
+		if len(row) >= sqlModeIdx {
+			if s, ok := row[sqlModeIdx].(string); ok {
+				sqlMode = s
+			}
+		}
+
+		modeChanged, err := changeSqlMode(sqlCtx, writer, sqlMode)
+		if err != nil {
+			return err
+		}
+
+		err = iohelp.WriteLine(writer, "delimiter END_PROCEDURE")
 		if err != nil {
 			return err
 		}
@@ -295,10 +308,16 @@ func dumpProcedures(sqlCtx *sql.Context, engine *engine.SqlEngine, root *doltdb.
 			return err
 		}
 
-		err = iohelp.WriteLine(writer, fmt.Sprintf("END_PROCEDURE\ndelimiter ;"))
+		err = iohelp.WriteLine(writer, "END_PROCEDURE\ndelimiter ;")
 		if err != nil {
 			return err
 		}
+
+		if modeChanged {
+			if err := resetSqlMode(writer); err != nil {
+				return err
+			}
+		}
 	}
 
 	return nil
@@ -321,6 +340,7 @@ func dumpTriggers(sqlCtx *sql.Context, engine *engine.SqlEngine, root *doltdb.Ro
 
 	typeColIdx := sch.IndexOfColName(doltdb.SchemasTablesTypeCol)
 	fragColIdx := sch.IndexOfColName(doltdb.SchemasTablesFragmentCol)
+	sqlModeIdx := sch.IndexOfColName(doltdb.SchemasTablesSqlModeCol)
 
 	defer func(iter sql.RowIter, context *sql.Context) {
 		err := iter.Close(context)
@@ -341,10 +361,26 @@ func dumpTriggers(sqlCtx *sql.Context, engine *engine.SqlEngine, root *doltdb.Ro
 			continue
 		}
 
+		sqlMode := ""
+		if s, ok := row[sqlModeIdx].(string); ok {
+			sqlMode = s
+		}
+
+		modeChanged, err := changeSqlMode(sqlCtx, writer, sqlMode)
+		if err != nil {
+			return err
+		}
+
 		err = iohelp.WriteLine(writer, fmt.Sprintf("%s;", row[fragColIdx]))
 		if err != nil {
 			return err
 		}
+
+		if modeChanged {
+			if err := resetSqlMode(writer); err != nil {
+				return err
+			}
+		}
 	}
 
 	return nil
@@ -368,6 +404,7 @@ func dumpViews(ctx *sql.Context, engine *engine.SqlEngine, root *doltdb.RootValu
 	typeColIdx := sch.IndexOfColName(doltdb.SchemasTablesTypeCol)
 	fragColIdx := sch.IndexOfColName(doltdb.SchemasTablesFragmentCol)
 	nameColIdx := sch.IndexOfColName(doltdb.SchemasTablesNameCol)
+	sqlModeIdx := sch.IndexOfColName(doltdb.SchemasTablesSqlModeCol)
 
 	defer func(iter sql.RowIter, context *sql.Context) {
 		err := iter.Close(context)
@@ -388,8 +425,18 @@ func dumpViews(ctx *sql.Context, engine *engine.SqlEngine, root *doltdb.RootValu
 			continue
 		}
 
+		sqlMode := ""
+		if s, ok := row[sqlModeIdx].(string); ok {
+			sqlMode = s
+		}
+
+		opts := sql.NewSqlModeFromString(sqlMode).ParserOptions()
 		// We used to store just the SELECT part of a view, but now we store the entire CREATE VIEW statement
-		cv, err := parse.Parse(ctx, row[fragColIdx].(string))
+		cv, err := planbuilder.ParseWithOptions(ctx, engine.GetUnderlyingEngine().Analyzer.Catalog, row[fragColIdx].(string), opts)
+		if err != nil {
+			return err
+		}
+		modeChanged, err := changeSqlMode(ctx, writer, sqlMode)
 		if err != nil {
 			return err
 		}
@@ -406,11 +453,60 @@ func dumpViews(ctx *sql.Context, engine *engine.SqlEngine, root *doltdb.RootValu
 				return err
 			}
 		}
+
+		if modeChanged {
+			if err := resetSqlMode(writer); err != nil {
+				return err
+			}
+		}
 	}
 
 	return nil
 }
 
+// changeSqlMode checks if the current SQL session's @@SQL_MODE is different from the requested |newSqlMode| and if so,
+// outputs a SQL statement to |writer| to save the current @@SQL_MODE to the @previousSqlMode variable and then outputs
+// a SQL statement to set the @@SQL_MODE to |sqlMode|. If |newSqlMode| is the identical to the current session's
+// SQL_MODE (the default, global @@SQL_MODE), then no statements are written to |writer|. The boolean return code
+// indicates if any statements were written.
+func changeSqlMode(ctx *sql.Context, writer io.WriteCloser, newSqlMode string) (bool, error) {
+	if newSqlMode == "" {
+		return false, nil
+	}
+
+	variable, err := ctx.Session.GetSessionVariable(ctx, "SQL_MODE")
+	if err != nil {
+		return false, err
+	}
+	currentSqlMode, ok := variable.(string)
+	if !ok {
+		return false, fmt.Errorf("unable to read @@SQL_MODE system variable from value '%v'", currentSqlMode)
+	}
+
+	if currentSqlMode == newSqlMode {
+		return false, nil
+	}
+
+	err = iohelp.WriteLine(writer, "SET @previousSqlMode=@@SQL_MODE;")
+	if err != nil {
+		return false, err
+	}
+
+	err = iohelp.WriteLine(writer, fmt.Sprintf("SET @@SQL_MODE='%s';", newSqlMode))
+	if err != nil {
+		return false, err
+	}
+
+	return true, nil
+}
+
+// resetSqlMode outputs a SQL statement to |writer| to reset @@SQL_MODE back to the previous value stored
+// by the last call to changeSqlMode. This function should only be called after changeSqlMode, otherwise the
+// @previousSqlMode variable will not be set correctly.
+func resetSqlMode(writer io.WriteCloser) error {
+	return iohelp.WriteLine(writer, "SET @@SQL_MODE=@previousSqlMode;")
+}
+
 type dumpOptions struct {
 	format     string
 	schemaOnly bool
@@ -575,9 +671,9 @@ func getDumpDestination(path string) mvdata.DataLocation {
 	return destLoc
 }
 
-// validateArgs returns either filename of directory name after checking each cases of user input arguments,
+// validateDumpArgs returns either filename of directory name after checking each cases of user input arguments,
 // handling errors for invalid arguments
-func validateArgs(apr *argparser.ArgParseResults) (string, errhand.VerboseError) {
+func validateDumpArgs(apr *argparser.ArgParseResults) (string, errhand.VerboseError) {
 	rf, _ := apr.GetValue(FormatFlag)
 	rf = strings.TrimPrefix(rf, ".")
 	fn, fnOk := apr.GetValue(filenameFlag)
@@ -637,7 +733,7 @@ func newTableArgs(tblName string, destination mvdata.DataLocation, batched, auto
 func dumpNonSqlTables(ctx context.Context, root *doltdb.RootValue, dEnv *env.DoltEnv, force bool, tblNames []string, rf string, dirName string, batched bool) errhand.VerboseError {
 	var fName string
 	if dirName == emptyStr {
-		dirName = fmt.Sprintf("doltdump/")
+		dirName = "doltdump/"
 	} else {
 		if !strings.HasSuffix(dirName, "/") {
 			dirName = fmt.Sprintf("%s/", dirName)
