@@ -18,18 +18,22 @@ import (
 	"context"
 	"os"
 	"runtime"
+	"strconv"
 	"strings"
 
 	gms "github.com/dolthub/go-mysql-server"
+	"github.com/dolthub/go-mysql-server/eventscheduler"
 	"github.com/dolthub/go-mysql-server/sql"
 	"github.com/dolthub/go-mysql-server/sql/analyzer"
 	"github.com/dolthub/go-mysql-server/sql/binlogreplication"
 	"github.com/dolthub/go-mysql-server/sql/mysql_db"
 	"github.com/dolthub/go-mysql-server/sql/rowexec"
 	_ "github.com/dolthub/go-mysql-server/sql/variables"
+	"github.com/sirupsen/logrus"
 
 	"github.com/dolthub/dolt/go/cmd/dolt/cli"
 	"github.com/dolthub/dolt/go/libraries/doltcore/branch_control"
+	"github.com/dolthub/dolt/go/libraries/doltcore/dconfig"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
 	dsqle "github.com/dolthub/dolt/go/libraries/doltcore/sqle"
 	dblr "github.com/dolthub/dolt/go/libraries/doltcore/sqle/binlogreplication"
@@ -51,6 +55,8 @@ type SqlEngine struct {
 type sessionFactory func(mysqlSess *sql.BaseSession, pro sql.DatabaseProvider) (*dsess.DoltSession, error)
 type contextFactory func(ctx context.Context, session sql.Session) (*sql.Context, error)
 
+type SystemVariables map[string]interface{}
+
 type SqlEngineConfig struct {
 	IsReadOnly              bool
 	IsServerLocked          bool
@@ -64,8 +70,10 @@ type SqlEngineConfig struct {
 	DoltTransactionCommit   bool
 	Bulk                    bool
 	JwksConfig              []JwksConfig
+	SystemVariables         SystemVariables
 	ClusterController       *cluster.Controller
 	BinlogReplicaController binlogreplication.BinlogReplicaController
+	EventSchedulerStatus    eventscheduler.SchedulerStatus
 }
 
 // NewSqlEngine returns a SqlEngine
@@ -105,6 +113,11 @@ func NewSqlEngine(
 		return nil, err
 	}
 
+	err = applySystemVariables(sql.SystemVariables, config.SystemVariables)
+	if err != nil {
+		return nil, err
+	}
+
 	all := append(dbs)
 
 	clusterDB := config.ClusterController.ClusterDatabase()
@@ -122,11 +135,35 @@ func NewSqlEngine(
 
 	config.ClusterController.RegisterStoredProcedures(pro)
 	pro.InitDatabaseHook = cluster.NewInitDatabaseHook(config.ClusterController, bThreads, pro.InitDatabaseHook)
-	config.ClusterController.ManageDatabaseProvider(pro)
+
+	sqlEngine := &SqlEngine{}
+
+	pro.DropDatabaseHook = config.ClusterController.DropDatabaseHook()
+	config.ClusterController.SetDropDatabase(pro.DropDatabase)
+
+	// Create the engine
+	engine := gms.New(analyzer.NewBuilder(pro).WithParallelism(parallelism).Build(), &gms.Config{
+		IsReadOnly:     config.IsReadOnly,
+		IsServerLocked: config.IsServerLocked,
+	}).WithBackgroundThreads(bThreads)
+
+	config.ClusterController.SetIsStandbyCallback(func(isStandby bool) {
+		pro.SetIsStandby(isStandby)
+
+		// Standbys are read only, primarys are not.
+		// We only change this here if the server was not forced read
+		// only by its startup config.
+		if !config.IsReadOnly {
+			engine.ReadOnly.Store(isStandby)
+		}
+	})
 
 	// Load in privileges from file, if it exists
-	persister := mysql_file_handler.NewPersister(config.PrivFilePath, config.DoltCfgDirPath)
-	data, err := persister.LoadData()
+	var persister cluster.MySQLDbPersister
+	persister = mysql_file_handler.NewPersister(config.PrivFilePath, config.DoltCfgDirPath)
+
+	persister = config.ClusterController.HookMySQLDbPersister(persister, engine.Analyzer.Catalog.MySQLDb)
+	data, err := persister.LoadData(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -136,12 +173,9 @@ func NewSqlEngine(
 	if bcController, err = branch_control.LoadData(config.BranchCtrlFilePath, config.DoltCfgDirPath); err != nil {
 		return nil, err
 	}
+	config.ClusterController.HookBranchControlPersistence(bcController, mrEnv.FileSystem())
 
-	// Set up engine
-	engine := gms.New(analyzer.NewBuilder(pro).WithParallelism(parallelism).Build(), &gms.Config{
-		IsReadOnly:     config.IsReadOnly,
-		IsServerLocked: config.IsServerLocked,
-	}).WithBackgroundThreads(bThreads)
+	// Setup the engine.
 	engine.Analyzer.Catalog.MySQLDb.SetPersister(persister)
 
 	engine.Analyzer.Catalog.MySQLDb.SetPlugins(map[string]mysql_db.PlaintextAuthPlugin{
@@ -155,9 +189,9 @@ func NewSqlEngine(
 		return nil, err
 	}
 
-	if dbg, ok := os.LookupEnv("DOLT_SQL_DEBUG_LOG"); ok && strings.ToLower(dbg) == "true" {
+	if dbg, ok := os.LookupEnv(dconfig.EnvSqlDebugLog); ok && strings.ToLower(dbg) == "true" {
 		engine.Analyzer.Debug = true
-		if verbose, ok := os.LookupEnv("DOLT_SQL_DEBUG_LOG_VERBOSE"); ok && strings.ToLower(verbose) == "true" {
+		if verbose, ok := os.LookupEnv(dconfig.EnvSqlDebugLogVerbose); ok && strings.ToLower(verbose) == "true" {
 			engine.Analyzer.Verbose = true
 		}
 	}
@@ -172,10 +206,17 @@ func NewSqlEngine(
 		return nil, err
 	}
 
-	sessionFactory := doltSessionFactory(pro, mrEnv.Config(), bcController, config.Autocommit)
+	sessFactory := doltSessionFactory(pro, mrEnv.Config(), bcController, config.Autocommit)
+
+	if engine.EventScheduler == nil {
+		err = configureEventScheduler(config, engine, sessFactory, pro)
+		if err != nil {
+			return nil, err
+		}
+	}
 
 	if config.BinlogReplicaController != nil {
-		binLogSession, err := sessionFactory(sql.NewBaseSession(), pro)
+		binLogSession, err := sessFactory(sql.NewBaseSession(), pro)
 		if err != nil {
 			return nil, err
 		}
@@ -186,12 +227,12 @@ func NewSqlEngine(
 		}
 	}
 
-	return &SqlEngine{
-		provider:       pro,
-		contextFactory: sqlContextFactory(),
-		dsessFactory:   sessionFactory,
-		engine:         engine,
-	}, nil
+	sqlEngine.provider = pro
+	sqlEngine.contextFactory = sqlContextFactory()
+	sqlEngine.dsessFactory = sessFactory
+	sqlEngine.engine = engine
+
+	return sqlEngine, nil
 }
 
 // NewRebasedSqlEngine returns a smalled rebased engine primarily used in filterbranch.
@@ -202,6 +243,13 @@ func NewRebasedSqlEngine(engine *gms.Engine, dbs map[string]dsess.SqlDatabase) *
 	}
 }
 
+func applySystemVariables(vars sql.SystemVariableRegistry, cfg SystemVariables) error {
+	if cfg != nil {
+		return vars.AssignValues(cfg)
+	}
+	return nil
+}
+
 // Databases returns a slice of all databases in the engine
 func (se *SqlEngine) Databases(ctx *sql.Context) []dsess.SqlDatabase {
 	databases := se.provider.AllDatabases(ctx)
@@ -266,18 +314,70 @@ func (se *SqlEngine) Close() error {
 
 // configureBinlogReplicaController configures the binlog replication controller with the |engine|.
 func configureBinlogReplicaController(config *SqlEngineConfig, engine *gms.Engine, session *dsess.DoltSession) error {
-	contextFactory := sqlContextFactory()
-
-	executionCtx, err := contextFactory(context.Background(), session)
+	ctxFactory := sqlContextFactory()
+	executionCtx, err := ctxFactory(context.Background(), session)
 	if err != nil {
 		return err
 	}
 	dblr.DoltBinlogReplicaController.SetExecutionContext(executionCtx)
-	engine.Analyzer.BinlogReplicaController = config.BinlogReplicaController
+	engine.Analyzer.Catalog.BinlogReplicaController = config.BinlogReplicaController
 
 	return nil
 }
 
+// configureEventScheduler configures the event scheduler with the |engine| for executing events, a |sessFactory|
+// for creating sessions, and a DoltDatabaseProvider, |pro|.
+func configureEventScheduler(config *SqlEngineConfig, engine *gms.Engine, sessFactory sessionFactory, pro *dsqle.DoltDatabaseProvider) error {
+	// need to give correct user, use the definer as user to run the event definition queries
+	ctxFactory := sqlContextFactory()
+
+	// getCtxFunc is used to create new session context for event scheduler.
+	// It starts a transaction that needs to be committed using the function returned.
+	getCtxFunc := func() (*sql.Context, func() error, error) {
+		sess, err := sessFactory(sql.NewBaseSession(), pro)
+		if err != nil {
+			return nil, func() error { return nil }, err
+		}
+
+		newCtx, err := ctxFactory(context.Background(), sess)
+		if err != nil {
+			return nil, func() error { return nil }, err
+		}
+
+		ts, ok := newCtx.Session.(sql.TransactionSession)
+		if !ok {
+			return nil, func() error { return nil }, nil
+		}
+
+		tr, err := sess.StartTransaction(newCtx, sql.ReadWrite)
+		if err != nil {
+			return nil, func() error { return nil }, err
+		}
+
+		ts.SetTransaction(tr)
+
+		return newCtx, func() error {
+			return ts.CommitTransaction(newCtx, tr)
+		}, nil
+	}
+
+	// A hidden env var allows overriding the event scheduler period for testing. This option is not
+	// exposed via configuration because we do not want to encourage customers to use it. If the value
+	// is equal to or less than 0, then the period is ignored and the default period, 30s, is used.
+	eventSchedulerPeriod := 0
+	eventSchedulerPeriodEnvVar := "DOLT_EVENT_SCHEDULER_PERIOD"
+	if s, ok := os.LookupEnv(eventSchedulerPeriodEnvVar); ok {
+		i, err := strconv.Atoi(s)
+		if err != nil {
+			logrus.Warnf("unable to parse value '%s' from env var '%s' as an integer", s, eventSchedulerPeriodEnvVar)
+		} else {
+			logrus.Warnf("overriding Dolt event scheduler period to %d seconds", i)
+			eventSchedulerPeriod = i
+		}
+	}
+	return engine.InitializeEventScheduler(getCtxFunc, config.EventSchedulerStatus, eventSchedulerPeriod)
+}
+
 // sqlContextFactory returns a contextFactory that creates a new sql.Context with the initial database provided
 func sqlContextFactory() contextFactory {
 	return func(ctx context.Context, session sql.Session) (*sql.Context, error) {
@@ -287,7 +387,7 @@ func sqlContextFactory() contextFactory {
 }
 
 // doltSessionFactory returns a sessionFactory that creates a new DoltSession
-func doltSessionFactory(pro dsqle.DoltDatabaseProvider, config config.ReadWriteConfig, bc *branch_control.Controller, autocommit bool) sessionFactory {
+func doltSessionFactory(pro *dsqle.DoltDatabaseProvider, config config.ReadWriteConfig, bc *branch_control.Controller, autocommit bool) sessionFactory {
 	return func(mysqlSess *sql.BaseSession, provider sql.DatabaseProvider) (*dsess.DoltSession, error) {
 		doltSession, err := dsess.NewDoltSession(mysqlSess, pro, config, bc)
 		if err != nil {
