@@ -227,11 +227,11 @@ func processFilterQuery(ctx context.Context, dEnv *env.DoltEnv, cm *doltdb.Commi
 
 	case *sqlparser.Delete:
 		_, itr, err = eng.Query(sqlCtx, query)
-	case *sqlparser.MultiAlterDDL:
+	case *sqlparser.AlterTable:
 		_, itr, err = eng.Query(sqlCtx, query)
 	case *sqlparser.DDL:
 		_, itr, err = eng.Query(sqlCtx, query)
-	case *sqlparser.Select, *sqlparser.OtherRead, *sqlparser.Show, *sqlparser.Explain, *sqlparser.Union:
+	case *sqlparser.Select, *sqlparser.OtherRead, *sqlparser.Show, *sqlparser.Explain, *sqlparser.SetOp:
 		return nil, fmt.Errorf("filter-branch queries must be write queries: '%s'", query)
 
 	default:
