@@ -53,16 +53,14 @@ var sqlDocs = cli.CommandDocumentationContent{
 
 Multiple SQL statements must be separated by semicolons. Use {{.EmphasisLeft}}-b{{.EmphasisRight}} to enable batch mode to speed up large batches of INSERT / UPDATE statements. Pipe SQL files to dolt sql (no {{.EmphasisLeft}}-q{{.EmphasisRight}}) to execute a SQL import or update script. 
 
-Queries can be saved to the query catalog with {{.EmphasisLeft}}-s{{.EmphasisRight}}. Alternatively {{.EmphasisLeft}}-x{{.EmphasisRight}} can be used to execute a saved query by name.
+By default this command uses the dolt database in the current working directory. If you would prefer to use a different directory, user the {{.EmphasisLeft}}--data-dir <directory>{{.EmphasisRight}} argument before the sql subcommand.
 
-By default this command uses the dolt database in the current working directory, as well as any dolt databases that are found in the current directory. Any databases created with CREATE DATABASE are placed in the current directory as well. Running with {{.EmphasisLeft}}--data-dir <directory>{{.EmphasisRight}} uses each of the subdirectories of the supplied directory (each subdirectory must be a valid dolt data repository) as databases. Subdirectories starting with '.' are ignored.`,
+If a server is running for the database in question, then the query will go through the server automatically. If connecting to a remote server is preferred, used the {{.EmphasisLeft}}--host <host>{{.EmphasisRight}} and {{.EmphasisLeft}}--port <port>{{.EmphasisRight}} global arguments. See 'dolt --help' for more information about global arguments.`,
 
 	Synopsis: []string{
 		"",
 		"< script.sql",
-		"[--data-dir {{.LessThan}}directory{{.GreaterThan}}] [-r {{.LessThan}}result format{{.GreaterThan}}]",
 		"-q {{.LessThan}}query{{.GreaterThan}} [-r {{.LessThan}}result format{{.GreaterThan}}] [-s {{.LessThan}}name{{.GreaterThan}} -m {{.LessThan}}message{{.GreaterThan}}] [-b]",
-		"-q {{.LessThan}}query{{.GreaterThan}} --data-dir {{.LessThan}}directory{{.GreaterThan}} [-r {{.LessThan}}result format{{.GreaterThan}}] [-b]",
 		"-x {{.LessThan}}name{{.GreaterThan}}",
 		"--list-saved",
 	},
@@ -92,6 +90,7 @@ const (
 	DefaultUser           = "root"
 	DefaultHost           = "localhost"
 	UseDbFlag             = "use-db"
+	ProfileFlag           = "profile"
 
 	welcomeMsg = `# Welcome to the DoltSQL shell.
 # Statements must be terminated with ';'.
@@ -195,6 +194,18 @@ func (cmd SqlCmd) Exec(ctx context.Context, commandStr string, args []string, dE
 		}
 	}
 
+	// restrict LOAD FILE invocations to current directory
+	wd, err := os.Getwd()
+	if err != nil {
+		wd = "/dev/null"
+	}
+	err = sql.SystemVariables.AssignValues(map[string]interface{}{
+		"secure_file_priv": wd,
+	})
+	if err != nil {
+		return HandleVErrAndExitCode(errhand.VerboseErrorFromError(err), usage)
+	}
+
 	queryist, sqlCtx, closeFunc, err := cliCtx.QueryEngine(ctx)
 	if err != nil {
 		return HandleVErrAndExitCode(errhand.VerboseErrorFromError(err), usage)
@@ -266,16 +277,6 @@ func (cmd SqlCmd) Exec(ctx context.Context, commandStr string, args []string, dE
 func sqlHandleVErrAndExitCode(queryist cli.Queryist, verr errhand.VerboseError, usage cli.UsagePrinter) int {
 	if verr != nil {
 		if msg := verr.Verbose(); strings.TrimSpace(msg) != "" {
-			if _, ok := queryist.(*engine.SqlEngine); !ok {
-				// We are in a context where we are attempting to connect to a remote database. These errors
-				// are unstructured, so we add some additional context around them.
-				tmpMsg := `You've encountered a new behavior in dolt which is not fully documented yet.
-A local dolt server is using your dolt data directory, and in an attempt to service your request, we are attempting to 
-connect to it. That has failed. You should stop the server, or reach out to @macneale on https://discord.gg/gqr7K4VNKe
-for help.`
-				cli.PrintErrln(tmpMsg)
-				msg = fmt.Sprintf("A local server is running, and dolt is failing to connect. Error connecting to remote database: \"%s\".\n", msg)
-			}
 			cli.PrintErrln(msg)
 		}
 
@@ -617,7 +618,10 @@ func execBatchMode(ctx *sql.Context, qryist cli.Queryist, input io.Reader, conti
 		if len(query) == 0 || query == "\n" {
 			continue
 		}
-		sqlStatement, err := sqlparser.Parse(query)
+
+		sqlMode := sql.LoadSqlMode(ctx)
+
+		sqlStatement, err := sqlparser.ParseWithOptions(query, sqlMode.ParserOptions())
 		if err == sqlparser.ErrEmpty {
 			continue
 		} else if err != nil {
@@ -644,7 +648,7 @@ func execBatchMode(ctx *sql.Context, qryist cli.Queryist, input io.Reader, conti
 		if rowIter != nil {
 			switch sqlStatement.(type) {
 			case *sqlparser.Select, *sqlparser.Insert, *sqlparser.Update, *sqlparser.Delete,
-				*sqlparser.OtherRead, *sqlparser.Show, *sqlparser.Explain, *sqlparser.Union:
+				*sqlparser.OtherRead, *sqlparser.Show, *sqlparser.Explain, *sqlparser.SetOp:
 				// For any statement that prints out result, print a newline to put the regular output on its own line
 				if fileReadProg != nil {
 					fileReadProg.printNewLineIfNeeded()
@@ -785,7 +789,12 @@ func execShell(sqlCtx *sql.Context, qryist cli.Queryist, format engine.PrintResu
 				}
 			}
 
+			db, ok := getDBFromSession(sqlCtx, qryist)
+			if ok {
+				sqlCtx.SetCurrentDatabase(db)
+			}
 			nextPrompt = fmt.Sprintf("%s> ", sqlCtx.GetCurrentDatabase())
+
 			return true
 		}()
 
@@ -803,6 +812,32 @@ func execShell(sqlCtx *sql.Context, qryist cli.Queryist, format engine.PrintResu
 	return nil
 }
 
+// getDBFromSession returns the current database name for the session, handling all the errors along the way by printing
+// red error messages to the CLI. If there was an issue getting the db name, the second return value is false.
+func getDBFromSession(sqlCtx *sql.Context, qryist cli.Queryist) (db string, ok bool) {
+	_, resp, err := qryist.Query(sqlCtx, "select database()")
+	if err != nil {
+		cli.Println(color.RedString("Failure to get DB Name for session" + err.Error()))
+		return db, false
+	}
+	// Expect single row/single column result with the db name.
+	row, err := resp.Next(sqlCtx)
+	if err != nil {
+		cli.Println(color.RedString("Failure to get DB Name for session" + err.Error()))
+		return db, false
+	}
+	if len(row) != 1 {
+		cli.Println(color.RedString("Failure to get DB Name for session" + err.Error()))
+		return db, false
+	}
+	if row[0] == nil {
+		db = ""
+	} else {
+		db = row[0].(string)
+	}
+	return db, true
+}
+
 // Returns a new auto completer with table names, column names, and SQL keywords.
 // TODO: update the completer on DDL, branch change, etc.
 func newCompleter(
@@ -945,7 +980,7 @@ func processParsedQuery(ctx *sql.Context, query string, qryist cli.Queryist, sql
 		}
 		cli.Println("Database changed")
 		return sch, nil, err
-	case *sqlparser.MultiAlterDDL, *sqlparser.Set, *sqlparser.Commit:
+	case *sqlparser.AlterTable, *sqlparser.Set, *sqlparser.Commit:
 		_, ri, err := qryist.Query(ctx, query)
 		if err != nil {
 			return nil, nil, err
