@@ -23,10 +23,14 @@ import (
 	"net/http"
 	"runtime"
 	"strconv"
+	"strings"
 	"time"
 
+	"github.com/dolthub/go-mysql-server/eventscheduler"
 	"github.com/dolthub/go-mysql-server/server"
 	"github.com/dolthub/go-mysql-server/sql"
+	"github.com/dolthub/go-mysql-server/sql/mysql_db"
+	"github.com/dolthub/go-mysql-server/sql/plan"
 	"github.com/dolthub/go-mysql-server/sql/types"
 	"github.com/dolthub/vitess/go/mysql"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
@@ -34,6 +38,7 @@ import (
 	goerrors "gopkg.in/src-d/go-errors.v1"
 
 	"github.com/dolthub/dolt/go/cmd/dolt/cli"
+	"github.com/dolthub/dolt/go/cmd/dolt/commands"
 	"github.com/dolthub/dolt/go/cmd/dolt/commands/engine"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
 	"github.com/dolthub/dolt/go/libraries/doltcore/remotesrv"
@@ -45,6 +50,14 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqlserver"
 )
 
+const (
+	LocalConnectionUser = "__dolt_local_user__"
+)
+
+// ExternalDisableUsers is called by implementing applications to disable users. This is not used by Dolt itself,
+// but will break compatibility with implementing applications that do not yet support users.
+var ExternalDisableUsers bool = false
+
 // Serve starts a MySQL-compatible server. Returns any errors that were encountered.
 func Serve(
 	ctx context.Context,
@@ -105,12 +118,14 @@ func Serve(
 				logrus.TraceLevel.String(),
 			),
 			Default: logrus.GetLevel().String(),
-			NotifyChanged: func(scope sql.SystemVariableScope, v sql.SystemVarValue) {
-				if level, err := logrus.ParseLevel(v.Val.(string)); err == nil {
-					logrus.SetLevel(level)
-				} else {
-					logrus.Warnf("could not parse requested log level %s as a log level. dolt_log_level variable value and logging behavior will diverge.", v.Val.(string))
+			NotifyChanged: func(scope sql.SystemVariableScope, v sql.SystemVarValue) error {
+				level, err := logrus.ParseLevel(v.Val.(string))
+				if err != nil {
+					return fmt.Errorf("could not parse requested log level %s as a log level. dolt_log_level variable value and logging behavior will diverge.", v.Val.(string))
 				}
+
+				logrus.SetLevel(level)
+				return nil
 			},
 		},
 	})
@@ -124,8 +139,15 @@ func Serve(
 		if err != nil {
 			return err, nil
 		}
+		dEnv.FS = fs
 	}
 
+	serverLock, startError := acquireGlobalSqlServerLock(serverConfig.Port(), dEnv)
+	if startError != nil {
+		return
+	}
+	defer dEnv.FS.Delete(dEnv.LockFile(), false)
+
 	mrEnv, err = env.MultiEnvForDirectory(ctx, dEnv.Config.WriteableConfig(), fs, dEnv.Version, dEnv.IgnoreLockFile, dEnv)
 	if err != nil {
 		return err, nil
@@ -155,9 +177,16 @@ func Serve(
 		Autocommit:              serverConfig.AutoCommit(),
 		DoltTransactionCommit:   serverConfig.DoltTransactionCommit(),
 		JwksConfig:              serverConfig.JwksConfig(),
+		SystemVariables:         serverConfig.SystemVars(),
 		ClusterController:       clusterController,
 		BinlogReplicaController: binlogreplication.DoltBinlogReplicaController,
 	}
+	esStatus, err := getEventSchedulerStatus(serverConfig.EventSchedulerStatus())
+	if err != nil {
+		return err, nil
+	}
+	config.EventSchedulerStatus = esStatus
+
 	sqlEngine, err := engine.NewSqlEngine(
 		ctx,
 		mrEnv,
@@ -170,15 +199,21 @@ func Serve(
 
 	// Add superuser if specified user exists; add root superuser if no user specified and no existing privileges
 	userSpecified := config.ServerUser != ""
-	privsExist := sqlEngine.GetUnderlyingEngine().Analyzer.Catalog.MySQLDb.UserTable().Data().Count() != 0
+
+	mysqlDb := sqlEngine.GetUnderlyingEngine().Analyzer.Catalog.MySQLDb
+	ed := mysqlDb.Editor()
+	var numUsers int
+	ed.VisitUsers(func(*mysql_db.User) { numUsers += 1 })
+	privsExist := numUsers != 0
 	if userSpecified {
-		superuser := sqlEngine.GetUnderlyingEngine().Analyzer.Catalog.MySQLDb.GetUser(config.ServerUser, "%", false)
+		superuser := mysqlDb.GetUser(ed, config.ServerUser, "%", false)
 		if userSpecified && superuser == nil {
-			sqlEngine.GetUnderlyingEngine().Analyzer.Catalog.MySQLDb.AddSuperUser(config.ServerUser, "%", config.ServerPass)
+			mysqlDb.AddSuperUser(ed, config.ServerUser, "%", config.ServerPass)
 		}
 	} else if !privsExist {
-		sqlEngine.GetUnderlyingEngine().Analyzer.Catalog.MySQLDb.AddSuperUser(defaultUser, "%", defaultPass)
+		mysqlDb.AddSuperUser(ed, defaultUser, "%", defaultPass)
 	}
+	ed.Close()
 
 	labels := serverConfig.MetricsLabels()
 
@@ -216,8 +251,14 @@ func Serve(
 		return
 	}
 
-	lck := env.NewDBLock(serverConfig.Port())
-	sqlserver.SetRunningServer(mySQLServer, &lck)
+	sqlserver.SetRunningServer(mySQLServer, serverLock)
+
+	ed = mysqlDb.Editor()
+	mysqlDb.AddSuperUser(ed, LocalConnectionUser, "localhost", serverLock.Secret)
+	ed.Close()
+	if ExternalDisableUsers {
+		mysqlDb.SetEnabled(false)
+	}
 
 	var metSrv *http.Server
 	if serverConfig.MetricsHost() != "" && serverConfig.MetricsPort() > 0 {
@@ -237,80 +278,82 @@ func Serve(
 	var remoteSrv *remotesrv.Server
 	if serverConfig.RemotesapiPort() != nil {
 		port := *serverConfig.RemotesapiPort()
-		if remoteSrvSqlCtx, err := sqlEngine.NewDefaultContext(ctx); err == nil {
-			listenaddr := fmt.Sprintf(":%d", port)
-			args := sqle.RemoteSrvServerArgs(remoteSrvSqlCtx, remotesrv.ServerArgs{
-				Logger:         logrus.NewEntry(lgr),
-				ReadOnly:       true,
-				HttpListenAddr: listenaddr,
-				GrpcListenAddr: listenaddr,
-			})
-			args = sqle.WithUserPasswordAuth(args, remotesrv.UserAuth{User: serverConfig.User(), Password: serverConfig.Password()})
-			args.TLSConfig = serverConf.TLSConfig
-			remoteSrv, err = remotesrv.NewServer(args)
-			if err != nil {
-				lgr.Errorf("error creating remotesapi server on port %d: %v", port, err)
-				startError = err
-				return
-			}
-			listeners, err := remoteSrv.Listeners()
-			if err != nil {
-				lgr.Errorf("error starting remotesapi server listeners on port %d: %v", port, err)
-				startError = err
-				return
-			} else {
-				go remoteSrv.Serve(listeners)
-			}
-		} else {
+		listenaddr := fmt.Sprintf(":%d", port)
+		args, err := sqle.RemoteSrvServerArgs(sqlEngine.NewDefaultContext, remotesrv.ServerArgs{
+			Logger:         logrus.NewEntry(lgr),
+			ReadOnly:       true,
+			HttpListenAddr: listenaddr,
+			GrpcListenAddr: listenaddr,
+		})
+		if err != nil {
 			lgr.Errorf("error creating SQL engine context for remotesapi server: %v", err)
 			startError = err
 			return
 		}
+
+		ctxFactory := func() (*sql.Context, error) { return sqlEngine.NewDefaultContext(ctx) }
+		authenticator := newAuthenticator(ctxFactory, sqlEngine.GetUnderlyingEngine().Analyzer.Catalog.MySQLDb)
+		args = sqle.WithUserPasswordAuth(args, authenticator)
+
+		args.TLSConfig = serverConf.TLSConfig
+		remoteSrv, err = remotesrv.NewServer(args)
+		if err != nil {
+			lgr.Errorf("error creating remotesapi server on port %d: %v", port, err)
+			startError = err
+			return
+		}
+		listeners, err := remoteSrv.Listeners()
+		if err != nil {
+			lgr.Errorf("error starting remotesapi server listeners on port %d: %v", port, err)
+			startError = err
+			return
+		} else {
+			go remoteSrv.Serve(listeners)
+		}
 	}
 
 	var clusterRemoteSrv *remotesrv.Server
 	if clusterController != nil {
-		if remoteSrvSqlCtx, err := sqlEngine.NewDefaultContext(ctx); err == nil {
-			args := clusterController.RemoteSrvServerArgs(remoteSrvSqlCtx, remotesrv.ServerArgs{
-				Logger: logrus.NewEntry(lgr),
-			})
-
-			clusterRemoteSrvTLSConfig, err := LoadClusterTLSConfig(serverConfig.ClusterConfig())
-			if err != nil {
-				lgr.Errorf("error starting remotesapi server for cluster config, could not load tls config: %v", err)
-				startError = err
-				return
-			}
-			args.TLSConfig = clusterRemoteSrvTLSConfig
-
-			clusterRemoteSrv, err = remotesrv.NewServer(args)
-			if err != nil {
-				lgr.Errorf("error creating remotesapi server on port %d: %v", *serverConfig.RemotesapiPort(), err)
-				startError = err
-				return
-			}
-
-			listeners, err := clusterRemoteSrv.Listeners()
-			if err != nil {
-				lgr.Errorf("error starting remotesapi server listeners for cluster config on %s: %v", clusterController.RemoteSrvListenAddr(), err)
-				startError = err
-				return
-			}
-
-			go clusterRemoteSrv.Serve(listeners)
-			go clusterController.Run()
-
-			clusterController.ManageQueryConnections(
-				mySQLServer.SessionManager().Iter,
-				sqlEngine.GetUnderlyingEngine().ProcessList.Kill,
-				mySQLServer.SessionManager().KillConnection,
-			)
-		} else {
+		args, err := clusterController.RemoteSrvServerArgs(sqlEngine.NewDefaultContext, remotesrv.ServerArgs{
+			Logger: logrus.NewEntry(lgr),
+		})
+		if err != nil {
 			lgr.Errorf("error creating SQL engine context for remotesapi server: %v", err)
 			startError = err
 			return
 		}
 
+		clusterRemoteSrvTLSConfig, err := LoadClusterTLSConfig(serverConfig.ClusterConfig())
+		if err != nil {
+			lgr.Errorf("error starting remotesapi server for cluster config, could not load tls config: %v", err)
+			startError = err
+			return
+		}
+		args.TLSConfig = clusterRemoteSrvTLSConfig
+
+		clusterRemoteSrv, err = remotesrv.NewServer(args)
+		if err != nil {
+			lgr.Errorf("error creating remotesapi server on port %d: %v", *serverConfig.RemotesapiPort(), err)
+			startError = err
+			return
+		}
+		clusterController.RegisterGrpcServices(sqlEngine.NewDefaultContext, clusterRemoteSrv.GrpcServer())
+
+		listeners, err := clusterRemoteSrv.Listeners()
+		if err != nil {
+			lgr.Errorf("error starting remotesapi server listeners for cluster config on %s: %v", clusterController.RemoteSrvListenAddr(), err)
+			startError = err
+			return
+		}
+
+		go clusterRemoteSrv.Serve(listeners)
+		go clusterController.Run()
+
+		clusterController.ManageQueryConnections(
+			mySQLServer.SessionManager().Iter,
+			sqlEngine.GetUnderlyingEngine().ProcessList.Kill,
+			mySQLServer.SessionManager().KillConnection,
+		)
 	}
 
 	if ok, f := mrEnv.IsLocked(); ok {
@@ -318,7 +361,7 @@ func Serve(
 		return
 	}
 
-	if err = mrEnv.Lock(lck); err != nil {
+	if err = mrEnv.Lock(serverLock); err != nil {
 		startError = err
 		return
 	}
@@ -351,6 +394,53 @@ func Serve(
 	return
 }
 
+// acquireGlobalSqlServerLock attempts to acquire a global lock on the SQL server. If no error is returned, then the lock was acquired.
+func acquireGlobalSqlServerLock(port int, dEnv *env.DoltEnv) (*env.DBLock, error) {
+	locked, _, err := dEnv.GetLock()
+	if err != nil {
+		return nil, err
+	}
+	if locked {
+		lockPath := dEnv.LockFile()
+		err = fmt.Errorf("Database locked by another sql-server; Lock file: %s", lockPath)
+		return nil, err
+	}
+
+	lck := env.NewDBLock(port)
+	err = dEnv.Lock(&lck)
+	if err != nil {
+		err = fmt.Errorf("Server can not start. Failed to acquire lock: %s", err.Error())
+		return nil, err
+	}
+
+	return &lck, nil
+}
+
+type remotesapiAuth struct {
+	ctxFactory func() (*sql.Context, error)
+	rawDb      *mysql_db.MySQLDb
+}
+
+func newAuthenticator(ctxFactory func() (*sql.Context, error), rawDb *mysql_db.MySQLDb) remotesrv.Authenticator {
+	return &remotesapiAuth{ctxFactory, rawDb}
+}
+
+func (r *remotesapiAuth) Authenticate(creds *remotesrv.RequestCredentials) bool {
+	err := commands.ValidatePasswordWithAuthResponse(r.rawDb, creds.Username, creds.Password)
+	if err != nil {
+		return false
+	}
+
+	ctx, err := r.ctxFactory()
+	if err != nil {
+		return false
+	}
+	ctx.Session.SetClient(sql.Client{User: creds.Username, Address: creds.Address, Capabilities: 0})
+
+	privOp := sql.NewDynamicPrivilegedOperation(plan.DynamicPrivilege_CloneAdmin)
+	return r.rawDb.UserHasPrivileges(ctx, privOp)
+}
+
 func LoadClusterTLSConfig(cfg cluster.Config) (*tls.Config, error) {
 	rcfg := cfg.RemotesAPIConfig()
 	if rcfg.TLSKey() == "" && rcfg.TLSCert() == "" {
@@ -508,3 +598,16 @@ func checkForUnixSocket(config ServerConfig) (string, bool, error) {
 
 	return "", false, nil
 }
+
+func getEventSchedulerStatus(status string) (eventscheduler.SchedulerStatus, error) {
+	switch strings.ToLower(status) {
+	case "on", "1":
+		return eventscheduler.SchedulerOn, nil
+	case "off", "0":
+		return eventscheduler.SchedulerOff, nil
+	case "disabled":
+		return eventscheduler.SchedulerDisabled, nil
+	default:
+		return eventscheduler.SchedulerDisabled, fmt.Errorf("Error while setting value '%s' to 'event_scheduler'.", status)
+	}
+}
