@@ -150,6 +150,8 @@ type ServerConfig interface {
 	BranchControlFilePath() string
 	// UserVars is an array containing user specific session variables
 	UserVars() []UserSessionVars
+	// SystemVars is a map setting global SQL system variables. For example, `secure_file_priv`.
+	SystemVars() engine.SystemVariables
 	// JwksConfig is an array containing jwks config
 	JwksConfig() []engine.JwksConfig
 	// AllowCleartextPasswords is true if the server should accept cleartext passwords.
@@ -163,6 +165,8 @@ type ServerConfig interface {
 	RemotesapiPort() *int
 	// ClusterConfig is the configuration for clustering in this sql-server.
 	ClusterConfig() cluster.Config
+	// EventSchedulerStatus is the configuration for enabling or disabling the event scheduler in this server.
+	EventSchedulerStatus() string
 }
 
 type validatingServerConfig interface {
@@ -198,6 +202,7 @@ type commandLineServerConfig struct {
 	socket                  string
 	remotesapiPort          *int
 	goldenMysqlConn         string
+	eventSchedulerStatus    string
 }
 
 var _ ServerConfig = (*commandLineServerConfig)(nil)
@@ -341,6 +346,10 @@ func (cfg *commandLineServerConfig) UserVars() []UserSessionVars {
 	return nil
 }
 
+func (cfg *commandLineServerConfig) SystemVars() engine.SystemVariables {
+	return nil
+}
+
 func (cfg *commandLineServerConfig) JwksConfig() []engine.JwksConfig {
 	return nil
 }
@@ -475,6 +484,22 @@ func (cfg *commandLineServerConfig) withGoldenMysqlConnectionString(cs string) *
 	return cfg
 }
 
+func (cfg *commandLineServerConfig) EventSchedulerStatus() string {
+	switch cfg.eventSchedulerStatus {
+	case "", "1":
+		return "ON"
+	case "0":
+		return "OFF"
+	default:
+		return strings.ToUpper(cfg.eventSchedulerStatus)
+	}
+}
+
+func (cfg *commandLineServerConfig) withEventScheduler(es string) *commandLineServerConfig {
+	cfg.eventSchedulerStatus = es
+	return cfg
+}
+
 // DefaultServerConfig creates a `*ServerConfig` that has all of the options set to their default values.
 func DefaultServerConfig() *commandLineServerConfig {
 	return &commandLineServerConfig{
