@@ -79,6 +79,8 @@ type BehaviorYAMLConfig struct {
 	// DoltTransactionCommit enables the @@dolt_transaction_commit system variable, which
 	// automatically creates a Dolt commit when any SQL transaction is committed.
 	DoltTransactionCommit *bool `yaml:"dolt_transaction_commit"`
+
+	EventSchedulerStatus *string `yaml:"event_scheduler,omitempty" minver:"1.17.0"`
 }
 
 // UserYAMLConfig contains server configuration regarding the user account clients must use to connect
@@ -152,9 +154,11 @@ type YAMLConfig struct {
 	ClusterCfg        *ClusterYAMLConfig    `yaml:"cluster,omitempty"`
 	PrivilegeFile     *string               `yaml:"privilege_file,omitempty"`
 	BranchControlFile *string               `yaml:"branch_control_file,omitempty"`
-	Vars              []UserSessionVars     `yaml:"user_session_vars"`
-	Jwks              []engine.JwksConfig   `yaml:"jwks"`
-	GoldenMysqlConn   *string               `yaml:"golden_mysql_conn,omitempty"`
+	// TODO: Rename to UserVars_
+	Vars            []UserSessionVars       `yaml:"user_session_vars"`
+	SystemVars_     *engine.SystemVariables `yaml:"system_variables,omitempty" minver:"1.11.1"`
+	Jwks            []engine.JwksConfig     `yaml:"jwks"`
+	GoldenMysqlConn *string                 `yaml:"golden_mysql_conn,omitempty"`
 }
 
 var _ ServerConfig = YAMLConfig{}
@@ -181,6 +185,7 @@ func serverConfigAsYAMLConfig(cfg ServerConfig) YAMLConfig {
 			strPtr(cfg.PersistenceBehavior()),
 			boolPtr(cfg.DisableClientMultiStatements()),
 			boolPtr(cfg.DoltTransactionCommit()),
+			strPtr(cfg.EventSchedulerStatus()),
 		},
 		UserConfig: UserYAMLConfig{
 			Name:     strPtr(cfg.User()),
@@ -434,6 +439,14 @@ func (cfg YAMLConfig) UserVars() []UserSessionVars {
 	return nil
 }
 
+func (cfg YAMLConfig) SystemVars() engine.SystemVariables {
+	if cfg.SystemVars_ == nil {
+		return engine.SystemVariables{}
+	}
+
+	return *cfg.SystemVars_
+}
+
 // JwksConfig is JSON Web Key Set config, and used to validate a user authed with a jwt (JSON Web Token).
 func (cfg YAMLConfig) JwksConfig() []engine.JwksConfig {
 	if cfg.Jwks != nil {
@@ -551,6 +564,20 @@ func (cfg YAMLConfig) ClusterConfig() cluster.Config {
 	return cfg.ClusterCfg
 }
 
+func (cfg YAMLConfig) EventSchedulerStatus() string {
+	if cfg.BehaviorConfig.EventSchedulerStatus == nil {
+		return "ON"
+	}
+	switch *cfg.BehaviorConfig.EventSchedulerStatus {
+	case "1":
+		return "ON"
+	case "0":
+		return "OFF"
+	default:
+		return strings.ToUpper(*cfg.BehaviorConfig.EventSchedulerStatus)
+	}
+}
+
 type ClusterYAMLConfig struct {
 	StandbyRemotes_ []StandbyRemoteYAMLConfig   `yaml:"standby_remotes"`
 	BootstrapRole_  string                      `yaml:"bootstrap_role"`
