@@ -57,7 +57,7 @@ func newNamespace(accessTbl *Access) *Namespace {
 		Users:     nil,
 		Hosts:     nil,
 		Values:    nil,
-		RWMutex:   &sync.RWMutex{},
+		RWMutex:   accessTbl.RWMutex,
 	}
 }
 
@@ -134,9 +134,6 @@ func (tbl *Namespace) Access() *Access {
 
 // Serialize returns the offset for the Namespace table written to the given builder.
 func (tbl *Namespace) Serialize(b *flatbuffers.Builder) flatbuffers.UOffsetT {
-	tbl.RWMutex.RLock()
-	defer tbl.RWMutex.RUnlock()
-
 	// Serialize the binlog
 	binlog := tbl.binlog.Serialize(b)
 	// Initialize field offset slices
@@ -198,15 +195,17 @@ func (tbl *Namespace) Serialize(b *flatbuffers.Builder) flatbuffers.UOffsetT {
 	return serial.BranchControlNamespaceEnd(b)
 }
 
+func (tbl *Namespace) reinit() {
+	tbl.binlog = NewNamespaceBinlog(nil)
+	tbl.Databases = nil
+	tbl.Branches = nil
+	tbl.Users = nil
+	tbl.Hosts = nil
+	tbl.Values = nil
+}
+
 // Deserialize populates the table with the data from the flatbuffers representation.
 func (tbl *Namespace) Deserialize(fb *serial.BranchControlNamespace) error {
-	tbl.RWMutex.Lock()
-	defer tbl.RWMutex.Unlock()
-
-	// Verify that the table is empty
-	if len(tbl.Values) != 0 {
-		return fmt.Errorf("cannot deserialize to a non-empty namespace table")
-	}
 	// Verify that all fields have the same length
 	if fb.DatabasesLength() != fb.BranchesLength() ||
 		fb.BranchesLength() != fb.UsersLength() ||
@@ -222,6 +221,9 @@ func (tbl *Namespace) Deserialize(fb *serial.BranchControlNamespace) error {
 	if err = tbl.binlog.Deserialize(binlog); err != nil {
 		return err
 	}
+
+	tbl.reinit()
+
 	// Initialize every slice
 	tbl.Databases = make([]MatchExpression, fb.DatabasesLength())
 	tbl.Branches = make([]MatchExpression, fb.BranchesLength())
