@@ -130,7 +130,7 @@ func NewTable(ctx context.Context, vrw types.ValueReadWriter, ns tree.NodeStore,
 		return newDoltDevTable(ctx, vrw, ns, sch, rows, indexes, autoIncVal)
 	}
 
-	schVal, err := encoding.MarshalSchemaAsNomsValue(ctx, vrw, sch)
+	schVal, err := encoding.MarshalSchema(ctx, vrw, sch)
 	if err != nil {
 		return nil, err
 	}
@@ -279,7 +279,7 @@ func (t nomsTable) GetSchemaHash(ctx context.Context) (hash.Hash, error) {
 
 // SetSchema implements Table.
 func (t nomsTable) SetSchema(ctx context.Context, sch schema.Schema) (Table, error) {
-	newSchemaVal, err := encoding.MarshalSchemaAsNomsValue(ctx, t.vrw, sch)
+	newSchemaVal, err := encoding.MarshalSchema(ctx, t.vrw, sch)
 	if err != nil {
 		return nil, err
 	}
@@ -676,17 +676,7 @@ func schemaFromRef(ctx context.Context, vrw types.ValueReadWriter, ref types.Ref
 }
 
 func schemaFromAddr(ctx context.Context, vrw types.ValueReadWriter, addr hash.Hash) (schema.Schema, error) {
-	schemaVal, err := vrw.ReadValue(ctx, addr)
-	if err != nil {
-		return nil, err
-	}
-
-	schema, err := encoding.UnmarshalSchemaNomsValue(ctx, vrw.Format(), schemaVal)
-	if err != nil {
-		return nil, err
-	}
-
-	return schema, nil
+	return encoding.UnmarshalSchemaAtAddr(ctx, vrw, addr)
 }
 
 type doltDevTable struct {
@@ -761,7 +751,7 @@ func (fields serialTableFields) write() (*serial.Table, error) {
 }
 
 func newDoltDevTable(ctx context.Context, vrw types.ValueReadWriter, ns tree.NodeStore, sch schema.Schema, rows Index, indexes IndexSet, autoIncVal types.Value) (Table, error) {
-	schVal, err := encoding.MarshalSchemaAsNomsValue(ctx, vrw, sch)
+	schVal, err := encoding.MarshalSchema(ctx, vrw, sch)
 	if err != nil {
 		return nil, err
 	}
@@ -831,7 +821,7 @@ func (t doltDevTable) GetSchema(ctx context.Context) (schema.Schema, error) {
 }
 
 func (t doltDevTable) SetSchema(ctx context.Context, sch schema.Schema) (Table, error) {
-	newSchemaVal, err := encoding.MarshalSchemaAsNomsValue(ctx, t.vrw, sch)
+	newSchemaVal, err := encoding.MarshalSchema(ctx, t.vrw, sch)
 	if err != nil {
 		return nil, err
 	}
@@ -1140,15 +1130,11 @@ func (t doltDevTable) fields() (serialTableFields, error) {
 }
 
 func getSchemaAtAddr(ctx context.Context, vrw types.ValueReadWriter, addr hash.Hash) (schema.Schema, error) {
-	val, err := vrw.ReadValue(ctx, addr)
-	if err != nil {
-		return nil, err
-	}
-	return encoding.UnmarshalSchemaNomsValue(ctx, vrw.Format(), val)
+	return encoding.UnmarshalSchemaAtAddr(ctx, vrw, addr)
 }
 
 func getAddrForSchema(ctx context.Context, vrw types.ValueReadWriter, sch schema.Schema) (hash.Hash, error) {
-	st, err := encoding.MarshalSchemaAsNomsValue(ctx, vrw, sch)
+	st, err := encoding.MarshalSchema(ctx, vrw, sch)
 	if err != nil {
 		return hash.Hash{}, err
 	}
