@@ -35,6 +35,10 @@ type MergeState struct {
 	commitSpecStr    string
 	preMergeWorking  *RootValue
 	unmergableTables []string
+	mergedTables     []string
+	// isCherryPick is set to true when the in-progress merge is a cherry-pick. This is needed so that
+	// commit knows to NOT create a commit with multiple parents when creating a commit for a cherry-pick.
+	isCherryPick bool
 }
 
 // todo(andy): this might make more sense in pkg merge
@@ -75,6 +79,12 @@ func (m MergeState) CommitSpecStr() string {
 	return m.commitSpecStr
 }
 
+// IsCherryPick returns true if the current merge state is for a cherry-pick operation. Cherry-picks use the same
+// code as merge, but need slightly different behavior (e.g. only recording one commit parent, instead of two).
+func (m MergeState) IsCherryPick() bool {
+	return m.isCherryPick
+}
+
 func (m MergeState) PreMergeWorkingRoot() *RootValue {
 	return m.preMergeWorking
 }
@@ -89,6 +99,10 @@ func (m MergeState) TablesWithSchemaConflicts() []string {
 	return m.unmergableTables
 }
 
+func (m MergeState) MergedTables() []string {
+	return m.mergedTables
+}
+
 func (m MergeState) IterSchemaConflicts(ctx context.Context, ddb *DoltDB, cb SchemaConflictFn) (err error) {
 	var to, from *RootValue
 
@@ -183,6 +197,11 @@ func (ws WorkingSet) WithUnmergableTables(tables []string) *WorkingSet {
 	return &ws
 }
 
+func (ws WorkingSet) WithMergedTables(tables []string) *WorkingSet {
+	ws.mergeState.mergedTables = tables
+	return &ws
+}
+
 func (ws WorkingSet) StartMerge(commit *Commit, commitSpecStr string) *WorkingSet {
 	ws.mergeState = &MergeState{
 		commit:          commit,
@@ -193,6 +212,20 @@ func (ws WorkingSet) StartMerge(commit *Commit, commitSpecStr string) *WorkingSe
 	return &ws
 }
 
+// StartCherryPick creates and returns a new working set based off of the current |ws| with the specified |commit|
+// and |commitSpecStr| referring to the commit being cherry-picked. The returned WorkingSet records that a cherry-pick
+// operation is in progress (i.e. conflicts being resolved). Note that this function does not update the current
+// session – the returned WorkingSet must still be set using DoltSession.SetWorkingSet().
+func (ws WorkingSet) StartCherryPick(commit *Commit, commitSpecStr string) *WorkingSet {
+	ws.mergeState = &MergeState{
+		commit:          commit,
+		commitSpecStr:   commitSpecStr,
+		preMergeWorking: ws.workingRoot,
+		isCherryPick:    true,
+	}
+	return &ws
+}
+
 func (ws WorkingSet) AbortMerge() *WorkingSet {
 	ws.workingRoot = ws.mergeState.PreMergeWorkingRoot()
 	ws.stagedRoot = ws.workingRoot
@@ -221,6 +254,18 @@ func (ws *WorkingSet) MergeActive() bool {
 	return ws.mergeState != nil
 }
 
+// MergeCommitParents returns true if there is an active merge in progress and
+// the recorded commit being merged into the active branch should be included as
+// a second parent of the created commit. This is the expected behavior for a
+// normal merge, but not for other pseudo-merges, like cherry-picks or reverts,
+// where the created commit should only have one parent.
+func (ws *WorkingSet) MergeCommitParents() bool {
+	if !ws.MergeActive() {
+		return false
+	}
+	return ws.MergeState().IsCherryPick() == false
+}
+
 func (ws WorkingSet) Meta() *datas.WorkingSetMeta {
 	return ws.meta
 }
@@ -299,11 +344,17 @@ func newWorkingSet(ctx context.Context, name string, vrw types.ValueReadWriter,
 			return nil, err
 		}
 
+		isCherryPick, err := dsws.MergeState.IsCherryPick(ctx, vrw)
+		if err != nil {
+			return nil, err
+		}
+
 		mergeState = &MergeState{
 			commit:           commit,
 			commitSpecStr:    commitSpec,
 			preMergeWorking:  preMergeWorkingRoot,
 			unmergableTables: unmergableTables,
+			isCherryPick:     isCherryPick,
 		}
 	}
 
@@ -345,7 +396,6 @@ func (ws *WorkingSet) writeValues(ctx context.Context, db *DoltDB) (
 	mergeState *datas.MergeState,
 	err error,
 ) {
-
 	if ws.stagedRoot == nil || ws.workingRoot == nil {
 		return types.Ref{}, types.Ref{}, nil, fmt.Errorf("StagedRoot and workingRoot must be set. This is a bug.")
 	}
@@ -379,7 +429,7 @@ func (ws *WorkingSet) writeValues(ctx context.Context, db *DoltDB) (
 			return types.Ref{}, types.Ref{}, nil, err
 		}
 
-		mergeState, err = datas.NewMergeState(ctx, db.vrw, preMergeWorking, dCommit, ws.mergeState.commitSpecStr, ws.mergeState.unmergableTables)
+		mergeState, err = datas.NewMergeState(ctx, db.vrw, preMergeWorking, dCommit, ws.mergeState.commitSpecStr, ws.mergeState.unmergableTables, ws.mergeState.isCherryPick)
 		if err != nil {
 			return types.Ref{}, types.Ref{}, nil, err
 		}
