@@ -20,6 +20,7 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"reflect"
 	"strconv"
 	"strings"
 	"time"
@@ -108,8 +109,7 @@ func (dEnv *DoltEnv) GetRemoteDB(ctx context.Context, format *types.NomsBinForma
 	}
 }
 
-func LoadWithoutDB(ctx context.Context, hdp HomeDirProvider, fs filesys.Filesys, version string) *DoltEnv {
-	cfg, cfgErr := LoadDoltCliConfig(hdp, fs)
+func createRepoState(fs filesys.Filesys) (*RepoState, error) {
 	repoState, rsErr := LoadRepoState(fs)
 
 	// deep copy remotes and backups ¯\_(ツ)_/¯ (see commit c59cbead)
@@ -127,6 +127,23 @@ func LoadWithoutDB(ctx context.Context, hdp HomeDirProvider, fs filesys.Filesys,
 		repoState.Backups = backups
 	}
 
+	return repoState, rsErr
+}
+
+func (dEnv *DoltEnv) ReloadRepoState() error {
+	rs, err := createRepoState(dEnv.FS)
+	if err != nil {
+		return err
+	}
+	dEnv.RepoState = rs
+	return nil
+}
+
+func LoadWithoutDB(ctx context.Context, hdp HomeDirProvider, fs filesys.Filesys, version string) *DoltEnv {
+	cfg, cfgErr := LoadDoltCliConfig(hdp, fs)
+
+	repoState, rsErr := createRepoState(fs)
+
 	return &DoltEnv{
 		Version:    version,
 		Config:     cfg,
@@ -349,7 +366,7 @@ func (dEnv *DoltEnv) bestEffortDeleteAll(dir string) {
 // InitRepo takes an empty directory and initializes it with a .dolt directory containing repo state, uncommitted license and readme, and creates a noms
 // database with dolt structure.
 func (dEnv *DoltEnv) InitRepo(ctx context.Context, nbf *types.NomsBinFormat, name, email, branchName string) error { // should remove name and email args
-	return dEnv.InitRepoWithTime(ctx, nbf, name, email, branchName, datas.CommitNowFunc())
+	return dEnv.InitRepoWithTime(ctx, nbf, name, email, branchName, datas.CommitterDate())
 }
 
 func (dEnv *DoltEnv) InitRepoWithTime(ctx context.Context, nbf *types.NomsBinFormat, name, email, branchName string, t time.Time) error { // should remove name and email args
@@ -411,7 +428,15 @@ func (dEnv *DoltEnv) createDirectories(dir string) (string, error) {
 	}
 
 	if dEnv.hasDoltDir(dir) {
-		return "", fmt.Errorf(".dolt directory already exists at '%s'", dir)
+		// Special case a completely empty directory. We can allow that.
+		dotDolt := mustAbs(dEnv, dbfactory.DoltDir)
+		entries, err := os.ReadDir(dotDolt)
+		if err != nil {
+			return "", err
+		}
+		if len(entries) != 0 {
+			return "", fmt.Errorf(".dolt directory already exists at '%s'", dir)
+		}
 	}
 
 	absDataDir := filepath.Join(absPath, dbfactory.DoltDataDir)
@@ -663,10 +688,16 @@ type repoStateReader struct {
 }
 
 func (r *repoStateReader) CWBHeadRef() (ref.DoltRef, error) {
+	if r.RepoState == nil && r.RSLoadErr != nil {
+		return nil, r.RSLoadErr
+	}
 	return r.RepoState.CWBHeadRef(), nil
 }
 
 func (r *repoStateReader) CWBHeadSpec() (*doltdb.CommitSpec, error) {
+	if r.RepoState == nil && r.RSLoadErr != nil {
+		return nil, r.RSLoadErr
+	}
 	return r.RepoState.CWBHeadSpec(), nil
 }
 
@@ -679,6 +710,10 @@ type repoStateWriter struct {
 }
 
 func (r *repoStateWriter) SetCWBHeadRef(ctx context.Context, marshalableRef ref.MarshalableRef) error {
+	if r.RepoState == nil && r.RSLoadErr != nil {
+		return r.RSLoadErr
+	}
+
 	r.RepoState.Head = marshalableRef
 	err := r.RepoState.Save(r.FS)
 
@@ -1177,7 +1212,7 @@ func NewDBLock(port int) DBLock {
 }
 
 // Lock writes this database's lockfile with the pid of the calling process or errors if it already exists
-func (dEnv *DoltEnv) Lock(lock DBLock) error {
+func (dEnv *DoltEnv) Lock(lock *DBLock) error {
 	if dEnv.IgnoreLockFile {
 		return nil
 	}
@@ -1241,7 +1276,14 @@ func (dEnv *DoltEnv) Unlock() error {
 }
 
 // WriteLockfile writes a lockfile encoding the pid of the calling process.
-func WriteLockfile(fs filesys.Filesys, lock DBLock) error {
+func WriteLockfile(fs filesys.Filesys, lock *DBLock) error {
+	// if the DoltDir doesn't exist, create it.
+	doltDir, _ := fs.Abs(dbfactory.DoltDir)
+	err := fs.MkDirs(doltDir)
+	if err != nil {
+		return err
+	}
+
 	lockFile, _ := fs.Abs(filepath.Join(dbfactory.DoltDir, ServerLockFile))
 
 	portStr := strconv.Itoa(lock.Port)
@@ -1249,7 +1291,7 @@ func WriteLockfile(fs filesys.Filesys, lock DBLock) error {
 		portStr = "-"
 	}
 
-	if filesys.LocalFS == fs {
+	if reflect.TypeOf(fs) == reflect.TypeOf(filesys.LocalFS) {
 		_, err := os.Create(lockFile)
 		if err != nil {
 			return err
@@ -1260,7 +1302,7 @@ func WriteLockfile(fs filesys.Filesys, lock DBLock) error {
 		}
 	}
 
-	err := fs.WriteFile(lockFile, []byte(fmt.Sprintf("%d:%s:%s", lock.Pid, portStr, lock.Secret)))
+	err = fs.WriteFile(lockFile, []byte(fmt.Sprintf("%d:%s:%s", lock.Pid, portStr, lock.Secret)))
 	if err != nil {
 		return err
 	}
