@@ -242,9 +242,8 @@ func NewPushOpts(ctx context.Context, apr *argparser.ArgParseResults, rsr RepoSt
 	return opts, nil
 }
 
-// NewFetchOpts returns remote and refSpec for given remote name. If remote name is not defined,
-// default remote is used. Default remote is "origin" if there are multiple remotes for now.
-func NewFetchOpts(args []string, rsr RepoStateReader) (Remote, []ref.RemoteRefSpec, error) {
+// RemoteForFetchArgs returns the remote and remaining arg strings for a fetch command
+func RemoteForFetchArgs(args []string, rsr RepoStateReader) (Remote, []string, error) {
 	var err error
 	remotes, err := rsr.GetRemotes()
 	if err != nil {
@@ -269,18 +268,17 @@ func NewFetchOpts(args []string, rsr RepoStateReader) (Remote, []ref.RemoteRefSp
 		return NoRemote, nil, fmt.Errorf("%w; '%s' %s", ErrUnknownRemote, remName, msg)
 	}
 
-	var rs []ref.RemoteRefSpec
+	return remote, args, nil
+}
+
+// ParseRefSpecs returns the ref specs for the string arguments given for the remote provided, or the default ref
+// specs for that remote if no arguments are provided.
+func ParseRefSpecs(args []string, rsr RepoStateReader, remote Remote) ([]ref.RemoteRefSpec, error) {
 	if len(args) != 0 {
-		rs, err = ParseRSFromArgs(remName, args)
+		return ParseRSFromArgs(remote.Name, args)
 	} else {
-		rs, err = GetRefSpecs(rsr, remName)
+		return GetRefSpecs(rsr, remote.Name)
 	}
-
-	if err != nil {
-		return NoRemote, nil, err
-	}
-
-	return remote, rs, err
 }
 
 func ParseRSFromArgs(remName string, args []string) ([]ref.RemoteRefSpec, error) {
@@ -393,9 +391,8 @@ func GetTrackingRef(branchRef ref.DoltRef, remote Remote) (ref.DoltRef, error) {
 }
 
 type PullSpec struct {
-	Msg        string
 	Squash     bool
-	Noff       bool
+	NoFF       bool
 	NoCommit   bool
 	NoEdit     bool
 	Force      bool
@@ -405,10 +402,47 @@ type PullSpec struct {
 	Branch     ref.DoltRef
 }
 
-// NewPullSpec returns PullSpec object using arguments passed into this function, which are remoteName, remoteRefName,
-// squash, noff, noCommit, noEdit,  refSpecs, force and remoteOnly. This function validates remote and gets remoteRef
+type PullSpecOpt func(*PullSpec)
+
+func WithSquash(squash bool) PullSpecOpt {
+	return func(ps *PullSpec) {
+		ps.Squash = squash
+	}
+}
+
+func WithNoFF(noff bool) PullSpecOpt {
+	return func(ps *PullSpec) {
+		ps.NoFF = noff
+	}
+}
+
+func WithNoCommit(nocommit bool) PullSpecOpt {
+	return func(ps *PullSpec) {
+		ps.NoCommit = nocommit
+	}
+}
+
+func WithNoEdit(noedit bool) PullSpecOpt {
+	return func(ps *PullSpec) {
+		ps.NoEdit = noedit
+	}
+}
+
+func WithForce(force bool) PullSpecOpt {
+	return func(ps *PullSpec) {
+		ps.Force = force
+	}
+}
+
+// NewPullSpec returns a PullSpec for the arguments given. This function validates remote and gets remoteRef
 // for given remoteRefName; if it's not defined, it uses current branch to get its upstream branch if it exists.
-func NewPullSpec(_ context.Context, rsr RepoStateReader, remoteName, remoteRefName string, squash, noff, noCommit, noEdit, force, remoteOnly bool) (*PullSpec, error) {
+func NewPullSpec(
+	_ context.Context,
+	rsr RepoStateReader,
+	remoteName, remoteRefName string,
+	remoteOnly bool,
+	opts ...PullSpecOpt,
+) (*PullSpec, error) {
 	refSpecs, err := GetRefSpecs(rsr, remoteName)
 	if err != nil {
 		return nil, err
@@ -448,17 +482,18 @@ func NewPullSpec(_ context.Context, rsr RepoStateReader, remoteName, remoteRefNa
 		remoteRef = ref.NewBranchRef(remoteRefName)
 	}
 
-	return &PullSpec{
-		Squash:     squash,
-		Noff:       noff,
-		NoCommit:   noCommit,
-		NoEdit:     noEdit,
+	spec := &PullSpec{
 		RemoteName: remoteName,
 		Remote:     remote,
 		RefSpecs:   refSpecs,
 		Branch:     remoteRef,
-		Force:      force,
-	}, nil
+	}
+
+	for _, opt := range opts {
+		opt(spec)
+	}
+
+	return spec, nil
 }
 
 func GetAbsRemoteUrl(fs filesys2.Filesys, cfg config.ReadableConfig, urlArg string) (string, string, error) {
