@@ -20,17 +20,15 @@ import (
 	"fmt"
 	"time"
 
-	"github.com/dolthub/dolt/go/libraries/doltcore/table/editor"
+	"github.com/dolthub/go-mysql-server/sql"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
+	"github.com/dolthub/dolt/go/libraries/doltcore/table/editor"
 	"github.com/dolthub/dolt/go/store/hash"
 )
 
-var ErrFailedToDetermineUnstagedDocs = errors.New("failed to determine unstaged docs")
 var ErrFailedToReadDatabase = errors.New("failed to read database")
-var ErrMergeFailedToUpdateDocs = errors.New("failed to update docs to the new working root")
-var ErrMergeFailedToUpdateRepoState = errors.New("unable to execute repo state update")
 var ErrFailedToDetermineMergeability = errors.New("failed to determine mergeability")
 
 type MergeSpec struct {
@@ -42,21 +40,67 @@ type MergeSpec struct {
 	StompedTblNames []string
 	WorkingDiffs    map[string]hash.Hash
 	Squash          bool
-	Msg             string
-	Noff            bool
+	NoFF            bool
 	NoCommit        bool
 	NoEdit          bool
 	Force           bool
-	AllowEmpty      bool
 	Email           string
 	Name            string
 	Date            time.Time
 }
 
-// NewMergeSpec returns MergeSpec object using arguments passed into this function, which are doltdb.Roots, username,
-// user email, commit msg, commitSpecStr, to squash, to noff, to force, noCommit, noEdit and date. This function
-// resolves head and merge commit, and it gets current diffs between current head and working set if it exists.
-func NewMergeSpec(ctx context.Context, rsr env.RepoStateReader, ddb *doltdb.DoltDB, roots doltdb.Roots, name, email, msg, commitSpecStr string, squash, noff, force, noCommit, noEdit bool, date time.Time) (*MergeSpec, error) {
+type MergeSpecOpt func(*MergeSpec)
+
+func WithNoFF(noFF bool) MergeSpecOpt {
+	return func(ms *MergeSpec) {
+		ms.NoFF = noFF
+	}
+}
+
+func WithNoCommit(noCommit bool) MergeSpecOpt {
+	return func(ms *MergeSpec) {
+		ms.NoCommit = noCommit
+	}
+}
+
+func WithNoEdit(noEdit bool) MergeSpecOpt {
+	return func(ms *MergeSpec) {
+		ms.NoEdit = noEdit
+	}
+}
+
+func WithForce(force bool) MergeSpecOpt {
+	return func(ms *MergeSpec) {
+		ms.Force = force
+	}
+}
+
+func WithSquash(squash bool) MergeSpecOpt {
+	return func(ms *MergeSpec) {
+		ms.Squash = squash
+	}
+}
+
+func WithPullSpecOpts(pullSpec *env.PullSpec) MergeSpecOpt {
+	return func(ms *MergeSpec) {
+		ms.NoEdit = pullSpec.NoEdit
+		ms.NoCommit = pullSpec.NoCommit
+		ms.Force = pullSpec.Force
+		ms.NoFF = pullSpec.NoFF
+		ms.Squash = pullSpec.Squash
+	}
+}
+
+// NewMergeSpec returns a MergeSpec with the arguments provided.
+func NewMergeSpec(
+	ctx context.Context,
+	rsr env.RepoStateReader,
+	ddb *doltdb.DoltDB,
+	roots doltdb.Roots,
+	name, email, commitSpecStr string,
+	date time.Time,
+	opts ...MergeSpecOpt,
+) (*MergeSpec, error) {
 	headCS, err := doltdb.NewCommitSpec("HEAD")
 	if err != nil {
 		return nil, err
@@ -98,7 +142,7 @@ func NewMergeSpec(ctx context.Context, rsr env.RepoStateReader, ddb *doltdb.Dolt
 		return nil, fmt.Errorf("%w; %s", ErrFailedToDetermineMergeability, err.Error())
 	}
 
-	return &MergeSpec{
+	spec := &MergeSpec{
 		HeadH:           headH,
 		MergeH:          mergeH,
 		HeadC:           headCM,
@@ -106,16 +150,16 @@ func NewMergeSpec(ctx context.Context, rsr env.RepoStateReader, ddb *doltdb.Dolt
 		MergeC:          mergeCM,
 		StompedTblNames: stompedTblNames,
 		WorkingDiffs:    workingDiffs,
-		Squash:          squash,
-		Msg:             msg,
-		Noff:            noff,
-		NoCommit:        noCommit,
-		NoEdit:          noEdit,
-		Force:           force,
 		Email:           email,
 		Name:            name,
 		Date:            date,
-	}, nil
+	}
+
+	for _, opt := range opts {
+		opt(spec)
+	}
+
+	return spec, nil
 }
 
 func ExecNoFFMerge(ctx context.Context, dEnv *env.DoltEnv, spec *MergeSpec) (map[string]*MergeStats, error) {
@@ -183,7 +227,7 @@ func ExecuteFFMerge(
 	return dEnv.UpdateWorkingSet(ctx, workingSet.WithWorkingRoot(workingRoot).WithStagedRoot(stagedRoot))
 }
 
-func ExecuteMerge(ctx context.Context, dEnv *env.DoltEnv, spec *MergeSpec) (map[string]*MergeStats, error) {
+func ExecuteMerge(ctx *sql.Context, dEnv *env.DoltEnv, spec *MergeSpec) (map[string]*MergeStats, error) {
 	tmpDir, err := dEnv.TempTableFilesDir()
 	if err != nil {
 		return nil, err
