@@ -27,11 +27,13 @@ import (
 	"github.com/dolthub/go-mysql-server/sql/analyzer"
 	"github.com/dolthub/go-mysql-server/sql/planbuilder"
 	"github.com/dolthub/go-mysql-server/sql/transform"
+	"github.com/dolthub/go-mysql-server/sql/types"
 	errorkinds "gopkg.in/src-d/go-errors.v1"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb/durable"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
+	"github.com/dolthub/dolt/go/libraries/doltcore/schema/typeinfo"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
 	"github.com/dolthub/dolt/go/store/hash"
@@ -48,15 +50,11 @@ var ErrUnableToMergeColumnDefaultValue = errorkinds.NewKind("unable to automatic
 	"in merge: %s for table '%s'; to continue merging, first manually apply the column alteration on this branch")
 
 // mergeProllyTable merges the table specified by |tm| using the specified |mergedSch| and returns the new table
-// instance, along with merge stats and any error. This function will merge the table artifacts (e.g. recorded
-// conflicts), migrate any existing table data to the specified |mergedSch|, and merge table data from both sides
-// of the merge together.
-func mergeProllyTable(ctx context.Context, tm *TableMerger, mergedSch schema.Schema) (*doltdb.Table, *MergeStats, error) {
-	err := maybeAbortDueToUnmergeableIndexes(tm.name, tm.leftSch, tm.rightSch, mergedSch)
-	if err != nil {
-		return nil, nil, err
-	}
-
+// instance, along with merge stats and any error. If |rewriteRows| is true, then any existing rows in the
+// table's primary index will also be rewritten. This function merges the table's artifacts (e.g. recorded
+// conflicts), migrates any existing table data to the specified |mergedSch|, and merges table data from both
+// sides of the merge together.
+func mergeProllyTable(ctx context.Context, tm *TableMerger, mergedSch schema.Schema, rewriteRows bool) (*doltdb.Table, *MergeStats, error) {
 	mergeTbl, err := mergeTableArtifacts(ctx, tm, tm.leftTbl)
 	if err != nil {
 		return nil, nil, err
@@ -72,7 +70,7 @@ func mergeProllyTable(ctx context.Context, tm *TableMerger, mergedSch schema.Sch
 		return nil, nil, err
 	}
 	leftRows := durable.ProllyMapFromIndex(lr)
-	valueMerger := newValueMerger(mergedSch, tm.leftSch, tm.rightSch, tm.ancSch, leftRows.Pool())
+	valueMerger := newValueMerger(mergedSch, tm.leftSch, tm.rightSch, tm.ancSch, leftRows.Pool(), tm.ns)
 	leftMapping := valueMerger.leftMapping
 
 	// We need a sql.Context to apply column default values in merges; if we don't have one already,
@@ -84,14 +82,20 @@ func mergeProllyTable(ctx context.Context, tm *TableMerger, mergedSch schema.Sch
 
 	// Migrate primary index data to rewrite the values on the left side of the merge if necessary
 	schemasDifferentSize := len(tm.leftSch.GetAllCols().GetColumns()) != len(mergedSch.GetAllCols().GetColumns())
-	if schemasDifferentSize || leftMapping.IsIdentityMapping() == false {
+	if rewriteRows || schemasDifferentSize || leftMapping.IsIdentityMapping() == false {
 		if err := migrateDataToMergedSchema(sqlCtx, tm, valueMerger, mergedSch); err != nil {
 			return nil, nil, err
 		}
 
 		// After we migrate the data on the left-side to the new, merged schema, we reset
 		// the left mapping to an identity mapping, since it's a direct mapping now.
-		valueMerger.leftMapping = val.NewIdentityOrdinalMapping(len(valueMerger.leftMapping))
+		// However, columns that did not exist on the left schema shouldn't be updated, because they
+		// still don't exist on the migrated data.
+		for i, col := range mergedSch.GetNonPKCols().GetColumns() {
+			if findNonPKColumnMappingByTagOrName(tm.leftSch, col) != -1 {
+				valueMerger.leftMapping[i] = i
+			}
+		}
 	}
 
 	// After we've migrated the existing data to the new schema, it's safe for us to update the schema on the table
@@ -101,7 +105,7 @@ func mergeProllyTable(ctx context.Context, tm *TableMerger, mergedSch schema.Sch
 	}
 
 	var stats *MergeStats
-	mergeTbl, stats, err = mergeProllyTableData(sqlCtx, tm, mergedSch, mergeTbl, valueMerger)
+	mergeTbl, stats, err = mergeProllyTableData(sqlCtx, tm, mergedSch, mergeTbl, valueMerger, rewriteRows)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -126,8 +130,10 @@ func mergeProllyTable(ctx context.Context, tm *TableMerger, mergedSch schema.Sch
 // to the right-side, we apply it to the left-side by merging it into the left-side's primary index
 // as well as any secondary indexes, and also checking for unique constraints incrementally. When
 // conflicts are detected, this function attempts to resolve them automatically if possible, and
-// if not, they are recorded as conflicts in the table's artifacts.
-func mergeProllyTableData(ctx *sql.Context, tm *TableMerger, finalSch schema.Schema, mergeTbl *doltdb.Table, valueMerger *valueMerger) (*doltdb.Table, *MergeStats, error) {
+// if not, they are recorded as conflicts in the table's artifacts. If |rebuildIndexes| is set to
+// true, then secondary indexes will be rebuilt, instead of being incrementally merged together. This
+// is less efficient, but safer, especially when type changes have been applied to a table's schema.
+func mergeProllyTableData(ctx *sql.Context, tm *TableMerger, finalSch schema.Schema, mergeTbl *doltdb.Table, valueMerger *valueMerger, rebuildIndexes bool) (*doltdb.Table, *MergeStats, error) {
 	iter, err := threeWayDiffer(ctx, tm, valueMerger)
 	if err != nil {
 		return nil, nil, err
@@ -137,7 +143,7 @@ func mergeProllyTableData(ctx *sql.Context, tm *TableMerger, finalSch schema.Sch
 	if err != nil {
 		return nil, nil, err
 	}
-	leftEditor := durable.ProllyMapFromIndex(lr).Mutate()
+	leftEditor := durable.ProllyMapFromIndex(lr).Rewriter(finalSch.GetKeyDescriptor(), finalSch.GetValueDescriptor())
 
 	ai, err := mergeTbl.GetArtifacts(ctx)
 	if err != nil {
@@ -186,7 +192,6 @@ func mergeProllyTableData(ctx *sql.Context, tm *TableMerger, finalSch schema.Sch
 		} else if err != nil {
 			return nil, nil, err
 		}
-
 		cnt, err := uniq.validateDiff(ctx, diff)
 		if err != nil {
 			return nil, nil, err
@@ -284,7 +289,7 @@ func mergeProllyTableData(ctx *sql.Context, tm *TableMerger, finalSch schema.Sch
 		return nil, nil, err
 	}
 
-	finalIdxs, err := mergeProllySecondaryIndexes(ctx, tm, leftIdxs, rightIdxs, finalSch, finalRows, conflicts.ae)
+	finalIdxs, err := mergeProllySecondaryIndexes(ctx, tm, leftIdxs, rightIdxs, finalSch, finalRows, conflicts.ae, rebuildIndexes)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -310,24 +315,6 @@ func mergeProllyTableData(ctx *sql.Context, tm *TableMerger, finalSch schema.Sch
 	return finalTbl, s, nil
 }
 
-func maybeAbortDueToUnmergeableIndexes(tableName string, leftSchema, rightSchema, targetSchema schema.Schema) error {
-	leftOk, err := validateTupleFields(leftSchema, targetSchema)
-	if err != nil {
-		return err
-	}
-
-	rightOk, err := validateTupleFields(rightSchema, targetSchema)
-	if err != nil {
-		return err
-	}
-
-	if !leftOk || !rightOk {
-		return fmt.Errorf("table %s can't be automatically merged.\nTo merge this table, make the schema on the source and target branch equal.", tableName)
-	}
-
-	return nil
-}
-
 func threeWayDiffer(ctx context.Context, tm *TableMerger, valueMerger *valueMerger) (*tree.ThreeWayDiffer[val.Tuple, val.TupleDesc], error) {
 	lr, err := tm.leftTbl.GetRowData(ctx)
 	if err != nil {
@@ -447,11 +434,25 @@ func (cv checkValidator) validateDiff(ctx *sql.Context, diff tree.ThreeWayDiff)
 			return 0, err
 		}
 
-		if result == nil || result == true {
-			// If a check constraint returns NULL or TRUE, then the check constraint is fulfilled
+		// MySQL treats NULL as TRUE for a check constraint
+		if result == nil {
+			result = true
+		}
+
+		// Coerce into a boolean; technically, this shouldn't be
+		// necessary, since check constraint expressions should always
+		// be of a boolean type, but Dolt has allowed this previously.
+		// https://github.com/dolthub/dolt/issues/6411
+		booleanResult, err := types.ConvertToBool(result)
+		if err != nil {
+			return 0, fmt.Errorf("unable to convert check constraint expression (%s) into boolean value: %v", checkName, err.Error())
+		}
+
+		if booleanResult {
+			// If a check constraint returns TRUE (or NULL), then the check constraint is fulfilled
 			// https://dev.mysql.com/doc/refman/8.0/en/create-table-check-constraints.html
 			continue
-		} else if result == false {
+		} else {
 			conflictCount++
 			meta, err := newCheckCVMeta(cv.sch, checkName)
 			if err != nil {
@@ -460,8 +461,6 @@ func (cv checkValidator) validateDiff(ctx *sql.Context, diff tree.ThreeWayDiff)
 			if err = cv.insertArtifact(ctx, diff.Key, newTuple, meta); err != nil {
 				return conflictCount, err
 			}
-		} else {
-			return 0, fmt.Errorf("unexpected result from check constraint expression: %v", result)
 		}
 	}
 
@@ -585,11 +584,18 @@ func newUniqValidator(ctx context.Context, sch schema.Schema, tm *TableMerger, v
 	return uv, nil
 }
 
+// validateDiff processes |diff| and checks for any unique constraint violations that need to be updated. The number
+// of violations recorded along with any error encountered is returned. Processing |diff| may resolve existing unique
+// constraint violations, in which case the violations returned may be a negative number.
 func (uv uniqValidator) validateDiff(ctx context.Context, diff tree.ThreeWayDiff) (violations int, err error) {
 	var value val.Tuple
 	switch diff.Op {
 	case tree.DiffOpRightAdd, tree.DiffOpRightModify:
 		value = diff.Right
+	case tree.DiffOpRightDelete:
+		// If we see a row deletion event from the right side, we grab the original/base value so that we can update our
+		// local copy of the secondary index.
+		value = diff.Base
 	case tree.DiffOpDivergentModifyResolved:
 		value = diff.Merged
 	default:
@@ -602,6 +608,23 @@ func (uv uniqValidator) validateDiff(ctx context.Context, diff tree.ThreeWayDiff
 		value = val.NewTuple(uv.valueMerger.syncPool, modifiedValue...)
 	}
 
+	// For a row deletion... we need to remove any unique constraint violations that were previously recorded for
+	// this row.
+	if diff.Op == tree.DiffOpRightDelete {
+		// First update the unique indexes to remove this row.
+		for _, idx := range uv.indexes {
+			err := idx.removeRow(ctx, diff.Key, value)
+			if err != nil {
+				return violations, err
+			}
+		}
+
+		// Then clear any unique constraint violation artifacts for this row. If there is only one unique constraint
+		// violation artifact left, it will also be cleared by this function (since unique constraint violations
+		// must always occur with at least two rows reported).
+		return uv.clearArtifact(ctx, diff.Key, diff.Base)
+	}
+
 	for _, idx := range uv.indexes {
 		err = idx.findCollisions(ctx, diff.Key, value, func(k, v val.Tuple) error {
 			violations++
@@ -611,7 +634,74 @@ func (uv uniqValidator) validateDiff(ctx context.Context, diff tree.ThreeWayDiff
 			break
 		}
 	}
-	return
+
+	// After detecting any unique constraint violations, we need to update our indexes with the updated row
+	if diff.Op == tree.DiffOpRightAdd || diff.Op == tree.DiffOpRightModify || diff.Op == tree.DiffOpDivergentModifyResolved {
+		for _, idx := range uv.indexes {
+			err := idx.insertRow(ctx, diff.Key, value)
+			if err != nil {
+				return violations, err
+			}
+
+			err = idx.clustered.Put(ctx, diff.Key, value)
+			if err != nil {
+				return violations, err
+			}
+		}
+	}
+
+	return violations, err
+}
+
+// deleteArtifact deletes the unique constraint violation artifact for the row identified by |key| and returns a
+// boolean that indicates if an artifact was deleted, as well as an error that indicates if there were any
+// unexpected errors encountered.
+func (uv uniqValidator) deleteArtifact(ctx context.Context, key val.Tuple) (bool, error) {
+	artifactKey := uv.edits.BuildArtifactKey(ctx, key, uv.srcHash, prolly.ArtifactTypeUniqueKeyViol)
+
+	has, err := uv.edits.Has(ctx, artifactKey)
+	if err != nil || !has {
+		return false, err
+	}
+
+	err = uv.edits.Delete(ctx, artifactKey)
+	if err != nil {
+		return false, err
+	}
+
+	return true, nil
+}
+
+// clearArtifactsForValue deletes the unique constraint violation artifact for the row identified by |key| and |value|
+// and then checks to see if only one unique constraint violation artifact remains, and if so, deletes it as well,
+// since only a single row remaining for a unique constraint violation means that the violation has been fully
+// resolved and no other rows conflict with that unique value.
+func (uv uniqValidator) clearArtifact(ctx context.Context, key val.Tuple, prevValue val.Tuple) (int, error) {
+	deleted, err := uv.deleteArtifact(ctx, key)
+	if err != nil || !deleted {
+		return 0, err
+	}
+
+	// Start the violation count at -1 to represent the artifact above that we just removed
+	violationCount := -1
+
+	for _, idx := range uv.indexes {
+		// TODO: Test with multiple unique indexes and constraint violations on different values
+		//       Multiple unique indexes won't work yet: https://github.com/dolthub/dolt/issues/6329
+		err := idx.findCollisions(ctx, key, prevValue, func(k, v val.Tuple) error {
+			deleted, err := uv.deleteArtifact(ctx, k)
+			if err != nil || !deleted {
+				return err
+			}
+			violationCount = violationCount - 1
+			return nil
+		})
+		if err != nil {
+			break
+		}
+	}
+
+	return violationCount, nil
 }
 
 func (uv uniqValidator) insertArtifact(ctx context.Context, key, value val.Tuple, meta UniqCVMeta) error {
@@ -624,16 +714,17 @@ func (uv uniqValidator) insertArtifact(ctx context.Context, key, value val.Tuple
 }
 
 type uniqIndex struct {
-	def          schema.Index
-	secondary    prolly.Map
-	clustered    prolly.Map
-	meta         UniqCVMeta
-	prefixDesc   val.TupleDesc
-	secondaryBld index.SecondaryKeyBuilder
-	clusteredBld index.ClusteredKeyBuilder
+	def              schema.Index
+	secondary        *prolly.MutableMap
+	clustered        *prolly.MutableMap
+	meta             UniqCVMeta
+	prefixDesc       val.TupleDesc
+	secondaryBld     index.SecondaryKeyBuilder
+	clusteredBld     index.ClusteredKeyBuilder
+	clusteredKeyDesc val.TupleDesc
 }
 
-func newUniqIndex(sch schema.Schema, def schema.Index, clusterd, secondary prolly.Map) (uniqIndex, error) {
+func newUniqIndex(sch schema.Schema, def schema.Index, clustered, secondary prolly.Map) (uniqIndex, error) {
 	meta, err := makeUniqViolMeta(sch, def)
 	if err != nil {
 		return uniqIndex{}, err
@@ -642,58 +733,104 @@ func newUniqIndex(sch schema.Schema, def schema.Index, clusterd, secondary proll
 	if schema.IsKeyless(sch) { // todo(andy): sad panda
 		secondary = prolly.ConvertToSecondaryKeylessIndex(secondary)
 	}
-	p := clusterd.Pool()
+	p := clustered.Pool()
 
 	prefixDesc := secondary.KeyDesc().PrefixDesc(def.Count())
-	secondaryBld := index.NewSecondaryKeyBuilder(sch, def, secondary.KeyDesc(), p)
-	clusteredBld := index.NewClusteredKeyBuilder(def, sch, clusterd.KeyDesc(), p)
+	secondaryBld := index.NewSecondaryKeyBuilder(sch, def, secondary.KeyDesc(), p, secondary.NodeStore())
+	clusteredBld := index.NewClusteredKeyBuilder(def, sch, clustered.KeyDesc(), p)
 
 	return uniqIndex{
-		def:          def,
-		secondary:    secondary,
-		clustered:    clusterd,
-		meta:         meta,
-		prefixDesc:   prefixDesc,
-		secondaryBld: secondaryBld,
-		clusteredBld: clusteredBld,
+		def:              def,
+		secondary:        secondary.Mutate(),
+		clustered:        clustered.Mutate(),
+		clusteredKeyDesc: clustered.KeyDesc(),
+		meta:             meta,
+		prefixDesc:       prefixDesc,
+		secondaryBld:     secondaryBld,
+		clusteredBld:     clusteredBld,
 	}, nil
 }
 
 type collisionFn func(key, value val.Tuple) error
 
+func (idx uniqIndex) insertRow(ctx context.Context, key, value val.Tuple) error {
+	secondaryIndexKey, err := idx.secondaryBld.SecondaryKeyFromRow(ctx, key, value)
+	if err != nil {
+		return err
+	}
+
+	// secondary indexes only use their key tuple
+	return idx.secondary.Put(ctx, secondaryIndexKey, val.EmptyTuple)
+}
+
+func (idx uniqIndex) removeRow(ctx context.Context, key, value val.Tuple) error {
+	secondaryIndexKey, err := idx.secondaryBld.SecondaryKeyFromRow(ctx, key, value)
+	if err != nil {
+		return err
+	}
+
+	err = idx.secondary.Delete(ctx, secondaryIndexKey)
+	if err != nil {
+		return err
+	}
+
+	clusteredIndexKey := idx.clusteredBld.ClusteredKeyFromIndexKey(secondaryIndexKey)
+	return idx.clustered.Delete(ctx, clusteredIndexKey)
+}
+
+// findCollisions searches this unique index to find any rows that have the same values as |value| for the columns
+// included in the unique constraint. For any matching row, the specified callback, |cb|, is invoked with the key
+// and value for the primary index, representing the conflicting row identified from the unique index.
 func (idx uniqIndex) findCollisions(ctx context.Context, key, value val.Tuple, cb collisionFn) error {
-	indexKey := idx.secondaryBld.SecondaryKeyFromRow(key, value)
+	indexKey, err := idx.secondaryBld.SecondaryKeyFromRow(ctx, key, value)
+	if err != nil {
+		return err
+	}
+
 	if idx.prefixDesc.HasNulls(indexKey) {
 		return nil // NULLs cannot cause unique violations
 	}
 
-	var collision val.Tuple
-	err := idx.secondary.GetPrefix(ctx, indexKey, idx.prefixDesc, func(k, _ val.Tuple) (err error) {
-		collision = k
+	// This code uses the secondary index to iterate over all rows (key/value pairs) that have the same prefix.
+	// The prefix here is all the value columns this index is set up to track
+	collisions := make([]val.Tuple, 0)
+	err = idx.secondary.GetPrefix(ctx, indexKey, idx.prefixDesc, func(k, _ val.Tuple) (err error) {
+		if k != nil {
+			collisions = append(collisions, k)
+		}
 		return
 	})
-	if err != nil || collision == nil {
+	if err != nil || len(collisions) == 0 {
 		return err
 	}
 
-	clusteredKey := idx.clusteredBld.ClusteredKeyFromIndexKey(collision)
-	if bytes.Equal(key, clusteredKey) {
-		return nil // collided with ourselves
-	}
+	collisionDetected := false
+	for _, collision := range collisions {
+		// Next find the key in the primary (aka clustered) index
+		clusteredKey := idx.clusteredBld.ClusteredKeyFromIndexKey(collision)
+		if bytes.Equal(key, clusteredKey) {
+			continue // collided with ourselves
+		}
 
-	// |prefix| was non-unique, find the clustered index row that
-	// collided with row(|key|, |value|) and pass both to |cb|
-	err = idx.clustered.Get(ctx, clusteredKey, func(k val.Tuple, v val.Tuple) error {
-		if k == nil {
-			s := idx.clustered.KeyDesc().Format(clusteredKey)
-			return errors.New("failed to find key: " + s)
+		// |prefix| was non-unique, find the clustered index row that
+		// collided with row(|key|, |value|) and pass both to |cb|
+		err = idx.clustered.Get(ctx, clusteredKey, func(k val.Tuple, v val.Tuple) error {
+			if k == nil {
+				s := idx.clusteredKeyDesc.Format(clusteredKey)
+				return errors.New("failed to find key: " + s)
+			}
+			collisionDetected = true
+			return cb(k, v)
+		})
+		if err != nil {
+			return err
 		}
-		return cb(k, v)
-	})
-	if err != nil {
-		return err
 	}
-	return cb(key, value)
+	if collisionDetected {
+		return cb(key, value)
+	} else {
+		return nil
+	}
 }
 
 // nullValidator enforces NOT NULL constraints on merge
@@ -931,7 +1068,7 @@ func (m *primaryMerger) merge(ctx *sql.Context, diff tree.ThreeWayDiff, sourceSc
 			}
 		} else {
 			tempTupleValue, err := remapTupleWithColumnDefaults(ctx, diff.Key, diff.Right, sourceSch.GetValueDescriptor(),
-				m.valueMerger.rightMapping, m.tableMerger, m.finalSch, m.valueMerger.syncPool)
+				m.valueMerger.rightMapping, m.tableMerger, m.finalSch, m.valueMerger.syncPool, true)
 			if err != nil {
 				return err
 			}
@@ -1056,8 +1193,8 @@ func resolveExpression(ctx *sql.Context, expression string, sch schema.Schema, t
 	if err != nil {
 		return nil, err
 	}
-	mockTable := memory.NewTable(tableName, sqlSch, nil)
 	mockDatabase := memory.NewDatabase("mydb")
+	mockTable := memory.NewLocalTable(mockDatabase.BaseDatabase, tableName, sqlSch, nil)
 	mockDatabase.AddTable(tableName, mockTable)
 	mockProvider := memory.NewDBProvider(mockDatabase)
 	catalog := analyzer.NewCatalog(mockProvider)
@@ -1099,16 +1236,18 @@ func remapTuple(tuple val.Tuple, desc val.TupleDesc, mapping val.OrdinalMapping)
 // remapTupleWithColumnDefaults takes the given |tuple| (and the |tupleDesc| that describes how to access its fields)
 // and uses |mapping| to map the tuple's data and return a new tuple. |tm| provides high access to the name of the table
 // currently being merged and associated node store. |mergedSch| is the new schema of the table and is used to look up
-// column default values to apply to any existing rows when a new column is added as part of a merge. |pool| is used to
+// column default values to apply to any existing rows when a new column is added as part of a merge. |pool| is used
 // to allocate memory for the new tuple. A pointer to the new tuple data is returned, along with any error encountered.
-func remapTupleWithColumnDefaults(ctx *sql.Context, keyTuple, valueTuple val.Tuple, tupleDesc val.TupleDesc, mapping val.OrdinalMapping, tm *TableMerger, mergedSch schema.Schema, pool pool.BuffPool) (val.Tuple, error) {
+// The |rightSide| parameter indicates if the tuple came from the right side of the merge; this is needed to determine
+// if the tuple data needs to be converted from the old schema type to a changed schema type.
+func remapTupleWithColumnDefaults(ctx *sql.Context, keyTuple, valueTuple val.Tuple, tupleDesc val.TupleDesc, mapping val.OrdinalMapping, tm *TableMerger, mergedSch schema.Schema, pool pool.BuffPool, rightSide bool) (val.Tuple, error) {
 	tb := val.NewTupleBuilder(mergedSch.GetValueDescriptor())
 
 	for to, from := range mapping {
 		var value interface{}
+		col := mergedSch.GetNonPKCols().GetByIndex(to)
 		if from == -1 {
 			// If the column is a new column, then look up any default value
-			col := mergedSch.GetNonPKCols().GetByIndex(to)
 			if col.Default != "" {
 				// TODO: Not great to reparse the expression for every single row... need to cache this
 				expression, err := resolveExpression(ctx, col.Default, mergedSch, tm.name)
@@ -1139,12 +1278,56 @@ func remapTupleWithColumnDefaults(ctx *sql.Context, keyTuple, valueTuple val.Tup
 				}
 			}
 		} else {
-			tb.PutRaw(to, tupleDesc.GetField(from, valueTuple))
+			value, err := index.GetField(ctx, tupleDesc, from, valueTuple, tm.ns)
+			if err != nil {
+				return nil, err
+			}
+
+			// If the type has changed, then call convert to convert the value to the new type
+			value, err = convertValueToNewType(value, col.TypeInfo, tm, from, rightSide)
+			if err != nil {
+				return nil, err
+			}
+
+			err = index.PutField(ctx, tm.ns, tb, to, value)
+			if err != nil {
+				return nil, err
+			}
 		}
 	}
 	return tb.Build(pool), nil
 }
 
+// convertValueToNewType handles converting a value from a previous type into a new type. |value| is the value from
+// the previous schema, |newTypeInfo| is the type info for the value in the new schema, |tm| is the TableMerger
+// instance that describes how the table is being merged, |from| is the field position in the value tuple from the
+// previous schema, and |rightSide| indicates whether the previous type info can be found on the right side of the merge
+// or the left side. If the previous type info is the same as the current type info for the merged schema, then this
+// function is a no-op and simply returns |value|. The converted value along with any unexpected error encountered is
+// returned.
+func convertValueToNewType(value interface{}, newTypeInfo typeinfo.TypeInfo, tm *TableMerger, from int, rightSide bool) (interface{}, error) {
+	var previousTypeInfo typeinfo.TypeInfo
+	if rightSide {
+		previousTypeInfo = tm.rightSch.GetNonPKCols().GetByIndex(from).TypeInfo
+	} else {
+		previousTypeInfo = tm.leftSch.GetNonPKCols().GetByIndex(from).TypeInfo
+	}
+
+	if newTypeInfo.Equals(previousTypeInfo) {
+		return value, nil
+	}
+
+	// If the type has changed, then call convert to convert the value to the new type
+	newValue, inRange, err := newTypeInfo.ToSqlType().Convert(value)
+	if err != nil {
+		return nil, err
+	}
+	if !inRange {
+		return nil, fmt.Errorf("out of range conversion for value %v to type %s", value, newTypeInfo.String())
+	}
+	return newValue, nil
+}
+
 func mergeTableArtifacts(ctx context.Context, tm *TableMerger, mergeTbl *doltdb.Table) (*doltdb.Table, error) {
 	la, err := tm.leftTbl.GetArtifacts(ctx)
 	if err != nil {
@@ -1191,23 +1374,29 @@ func mergeTableArtifacts(ctx context.Context, tm *TableMerger, mergeTbl *doltdb.
 // a three-way cell edit (tree.DiffOpDivergentModifyResolved).
 type valueMerger struct {
 	numCols                                int
-	vD                                     val.TupleDesc
+	baseVD, rightVD, resultVD              val.TupleDesc
+	resultSchema                           schema.Schema
 	leftMapping, rightMapping, baseMapping val.OrdinalMapping
 	syncPool                               pool.BuffPool
 	keyless                                bool
+	ns                                     tree.NodeStore
 }
 
-func newValueMerger(merged, leftSch, rightSch, baseSch schema.Schema, syncPool pool.BuffPool) *valueMerger {
+func newValueMerger(merged, leftSch, rightSch, baseSch schema.Schema, syncPool pool.BuffPool, ns tree.NodeStore) *valueMerger {
 	leftMapping, rightMapping, baseMapping := generateSchemaMappings(merged, leftSch, rightSch, baseSch)
 
 	return &valueMerger{
 		numCols:      merged.GetNonPKCols().Size(),
-		vD:           merged.GetValueDescriptor(),
+		baseVD:       baseSch.GetValueDescriptor(),
+		rightVD:      rightSch.GetValueDescriptor(),
+		resultVD:     merged.GetValueDescriptor(),
+		resultSchema: merged,
 		leftMapping:  leftMapping,
 		rightMapping: rightMapping,
 		baseMapping:  baseMapping,
 		syncPool:     syncPool,
 		keyless:      schema.IsKeyless(merged),
+		ns:           ns,
 	}
 }
 
@@ -1261,7 +1450,7 @@ func migrateDataToMergedSchema(ctx *sql.Context, tm *TableMerger, vm *valueMerge
 		return err
 	}
 	leftRows := durable.ProllyMapFromIndex(lr)
-	mut := leftRows.Mutate()
+	mut := leftRows.Rewriter(mergedSch.GetKeyDescriptor(), mergedSch.GetValueDescriptor())
 	mapIter, err := mut.IterAll(ctx)
 	if err != nil {
 		return err
@@ -1281,7 +1470,7 @@ func migrateDataToMergedSchema(ctx *sql.Context, tm *TableMerger, vm *valueMerge
 			return err
 		}
 
-		newValueTuple, err := remapTupleWithColumnDefaults(ctx, keyTuple, valueTuple, valueDescriptor, vm.leftMapping, tm, mergedSch, vm.syncPool)
+		newValueTuple, err := remapTupleWithColumnDefaults(ctx, keyTuple, valueTuple, valueDescriptor, vm.leftMapping, tm, mergedSch, vm.syncPool, false)
 		if err != nil {
 			return err
 		}
@@ -1304,14 +1493,17 @@ func migrateDataToMergedSchema(ctx *sql.Context, tm *TableMerger, vm *valueMerge
 	}
 	tm.leftTbl = newTable
 
-	// TODO: for now... we don't actually need to migrate any of the data held in secondary indexes (yet).
-	//       We're currently dealing with column adds/drops/renames/reorders, but none of those directly affect
-	//       secondary indexes. Columns drops *should*, but currently Dolt just drops any index referencing the
-	//       dropped column, so there's nothing to do currently.
+	// NOTE: We don't handle migrating secondary index data to the new schema here. In most cases, a schema change
+	//       won't affect secondary index data, but there are a few cases where we do rebuild the indexes. Dropping
+	//       a column *should* keep the index, but remove that column from it and rebuild it, but Dolt/GMS currently
+	//       drop the index completely if a used column is removed.
 	//       https://github.com/dolthub/dolt/issues/5641
-	//       Once we start handling type changes changes or primary key changes, or fix the bug above,
-	//       then we will need to start migrating secondary index data, too.
-
+	//
+	//       Most of the currently supported type changes for a schema merge don't require rebuilding secondary
+	//       indexes either. The exception is converting a column to BINARY, which requires us to rewrite the data
+	//       and ensure it's right-padded with null bytes, up to the column length. This is handled at the end of
+	//       the table merge. If we detect that the table rows needed to be rewritten, then we'll rebuild all indexes
+	//       on the table, just to be safe.
 	return nil
 }
 
@@ -1319,17 +1511,17 @@ func migrateDataToMergedSchema(ctx *sql.Context, tm *TableMerger, vm *valueMerge
 // tuples. It returns the merged cell value tuple and a bool indicating if a
 // conflict occurred. tryMerge should only be called if left and right produce
 // non-identical diffs against base.
-func (m *valueMerger) tryMerge(left, right, base val.Tuple) (val.Tuple, bool) {
+func (m *valueMerger) tryMerge(ctx context.Context, left, right, base val.Tuple) (val.Tuple, bool, error) {
 	// If we're merging a keyless table and the keys match, but the values are different,
 	// that means that the row data is the same, but the cardinality has changed, and if the
 	// cardinality has changed in different ways on each merge side, we can't auto resolve.
 	if m.keyless {
-		return nil, false
+		return nil, false, nil
 	}
 
 	if base != nil && (left == nil) != (right == nil) {
 		// One row deleted, the other modified
-		return nil, false
+		return nil, false, nil
 	}
 
 	// Because we have non-identical diffs, left and right are guaranteed to be
@@ -1340,52 +1532,142 @@ func (m *valueMerger) tryMerge(left, right, base val.Tuple) (val.Tuple, bool) {
 
 	mergedValues := make([][]byte, m.numCols)
 	for i := 0; i < m.numCols; i++ {
-		v, isConflict := m.processColumn(i, left, right, base)
+		v, isConflict, err := m.processColumn(ctx, i, left, right, base)
+		if err != nil {
+			return nil, false, err
+		}
 		if isConflict {
-			return nil, false
+			return nil, false, nil
 		}
 		mergedValues[i] = v
 	}
 
-	return val.NewTuple(m.syncPool, mergedValues...), true
+	return val.NewTuple(m.syncPool, mergedValues...), true, nil
 }
 
 // processColumn returns the merged value of column |i| of the merged schema,
 // based on the |left|, |right|, and |base| schema.
-func (m *valueMerger) processColumn(i int, left, right, base val.Tuple) ([]byte, bool) {
+func (m *valueMerger) processColumn(ctx context.Context, i int, left, right, base val.Tuple) (result []byte, conflict bool, err error) {
 	// missing columns are coerced into NULL column values
-	var leftCol []byte
-	if l := m.leftMapping[i]; l != -1 {
-		leftCol = left.GetField(l)
+
+	var baseCol []byte
+	var baseColIdx = -1
+	var baseColExists = false
+	if base != nil {
+		baseCol, baseColIdx, baseColExists = getColumn(&base, &m.baseMapping, i)
 	}
-	var rightCol []byte
-	if r := m.rightMapping[i]; r != -1 {
-		rightCol = right.GetField(r)
+	leftCol, leftColIdx, leftColExists := getColumn(&left, &m.leftMapping, i)
+	rightCol, rightColIdx, rightColExists := getColumn(&right, &m.rightMapping, i)
+	resultType := m.resultVD.Types[i]
+
+	// We previously asserted that left and right are not nil.
+	//But base can be nil in the event of convergent inserts.
+	if base == nil || !baseColExists {
+		// There are two possible cases:
+		// - The base row doesn't exist, or
+		// - The column doesn't exist in the base row
+		// Regardless, both left and right are inserts, or one is an insert and the other doesn't exist.
+
+		if !rightColExists {
+			return leftCol, false, nil
+		}
+
+		rightCol, err = convert(ctx, m.rightVD, m.resultVD, m.resultSchema, rightColIdx, i, right, rightCol, m.ns)
+		if err != nil {
+			return nil, false, err
+		}
+
+		if !leftColExists {
+			return rightCol, false, nil
+		}
+
+		if m.resultVD.Comparator().CompareValues(i, leftCol, rightCol, resultType) == 0 {
+			// columns are equal, return either.
+			return leftCol, false, nil
+		}
+		// conflicting inserts
+		return nil, true, nil
 	}
 
-	if m.vD.Comparator().CompareValues(i, leftCol, rightCol, m.vD.Types[i]) == 0 {
-		return leftCol, false
+	// We can now assume that both left are right contain byte-level changes to an existing column.
+	// But we need to know if those byte-level changes represent a modification to the underlying value,
+	// and whether those changes represent the *same* modification, otherwise there's a conflict.
+
+	// We can't just look at the bytes to determine this, because if a cell's byte representation changed,
+	// but only because of a schema change, we shouldn't consider that a conflict.
+	// Conversely, if there was a schema change on only one side, we shouldn't consider the cells equal
+	// even if they have the same bytes.
+
+	// Thus, we must convert all cells to the type in the result schema before comparing them.
+
+	if baseCol != nil {
+		baseCol, err = convert(ctx, m.baseVD, m.resultVD, m.resultSchema, baseColIdx, i, base, baseCol, m.ns)
+		if err != nil {
+			return nil, false, err
+		}
 	}
 
-	if base == nil {
-		// Conflicting insert
-		return nil, true
+	var leftModified, rightModified bool
+
+	if leftColIdx == -1 && rightColIdx == -1 {
+		// Both branches are implicitly NULL
+		return nil, false, err
 	}
 
-	var baseVal []byte
-	if b := m.baseMapping[i]; b != -1 {
-		baseVal = base.GetField(b)
+	if rightColIdx == -1 {
+		// The right branch is implicitly NULL
+		rightModified = baseCol != nil
+	} else {
+		// Attempt to convert the right column to match the result schema, then compare it to the base.
+		rightCol, err = convert(ctx, m.rightVD, m.resultVD, m.resultSchema, rightColIdx, i, right, rightCol, m.ns)
+		if err != nil {
+			return nil, true, nil
+		}
+		rightModified = m.resultVD.Comparator().CompareValues(i, rightCol, baseCol, resultType) != 0
 	}
 
-	leftModified := m.vD.Comparator().CompareValues(i, leftCol, baseVal, m.vD.Types[i]) != 0
-	rightModified := m.vD.Comparator().CompareValues(i, rightCol, baseVal, m.vD.Types[i]) != 0
+	// The left value was previously converted in `migrateDataToMergedSchema`, so we don't need to convert it here.
+	if m.resultVD.Comparator().CompareValues(i, leftCol, rightCol, resultType) == 0 {
+		// columns are equal, return either.
+		return leftCol, false, nil
+	}
+
+	leftModified = m.resultVD.Comparator().CompareValues(i, leftCol, baseCol, resultType) != 0
 
 	switch {
 	case leftModified && rightModified:
-		return nil, true
+		// concurrent modification
+		return nil, true, nil
 	case leftModified:
-		return leftCol, false
+		return leftCol, false, nil
 	default:
-		return rightCol, false
+		return rightCol, false, nil
 	}
 }
+
+func getColumn(tuple *val.Tuple, mapping *val.OrdinalMapping, idx int) (col []byte, colIndex int, exists bool) {
+	colIdx := (*mapping)[idx]
+	if colIdx == -1 {
+		return nil, -1, false
+	}
+	return tuple.GetField(colIdx), colIdx, true
+}
+
+// convert takes the `i`th column in the provided tuple and converts it to the type specified in the provided schema.
+// returns the new representation, and a bool indicating success.
+func convert(ctx context.Context, fromDesc, toDesc val.TupleDesc, toSchema schema.Schema, fromIndex, toIndex int, tuple val.Tuple, originalValue []byte, ns tree.NodeStore) ([]byte, error) {
+	if fromDesc.Types[fromIndex] == toDesc.Types[toIndex] {
+		// No conversion is necessary here.
+		return originalValue, nil
+	}
+	parsedCell, err := index.GetField(ctx, fromDesc, fromIndex, tuple, ns)
+	if err != nil {
+		return nil, err
+	}
+	sqlType := toSchema.GetNonPKCols().GetByIndex(toIndex).TypeInfo.ToSqlType()
+	convertedCell, _, err := sqlType.Convert(parsedCell)
+	if err != nil {
+		return nil, err
+	}
+	return index.Serialize(ctx, ns, toDesc.Types[toIndex], convertedCell)
+}
