@@ -113,7 +113,7 @@ type MergedTable struct {
 
 // MergeTable merges schema and table data for the table tblName.
 // TODO: this code will loop infinitely when merging certain schema changes
-func (rm *RootMerger) MergeTable(ctx context.Context, tblName string, opts editor.Options, mergeOpts MergeOpts) (*MergedTable, *MergeStats, error) {
+func (rm *RootMerger) MergeTable(ctx *sql.Context, tblName string, opts editor.Options, mergeOpts MergeOpts) (*MergedTable, *MergeStats, error) {
 	tm, err := rm.makeTableMerger(ctx, tblName)
 	if err != nil {
 		return nil, nil, err
@@ -126,7 +126,7 @@ func (rm *RootMerger) MergeTable(ctx context.Context, tblName string, opts edito
 	}
 
 	// Calculate a merge of the schemas, but don't apply it yet
-	mergeSch, schConflicts, err := SchemaMerge(ctx, tm.vrw.Format(), tm.leftSch, tm.rightSch, tm.ancSch, tblName)
+	mergeSch, schConflicts, tableRewrite, err := SchemaMerge(ctx, tm.vrw.Format(), tm.leftSch, tm.rightSch, tm.ancSch, tblName)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -148,7 +148,7 @@ func (rm *RootMerger) MergeTable(ctx context.Context, tblName string, opts edito
 
 	var tbl *doltdb.Table
 	if types.IsFormat_DOLT(tm.vrw.Format()) {
-		tbl, stats, err = mergeProllyTable(ctx, tm, mergeSch)
+		tbl, stats, err = mergeProllyTable(ctx, tm, mergeSch, tableRewrite)
 	} else {
 		tbl, stats, err = mergeNomsTable(ctx, tm, mergeSch, rm.vrw, opts)
 	}
@@ -283,55 +283,6 @@ func (rm *RootMerger) maybeShortCircuit(ctx context.Context, tm *TableMerger, op
 	return nil, nil, nil
 }
 
-func validateTupleFields(existingSch schema.Schema, targetSch schema.Schema) (bool, error) {
-	existingVD := existingSch.GetValueDescriptor()
-	targetVD := targetSch.GetValueDescriptor()
-	_, valMapping, err := schema.MapSchemaBasedOnTagAndName(existingSch, targetSch)
-	if err != nil {
-		return false, err
-	}
-
-	for existingIndex, targetIndex := range valMapping {
-		// If the column was dropped, continue to the next column targetIndex
-		if targetIndex == -1 {
-			continue
-		}
-
-		// If the field types have changed between existing and target, bail.
-		if existingVD.Types[existingIndex].Enc != targetVD.Types[targetIndex].Enc {
-			return false, nil
-		}
-
-		// If the collation was changed, bail.
-		// Different collations will affect the ordering of any secondary indexes using this column.
-		existingStr, ok1 := existingSch.GetNonPKCols().GetByIndex(existingIndex).TypeInfo.ToSqlType().(sql.StringType)
-		targetStr, ok2 := targetSch.GetNonPKCols().GetByIndex(targetIndex).TypeInfo.ToSqlType().(sql.StringType)
-
-		if ok1 && ok2 && !existingStr.Collation().Equals(targetStr.Collation()) {
-			return false, nil
-		}
-	}
-
-	_, valMapping, err = schema.MapSchemaBasedOnTagAndName(targetSch, existingSch)
-	if err != nil {
-		return false, err
-	}
-
-	for targetIndex, existingIndex := range valMapping {
-		if targetIndex == existingIndex {
-			continue
-		}
-
-		col := targetSch.GetNonPKCols().GetByIndex(targetIndex)
-		// If we haven't bailed so far, then these fields were added at the end.
-		// If one of these fields is NOT NULL, without a default value, then fail.
-		if !col.IsNullable() && col.Default == "" {
-			return false, nil
-		}
-	}
-	return true, nil
-}
-
 func setConflicts(ctx context.Context, cons durable.ConflictIndex, tbl, mergeTbl, ancTbl, tableToUpdate *doltdb.Table) (*doltdb.Table, error) {
 	ancSch, err := ancTbl.GetSchema(ctx)
 	if err != nil {
