@@ -21,8 +21,9 @@ import (
 	"sort"
 	"strings"
 
-	sqle "github.com/dolthub/go-mysql-server"
 	"github.com/dolthub/go-mysql-server/sql"
+	ast "github.com/dolthub/vitess/go/vt/sqlparser"
+	errorkinds "gopkg.in/src-d/go-errors.v1"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
@@ -42,6 +43,10 @@ const (
 	DuplicateIndexColumnSet
 )
 
+var ErrUnmergeableNewColumn = errorkinds.NewKind("Unable to merge new column `%s` in table `%s` because it is not-nullable and has no default value, so existing rows can't be updated automatically. To complete this merge, either manually add this new column to the target branch of the merge and update any existing rows, or change the column's definition on the other branch of the merge so that it is nullable or has a default value.")
+
+var ErrDefaultCollationConflict = errorkinds.NewKind("Unable to merge table '%s', because its default collation setting has changed on both sides of the merge. Manually change the table's default collation setting on one of the sides of the merge and retry this merge.")
+
 type SchemaConflict struct {
 	TableName    string
 	ColConflicts []ColConflict
@@ -147,8 +152,10 @@ func (c ChkConflict) String() string {
 
 var ErrMergeWithDifferentPks = errors.New("error: cannot merge two tables with different primary keys")
 
-// SchemaMerge performs a three-way merge of ourSch, theirSch, and ancSch.
-func SchemaMerge(ctx context.Context, format *storetypes.NomsBinFormat, ourSch, theirSch, ancSch schema.Schema, tblName string) (sch schema.Schema, sc SchemaConflict, err error) {
+// SchemaMerge performs a three-way merge of |ourSch|, |theirSch|, and |ancSch|, and returns: the merged schema,
+// any schema conflicts identified, whether moving to the new schema requires a full table rewrite, and any
+// unexpected error encountered while merging the schemas.
+func SchemaMerge(ctx context.Context, format *storetypes.NomsBinFormat, ourSch, theirSch, ancSch schema.Schema, tblName string) (sch schema.Schema, sc SchemaConflict, tableRewrite bool, err error) {
 	// (sch - ancSch) ∪ (mergeSch - ancSch) ∪ (sch ∩ mergeSch)
 	sc = SchemaConflict{
 		TableName: tblName,
@@ -157,33 +164,38 @@ func SchemaMerge(ctx context.Context, format *storetypes.NomsBinFormat, ourSch,
 	// TODO: We'll remove this once it's possible to get diff and merge on different primary key sets
 	// TODO: decide how to merge different orders of PKS
 	if !schema.ArePrimaryKeySetsDiffable(format, ourSch, theirSch) || !schema.ArePrimaryKeySetsDiffable(format, ourSch, ancSch) {
-		return nil, SchemaConflict{}, ErrMergeWithDifferentPks
+		return nil, SchemaConflict{}, false, ErrMergeWithDifferentPks
 	}
 
 	var mergedCC *schema.ColCollection
-	mergedCC, sc.ColConflicts, err = mergeColumns(format, ourSch.GetAllCols(), theirSch.GetAllCols(), ancSch.GetAllCols())
+	mergedCC, sc.ColConflicts, tableRewrite, err = mergeColumns(tblName, format, ourSch.GetAllCols(), theirSch.GetAllCols(), ancSch.GetAllCols())
 	if err != nil {
-		return nil, SchemaConflict{}, err
+		return nil, SchemaConflict{}, false, err
 	}
 	if len(sc.ColConflicts) > 0 {
-		return nil, sc, nil
+		return nil, sc, tableRewrite, nil
 	}
 
 	var mergedIdxs schema.IndexCollection
 	mergedIdxs, sc.IdxConflicts = mergeIndexes(mergedCC, ourSch, theirSch, ancSch)
 	if len(sc.IdxConflicts) > 0 {
-		return nil, sc, nil
+		return nil, sc, tableRewrite, nil
 	}
 
 	sch, err = schema.SchemaFromCols(mergedCC)
 	if err != nil {
-		return nil, sc, err
+		return nil, sc, false, err
+	}
+
+	sch, err = mergeTableCollation(ctx, tblName, ancSch, ourSch, theirSch, sch)
+	if err != nil {
+		return nil, sc, false, err
 	}
 
 	// TODO: Merge conflict should have blocked any primary key ordinal changes
 	err = sch.SetPkOrdinals(ourSch.GetPkOrdinals())
 	if err != nil {
-		return nil, sc, err
+		return nil, sc, false, err
 	}
 
 	_ = mergedIdxs.Iter(func(index schema.Index) (stop bool, err error) {
@@ -195,17 +207,17 @@ func SchemaMerge(ctx context.Context, format *storetypes.NomsBinFormat, ourSch,
 	var mergedChks []schema.Check
 	mergedChks, sc.ChkConflicts, err = mergeChecks(ctx, ourSch.Checks(), theirSch.Checks(), ancSch.Checks())
 	if err != nil {
-		return nil, SchemaConflict{}, err
+		return nil, SchemaConflict{}, false, err
 	}
 	if len(sc.ChkConflicts) > 0 {
-		return nil, sc, nil
+		return nil, sc, false, nil
 	}
 
 	// Look for invalid CHECKs
 	for _, chk := range mergedChks {
 		// CONFLICT: a CHECK now references a column that no longer exists in schema
 		if ok, err := isCheckReferenced(sch, chk); err != nil {
-			return nil, sc, err
+			return nil, sc, false, err
 		} else if !ok {
 			// Append to conflicts
 			sc.ChkConflicts = append(sc.ChkConflicts, ChkConflict{
@@ -220,7 +232,7 @@ func SchemaMerge(ctx context.Context, format *storetypes.NomsBinFormat, ourSch,
 		sch.Checks().AddCheck(chk.Name(), chk.Expression(), chk.Enforced())
 	}
 
-	return sch, sc, nil
+	return sch, sc, tableRewrite, nil
 }
 
 // ForeignKeysMerge performs a three-way merge of (ourRoot, theirRoot, ancRoot) and using mergeRoot to validate FKs.
@@ -288,6 +300,12 @@ func ForeignKeysMerge(ctx context.Context, mergedRoot, ourRoot, theirRoot, ancRo
 	})
 
 	err = ourNewFKs.Iter(func(ourFK doltdb.ForeignKey) (stop bool, err error) {
+		// The common set of FKs may already have this FK, if it was added on both branches
+		if commonFK, ok := common.GetByNameCaseInsensitive(ourFK.Name); ok && commonFK.EqualDefs(ourFK) {
+			// Skip this one if it's identical to the one in the common set
+			return false, nil
+		}
+
 		return false, common.AddKeys(ourFK)
 	})
 	if err != nil {
@@ -295,6 +313,12 @@ func ForeignKeysMerge(ctx context.Context, mergedRoot, ourRoot, theirRoot, ancRo
 	}
 
 	err = theirNewFKs.Iter(func(theirFK doltdb.ForeignKey) (stop bool, err error) {
+		// The common set of FKs may already have this FK, if it was added on both branches
+		if commonFK, ok := common.GetByNameCaseInsensitive(theirFK.Name); ok && commonFK.EqualDefs(theirFK) {
+			// Skip this one if it's identical to the one in the common set
+			return false, nil
+		}
+
 		return false, common.AddKeys(theirFK)
 	})
 	if err != nil {
@@ -309,26 +333,62 @@ func ForeignKeysMerge(ctx context.Context, mergedRoot, ourRoot, theirRoot, ancRo
 	return common, conflicts, err
 }
 
+// checkUnmergeableNewColumns checks the |columnMappings| to see if a new column has been added that does not allow
+// NULL values and has no default value, and returns an error if one is found. New, non-nullable columns that don't
+// have a default value cannot be merged automatically, since we don't know what value to set for any existing rows,
+// so instead of reporting a schema conflict and allowing customers to use the conflict resolution workflow, we return
+// an error and instruct them how to manually fix the problem.
+func checkUnmergeableNewColumns(tblName string, columnMappings columnMappings) error {
+	for _, mapping := range columnMappings {
+		anc := mapping.anc
+		ours := mapping.ours
+		theirs := mapping.theirs
+
+		// if a new column was added...
+		if anc == nil && ((ours == nil && theirs != nil) || (ours != nil && theirs == nil)) {
+			newCol := ours
+			if newCol == nil {
+				newCol = theirs
+			}
+
+			// If the new column is not nullable and has no default value, then we can't auto merge it
+			// (if there is any existing row data), so we need to error out and report the schema conflict.
+			if newCol.IsNullable() == false && newCol.Default == "" {
+				return ErrUnmergeableNewColumn.New(newCol, tblName)
+			}
+		}
+	}
+	return nil
+}
+
 // mergeColumns merges the columns from |ourCC|, |theirCC| into a single column collection, using the ancestor column
 // definitions in |ancCC| to determine on which side a column has changed. If merging is not possible because of
 // conflicting changes to the columns in |ourCC| and |theirCC|, then a set of ColConflict instances are returned
 // describing the conflicts. |format| indicates what storage format is in use, and is needed to determine compatibility
 // between types, since different storage formats have different restrictions on how much types can change and remain
-// compatible with the current stored format. If any unexpected error occurs, then that error is returned and the
-// other response fields should be ignored.
-func mergeColumns(format *storetypes.NomsBinFormat, ourCC, theirCC, ancCC *schema.ColCollection) (*schema.ColCollection, []ColConflict, error) {
+// compatible with the current stored format. The merged columns, any column conflicts, and a boolean value stating if
+// a full table rewrite is needed to align the existing table rows with the new, merged schema. If any unexpected error
+// occurs, then that error is returned and the other response fields should be ignored.
+func mergeColumns(tblName string, format *storetypes.NomsBinFormat, ourCC, theirCC, ancCC *schema.ColCollection) (*schema.ColCollection, []ColConflict, bool, error) {
 	columnMappings, err := mapColumns(ourCC, theirCC, ancCC)
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, false, err
 	}
 
 	conflicts, err := checkSchemaConflicts(columnMappings)
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, false, err
+	}
+
+	err = checkUnmergeableNewColumns(tblName, columnMappings)
+	if err != nil {
+		return nil, nil, false, err
 	}
 
 	compatChecker := newTypeCompatabilityCheckerForStorageFormat(format)
 
+	tableRewrite := false
+
 	// After we've checked for schema conflicts, merge the columns together
 	// TODO: We don't currently preserve all column position changes; the returned merged columns are always based on
 	//	     their position in |ourCC|, with any new columns from |theirCC| added at the end of the column collection.
@@ -355,11 +415,19 @@ func mergeColumns(format *storetypes.NomsBinFormat, ourCC, theirCC, ancCC *schem
 				oursChanged := !anc.Equals(*ours)
 				theirsChanged := !anc.Equals(*theirs)
 				if oursChanged && theirsChanged {
-					// This is a schema change conflict and has already been handled by checkSchemaConflicts
+					// If both columns changed in the same way, the modifications converge, so accept the column.
+					// If not, don't report a conflict, since this case is already handled in checkForColumnConflicts.
+					if ours.Equals(*theirs) {
+						mergedColumns = append(mergedColumns, *theirs)
+					}
 				} else if theirsChanged {
 					// In this case, only theirsChanged, so we need to check if moving from ours->theirs
 					// is valid, otherwise it's a conflict
-					if compatChecker.IsTypeChangeCompatible(ours.TypeInfo, theirs.TypeInfo) {
+					compatible, rewrite := compatChecker.IsTypeChangeCompatible(ours.TypeInfo, theirs.TypeInfo)
+					if rewrite {
+						tableRewrite = true
+					}
+					if compatible {
 						mergedColumns = append(mergedColumns, *theirs)
 					} else {
 						conflicts = append(conflicts, ColConflict{
@@ -371,7 +439,11 @@ func mergeColumns(format *storetypes.NomsBinFormat, ourCC, theirCC, ancCC *schem
 				} else if oursChanged {
 					// In this case, only oursChanged, so we need to check if moving from theirs->ours
 					// is valid, otherwise it's a conflict
-					if compatChecker.IsTypeChangeCompatible(theirs.TypeInfo, ours.TypeInfo) {
+					compatible, rewrite := compatChecker.IsTypeChangeCompatible(theirs.TypeInfo, ours.TypeInfo)
+					if rewrite {
+						tableRewrite = true
+					}
+					if compatible {
 						mergedColumns = append(mergedColumns, *ours)
 					} else {
 						conflicts = append(conflicts, ColConflict{
@@ -384,9 +456,12 @@ func mergeColumns(format *storetypes.NomsBinFormat, ourCC, theirCC, ancCC *schem
 					// if neither side changed, just use ours
 					mergedColumns = append(mergedColumns, *ours)
 				}
-			} else if ours.Equals(*theirs) {
-				// if the columns are identical, just use ours
-				mergedColumns = append(mergedColumns, *ours)
+			} else {
+				// If both columns changed in the same way, the modifications converge, so accept the column.
+				// If not, don't report a conflict, since this case is already handled in checkForColumnConflicts.
+				if ours.Equals(*theirs) {
+					mergedColumns = append(mergedColumns, *ours)
+				}
 			}
 		}
 	}
@@ -394,10 +469,10 @@ func mergeColumns(format *storetypes.NomsBinFormat, ourCC, theirCC, ancCC *schem
 	// Check that there are no duplicate column names or tags in the merged column set
 	conflicts = append(conflicts, checkForColumnConflicts(mergedColumns)...)
 	if conflicts != nil {
-		return nil, conflicts, nil
+		return nil, conflicts, false, nil
 	}
 
-	return schema.NewColCollection(mergedColumns...), nil, nil
+	return schema.NewColCollection(mergedColumns...), nil, tableRewrite, nil
 }
 
 // checkForColumnConflicts iterates over |mergedColumns|, checks for duplicate column names or column tags, and returns
@@ -958,7 +1033,7 @@ func checksInCommon(ourChks, theirChks, ancChks []schema.Check) ([]schema.Check,
 
 		// NO CONFLICT: CHECK was only modified in their branch, so update check definition with theirs
 		if ancChk == ourChk {
-			common = append(common, ourChk)
+			common = append(common, theirChk)
 			continue
 		}
 
@@ -1031,6 +1106,25 @@ func chkCollectionModified(anc, child []schema.Check) []schema.Check {
 	return result
 }
 
+// mergeTableCollation checks how the table's default collation setting has changed from |ancSch| to |ourSch|, as
+// well as from |ancSch| to |theirSch|, and then sets the collation in |mergedSch| and returns it. If the default
+// table collation setting was changed on both sides of the merge (to different collations), then an error is returned.
+func mergeTableCollation(_ context.Context, tblName string, ancSch, ourSch, theirSch, mergedSch schema.Schema) (schema.Schema, error) {
+	// Update the default charset/collation setting if it changed on only one side
+	ourCollationChanged := ancSch != nil && ancSch.GetCollation() != ourSch.GetCollation()
+	theirCollationChanged := ancSch != nil && ancSch.GetCollation() != theirSch.GetCollation()
+
+	if ourCollationChanged && theirCollationChanged && ourSch.GetCollation() != theirSch.GetCollation() {
+		return nil, ErrDefaultCollationConflict.New(tblName)
+	}
+	mergedSch.SetCollation(ourSch.GetCollation())
+	if theirCollationChanged {
+		mergedSch.SetCollation(theirSch.GetCollation())
+	}
+
+	return mergedSch, nil
+}
+
 // mergeChecks attempts to combine ourChks, theirChks, and ancChks into a single collection, or gathers the conflicts
 func mergeChecks(ctx context.Context, ourChks, theirChks, ancChks schema.CheckCollection) ([]schema.Check, []ChkConflict, error) {
 	// Handles modifications
@@ -1073,7 +1167,7 @@ func mergeChecks(ctx context.Context, ourChks, theirChks, ancChks schema.CheckCo
 			CheckExpression: chk.Expression(),
 			Enforced:        chk.Enforced(),
 		}
-		colNames, err := sqle.ColumnsFromCheckDefinition(sql.NewContext(ctx), &chkDef)
+		colNames, err := ColumnsFromCheckDefinition(sql.NewContext(ctx), &chkDef)
 		if err != nil {
 			return nil, nil, err
 		}
@@ -1095,7 +1189,7 @@ func mergeChecks(ctx context.Context, ourChks, theirChks, ancChks schema.CheckCo
 			CheckExpression: ourChk.Expression(),
 			Enforced:        ourChk.Enforced(),
 		}
-		colNames, err := sqle.ColumnsFromCheckDefinition(sql.NewContext(ctx), &chkDef)
+		colNames, err := ColumnsFromCheckDefinition(sql.NewContext(ctx), &chkDef)
 		if err != nil {
 			return nil, nil, err
 		}
@@ -1186,7 +1280,7 @@ func isCheckReferenced(sch schema.Schema, chk schema.Check) (bool, error) {
 		CheckExpression: chk.Expression(),
 		Enforced:        chk.Enforced(),
 	}
-	colNames, err := sqle.ColumnsFromCheckDefinition(sql.NewEmptyContext(), &chkDef)
+	colNames, err := ColumnsFromCheckDefinition(sql.NewEmptyContext(), &chkDef)
 	if err != nil {
 		return false, err
 	}
@@ -1200,3 +1294,39 @@ func isCheckReferenced(sch schema.Schema, chk schema.Check) (bool, error) {
 
 	return true, nil
 }
+
+// ColumnsFromCheckDefinition retrieves the Column Names referenced by a CheckDefinition
+func ColumnsFromCheckDefinition(ctx *sql.Context, def *sql.CheckDefinition) ([]string, error) {
+	// Evaluate the CheckDefinition to get evaluated Expression
+	parseStr := fmt.Sprintf("select %s", def.CheckExpression)
+	parsed, err := ast.Parse(parseStr)
+	if err != nil {
+		return nil, err
+	}
+
+	selectStmt, ok := parsed.(*ast.Select)
+	if !ok || len(selectStmt.SelectExprs) != 1 {
+		err := sql.ErrInvalidCheckConstraint.New(def.CheckExpression)
+		return nil, err
+	}
+
+	expr := selectStmt.SelectExprs[0]
+	ae, ok := expr.(*ast.AliasedExpr)
+	if !ok {
+		err := sql.ErrInvalidCheckConstraint.New(def.CheckExpression)
+		return nil, err
+	}
+
+	// Look for any column references in the evaluated Expression
+	var cols []string
+	ast.Walk(func(n ast.SQLNode) (kontinue bool, err error) {
+		switch n := n.(type) {
+		case *ast.ColName:
+			colName := n.Name.Lowered()
+			cols = append(cols, colName)
+		default:
+		}
+		return true, nil
+	}, ae.Expr)
+	return cols, nil
+}
