@@ -211,6 +211,8 @@ func TestRowMerge(t *testing.T) {
 		t.Skip()
 	}
 
+	ctx := context.Background()
+
 	tests := make([]rowMergeTest, len(testCases))
 	for i, t := range testCases {
 		tests[i] = createRowMergeStruct(t)
@@ -218,9 +220,10 @@ func TestRowMerge(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			v := newValueMerger(test.mergedSch, test.leftSch, test.rightSch, test.baseSch, syncPool)
+			v := newValueMerger(test.mergedSch, test.leftSch, test.rightSch, test.baseSch, syncPool, nil)
 
-			merged, ok := v.tryMerge(test.row, test.mergeRow, test.ancRow)
+			merged, ok, err := v.tryMerge(ctx, test.row, test.mergeRow, test.ancRow)
+			assert.NoError(t, err)
 			assert.Equal(t, test.expectConflict, !ok)
 			vD := test.mergedSch.GetValueDescriptor()
 			assert.Equal(t, vD.Format(test.expectedResult), vD.Format(merged))
