@@ -20,7 +20,8 @@ import (
 
 	fb "github.com/dolthub/flatbuffers/v23/go"
 	"github.com/dolthub/go-mysql-server/sql"
-	"github.com/dolthub/go-mysql-server/sql/parse"
+	"github.com/dolthub/go-mysql-server/sql/planbuilder"
+	sqltypes "github.com/dolthub/go-mysql-server/sql/types"
 
 	"github.com/dolthub/dolt/go/gen/fb/serial"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
@@ -205,7 +206,8 @@ func serializeSchemaColumns(b *fb.Builder, sch schema.Schema) fb.UOffsetT {
 		col := cols[i]
 		co := b.CreateString(col.Comment)
 		do := b.CreateString(col.Default)
-		to := b.CreateString(sqlTypeString(col.TypeInfo))
+		typeString := sqlTypeString(col.TypeInfo)
+		to := b.CreateString(typeString)
 		no := b.CreateString(col.Name)
 
 		serial.ColumnStart(b)
@@ -287,7 +289,7 @@ func deserializeColumns(ctx context.Context, s *serial.TableSchema) ([]schema.Co
 	c := serial.Column{}
 	for i := range cols {
 		s.Columns(&c, i)
-		sqlType, err := typeinfoFromSqlType(ctx, string(c.SqlType()))
+		sqlType, err := typeinfoFromSqlType(string(c.SqlType()))
 		if err != nil {
 			return nil, err
 		}
@@ -342,6 +344,11 @@ func serializeSecondaryIndexes(b *fb.Builder, sch schema.Schema, indexes []schem
 		}
 		po := b.EndVector(len(prefixLengths))
 
+		var ftInfo fb.UOffsetT
+		if idx.IsFullText() {
+			ftInfo = serializeFullTextInfo(b, idx)
+		}
+
 		serial.IndexStart(b)
 		serial.IndexAddName(b, no)
 		serial.IndexAddComment(b, co)
@@ -352,6 +359,10 @@ func serializeSecondaryIndexes(b *fb.Builder, sch schema.Schema, indexes []schem
 		serial.IndexAddSystemDefined(b, !idx.IsUserDefined())
 		serial.IndexAddPrefixLengths(b, po)
 		serial.IndexAddSpatialKey(b, idx.IsSpatial())
+		serial.IndexAddFulltextKey(b, idx.IsFullText())
+		if idx.IsFullText() {
+			serial.IndexAddFulltextInfo(b, ftInfo)
+		}
 		offs[i] = serial.IndexEnd(b)
 	}
 
@@ -371,10 +382,12 @@ func deserializeSecondaryIndexes(sch schema.Schema, s *serial.TableSchema) error
 
 		name := string(idx.Name())
 		props := schema.IndexProperties{
-			IsUnique:      idx.UniqueKey(),
-			IsSpatial:     idx.SpatialKey(),
-			IsUserDefined: !idx.SystemDefined(),
-			Comment:       string(idx.Comment()),
+			IsUnique:           idx.UniqueKey(),
+			IsSpatial:          idx.SpatialKey(),
+			IsFullText:         idx.FulltextKey(),
+			IsUserDefined:      !idx.SystemDefined(),
+			Comment:            string(idx.Comment()),
+			FullTextProperties: deserializeFullTextInfo(&idx),
 		}
 
 		tags := make([]uint64, idx.IndexColumnsLength())
@@ -433,6 +446,62 @@ func deserializeChecks(sch schema.Schema, s *serial.TableSchema) error {
 	return nil
 }
 
+func serializeFullTextInfo(b *fb.Builder, idx schema.Index) fb.UOffsetT {
+	props := idx.FullTextProperties()
+
+	configTable := b.CreateString(props.ConfigTable)
+	posTable := b.CreateString(props.PositionTable)
+	docCountTable := b.CreateString(props.DocCountTable)
+	globalCountTable := b.CreateString(props.GlobalCountTable)
+	rowCountTable := b.CreateString(props.RowCountTable)
+	keyName := b.CreateString(props.KeyName)
+
+	keyPositions := idx.FullTextProperties().KeyPositions
+	serial.FulltextInfoStartKeyPositionsVector(b, len(keyPositions))
+	for j := len(keyPositions) - 1; j >= 0; j-- {
+		b.PrependUint16(keyPositions[j])
+	}
+	keyPos := b.EndVector(len(keyPositions))
+
+	serial.FulltextInfoStart(b)
+	serial.FulltextInfoAddConfigTable(b, configTable)
+	serial.FulltextInfoAddPositionTable(b, posTable)
+	serial.FulltextInfoAddDocCountTable(b, docCountTable)
+	serial.FulltextInfoAddGlobalCountTable(b, globalCountTable)
+	serial.FulltextInfoAddRowCountTable(b, rowCountTable)
+	serial.FulltextInfoAddKeyType(b, props.KeyType)
+	serial.FulltextInfoAddKeyName(b, keyName)
+	serial.FulltextInfoAddKeyPositions(b, keyPos)
+	return serial.FulltextInfoEnd(b)
+}
+
+func deserializeFullTextInfo(idx *serial.Index) schema.FullTextProperties {
+	fulltext := serial.FulltextInfo{}
+	if idx.FulltextInfo(&fulltext) == nil {
+		return schema.FullTextProperties{}
+	}
+
+	var keyPositions []uint16
+	keyPositionsLength := fulltext.KeyPositionsLength()
+	if keyPositionsLength > 0 {
+		keyPositions = make([]uint16, keyPositionsLength)
+		for j := range keyPositions {
+			keyPositions[j] = fulltext.KeyPositions(j)
+		}
+	}
+
+	return schema.FullTextProperties{
+		ConfigTable:      string(fulltext.ConfigTable()),
+		PositionTable:    string(fulltext.PositionTable()),
+		DocCountTable:    string(fulltext.DocCountTable()),
+		GlobalCountTable: string(fulltext.GlobalCountTable()),
+		RowCountTable:    string(fulltext.RowCountTable()),
+		KeyType:          fulltext.KeyType(),
+		KeyName:          string(fulltext.KeyName()),
+		KeyPositions:     keyPositions,
+	}
+}
+
 func keylessSerialSchema(s *serial.TableSchema) bool {
 	n := s.ColumnsLength()
 	if n < 2 {
@@ -468,15 +537,26 @@ func sqlTypeString(t typeinfo.TypeInfo) string {
 			return fmt.Sprintf("%s SRID %d", typ.String(), srid)
 		}
 	}
+
+	// For datetime types, always store the precision explicitly so that it can be read back precisely, although MySQL
+	// omits the precision when it's 0 (the default).
+	if sqltypes.IsDatetimeType(typ) || sqltypes.IsTimestampType(typ) {
+		dt := typ.(sql.DatetimeType)
+		if dt.Precision() == 0 {
+			return fmt.Sprintf("%s(0)", typ.String())
+		}
+		return typ.String()
+	}
+
 	return typ.String()
 }
 
-func typeinfoFromSqlType(ctx context.Context, s string) (typeinfo.TypeInfo, error) {
-	t, err := parse.ParseColumnTypeString(sql.NewContext(ctx), s)
+func typeinfoFromSqlType(s string) (typeinfo.TypeInfo, error) {
+	sqlType, err := planbuilder.ParseColumnTypeString(s)
 	if err != nil {
 		return nil, err
 	}
-	return typeinfo.FromSqlType(t)
+	return typeinfo.FromSqlType(sqlType)
 }
 
 func encodingFromTypeinfo(t typeinfo.TypeInfo) serial.Encoding {
