@@ -87,6 +87,9 @@ type Schema interface {
 
 	// SetCollation sets the table's collation.
 	SetCollation(collation Collation)
+
+	// Copy returns a copy of this Schema that can be safely modified independently.
+	Copy() Schema
 }
 
 // ColumnOrder is used in ALTER TABLE statements to change the order of inserted / modified columns.
@@ -137,6 +140,22 @@ func HasAutoIncrement(sch Schema) (ok bool) {
 	return
 }
 
+// GetAutoIncrementColumn returns the auto increment column if one exists, with an existence boolean
+func GetAutoIncrementColumn(sch Schema) (col Column, ok bool) {
+	var aiCol Column
+	var found bool
+	_ = sch.GetAllCols().Iter(func(tag uint64, col Column) (stop bool, err error) {
+		if col.AutoIncrement {
+			aiCol = col
+			found = true
+			stop = true
+		}
+		return
+	})
+
+	return aiCol, found
+}
+
 // SchemasAreEqual tests equality of two schemas.
 func SchemasAreEqual(sch1, sch2 Schema) bool {
 	if sch1 == nil && sch2 == nil {
@@ -282,8 +301,8 @@ func MapSchemaBasedOnTagAndName(inSch, outSch Schema) ([]int, []int, error) {
 
 	err := inSch.GetPKCols().Iter(func(tag uint64, col Column) (stop bool, err error) {
 		i := inSch.GetPKCols().TagToIdx[tag]
-		if col, ok := outSch.GetPKCols().GetByTag(tag); ok {
-			j := outSch.GetPKCols().TagToIdx[col.Tag]
+		if foundCol, ok := outSch.GetPKCols().GetByTag(tag); ok {
+			j := outSch.GetPKCols().TagToIdx[foundCol.Tag]
 			keyMapping[i] = j
 		} else {
 			return true, fmt.Errorf("could not map primary key column %s", col.Name)
