@@ -65,6 +65,8 @@ func GetTypeConverter(ctx context.Context, srcTi TypeInfo, destTi TypeInfo) (tc
 		return enumTypeConverter(ctx, src, destTi)
 	case *floatType:
 		return floatTypeConverter(ctx, src, destTi)
+	case *geomcollType:
+		return geomcollTypeConverter(ctx, src, destTi)
 	case *geometryType:
 		return geometryTypeConverter(ctx, src, destTi)
 	case *inlineBlobType:
@@ -75,6 +77,12 @@ func GetTypeConverter(ctx context.Context, srcTi TypeInfo, destTi TypeInfo) (tc
 		return jsonTypeConverter(ctx, src, destTi)
 	case *linestringType:
 		return linestringTypeConverter(ctx, src, destTi)
+	case *multilinestringType:
+		return multilinestringTypeConverter(ctx, src, destTi)
+	case *multipointType:
+		return multipointTypeConverter(ctx, src, destTi)
+	case *multipolygonType:
+		return multipolygonTypeConverter(ctx, src, destTi)
 	case *pointType:
 		return pointTypeConverter(ctx, src, destTi)
 	case *polygonType:
@@ -142,11 +150,15 @@ func wrapConvertValueToNomsValue(
 				return nil, err
 			}
 		case types.LineString:
-			vInt = ConvertTypesLineStringToSQLLineString(val)
+			vInt = types.ConvertTypesLineStringToSQLLineString(val)
 		case types.Point:
-			vInt = ConvertTypesPointToSQLPoint(val)
+			vInt = types.ConvertTypesPointToSQLPoint(val)
 		case types.Polygon:
-			vInt = ConvertTypesPolygonToSQLPolygon(val)
+			vInt = types.ConvertTypesPolygonToSQLPolygon(val)
+		case types.MultiPoint:
+			vInt = types.ConvertTypesMultiPointToSQLMultiPoint(val)
+		case types.MultiLineString:
+			vInt = types.ConvertTypesMultiLineStringToSQLMultiLineString(val)
 		case types.String:
 			vInt = string(val)
 		case types.Timestamp:
