@@ -26,7 +26,7 @@ import (
 	"github.com/dolthub/go-mysql-server/sql"
 	"github.com/dolthub/go-mysql-server/sql/binlogreplication"
 	"github.com/dolthub/go-mysql-server/sql/mysql_db"
-	"github.com/dolthub/go-mysql-server/sql/parse"
+	"github.com/dolthub/go-mysql-server/sql/planbuilder"
 	"github.com/dolthub/go-mysql-server/sql/rowexec"
 	"github.com/dolthub/go-mysql-server/sql/types"
 	"github.com/dolthub/vitess/go/mysql"
@@ -37,7 +37,6 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
-	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/globalstate"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/writer"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqlserver"
 )
@@ -747,7 +746,7 @@ func getTableWriter(ctx *sql.Context, engine *gms.Engine, tableName, databaseNam
 		return nil, nil, err
 	}
 
-	tracker, err := globalstate.NewAutoIncrementTracker(ctx, ws)
+	tracker, err := dsess.NewAutoIncrementTracker(ctx, sqlDatabase.Name(), ws)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -871,16 +870,16 @@ func convertVitessJsonExpressionString(ctx *sql.Context, value sqltypes.Value) (
 		strValue = strValue[len("EXPRESSION(") : len(strValue)-1]
 	}
 
-	node, err := parse.Parse(ctx, "SELECT "+strValue)
-	if err != nil {
-		return nil, err
-	}
-
 	server, _ := sqlserver.GetRunningServer()
 	if server == nil {
 		return nil, fmt.Errorf("unable to access running SQL server")
 	}
 
+	node, err := planbuilder.Parse(ctx, server.Engine.Analyzer.Catalog, "SELECT "+strValue)
+	if err != nil {
+		return nil, err
+	}
+
 	analyze, err := server.Engine.Analyzer.Analyze(ctx, node, nil)
 	if err != nil {
 		return nil, err
