@@ -30,12 +30,18 @@ import (
 	"sync"
 	"time"
 
+	"github.com/cenkalti/backoff/v4"
 	"github.com/dolthub/go-mysql-server/sql"
+	"github.com/dolthub/go-mysql-server/sql/mysql_db"
 	gmstypes "github.com/dolthub/go-mysql-server/sql/types"
 	"github.com/sirupsen/logrus"
 	"google.golang.org/grpc"
+	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/credentials"
+	"google.golang.org/grpc/status"
 
+	replicationapi "github.com/dolthub/dolt/go/gen/proto/dolt/services/replicationapi/v1alpha1"
+	"github.com/dolthub/dolt/go/libraries/doltcore/branch_control"
 	"github.com/dolthub/dolt/go/libraries/doltcore/creds"
 	"github.com/dolthub/dolt/go/libraries/doltcore/dbfactory"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
@@ -45,6 +51,7 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/clusterdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
 	"github.com/dolthub/dolt/go/libraries/utils/config"
+	"github.com/dolthub/dolt/go/libraries/utils/filesys"
 	"github.com/dolthub/dolt/go/libraries/utils/jwtauth"
 	"github.com/dolthub/dolt/go/store/types"
 )
@@ -57,6 +64,15 @@ const RoleDetectedBrokenConfig Role = "detected_broken_config"
 
 const PersistentConfigPrefix = "sqlserver.cluster"
 
+// State for any ongoing DROP DATABASE replication attempts we have
+// outstanding. When we create a database, we cancel all on going DROP DATABASE
+// replication attempts.
+type databaseDropReplication struct {
+	ctx    context.Context
+	cancel func()
+	wg     *sync.WaitGroup
+}
+
 type Controller struct {
 	cfg           Config
 	persistentCfg config.ReadWriteConfig
@@ -69,16 +85,30 @@ type Controller struct {
 	cinterceptor  clientinterceptor
 	lgr           *logrus.Logger
 
-	provider       dbProvider
-	iterSessions   IterSessions
-	killQuery      func(uint32)
-	killConnection func(uint32) error
+	standbyCallback IsStandbyCallback
+	iterSessions    IterSessions
+	killQuery       func(uint32)
+	killConnection  func(uint32) error
 
 	jwks      *jwtauth.MultiJWKS
 	tlsCfg    *tls.Config
 	grpcCreds credentials.PerRPCCredentials
 	pub       ed25519.PublicKey
 	priv      ed25519.PrivateKey
+
+	replicationClients []*replicationServiceClient
+
+	mysqlDb          *mysql_db.MySQLDb
+	mysqlDbPersister *replicatingMySQLDbPersister
+	mysqlDbReplicas  []*mysqlDbReplica
+
+	branchControlController *branch_control.Controller
+	branchControlFilesys    filesys.Filesys
+	bcReplication           *branchControlReplication
+
+	dropDatabase             func(*sql.Context, string) error
+	outstandingDropDatabases map[string]*databaseDropReplication
+	remoteSrvDBCache         remotesrv.DBCache
 }
 
 type sqlvars interface {
@@ -86,11 +116,10 @@ type sqlvars interface {
 	GetGlobal(name string) (sql.SystemVariable, interface{}, bool)
 }
 
-// We can manage certain aspects of the exposed databases on the server through
-// this.
-type dbProvider interface {
-	SetIsStandby(bool)
-}
+// Our IsStandbyCallback gets called with |true| or |false| when the server
+// becomes a standby or a primary respectively. Standby replicas should be read
+// only.
+type IsStandbyCallback func(bool)
 
 type procedurestore interface {
 	Register(sql.ExternalStoredProcedureDetails)
@@ -154,6 +183,26 @@ func NewController(lgr *logrus.Logger, cfg Config, pCfg config.ReadWriteConfig)
 	ret.sinterceptor.keyProvider = ret.jwks
 	ret.sinterceptor.jwtExpected = JWTExpectations()
 
+	ret.replicationClients, err = ret.replicationServiceClients(context.Background())
+	if err != nil {
+		return nil, err
+	}
+	ret.mysqlDbReplicas = make([]*mysqlDbReplica, len(ret.replicationClients))
+	for i := range ret.mysqlDbReplicas {
+		bo := backoff.NewExponentialBackOff()
+		bo.InitialInterval = time.Second
+		bo.MaxInterval = time.Minute
+		bo.MaxElapsedTime = 0
+		ret.mysqlDbReplicas[i] = &mysqlDbReplica{
+			lgr:     lgr.WithFields(logrus.Fields{}),
+			client:  ret.replicationClients[i],
+			backoff: bo,
+		}
+		ret.mysqlDbReplicas[i].cond = sync.NewCond(&ret.mysqlDbReplicas[i].mu)
+	}
+
+	ret.outstandingDropDatabases = make(map[string]*databaseDropReplication)
+
 	return ret, nil
 }
 
@@ -164,11 +213,23 @@ func (c *Controller) Run() {
 		defer wg.Done()
 		c.jwks.Run()
 	}()
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+		c.mysqlDbPersister.Run()
+	}()
+	wg.Add(1)
+	go func() {
+		defer wg.Done()
+		c.bcReplication.Run()
+	}()
 	wg.Wait()
 }
 
 func (c *Controller) GracefulStop() error {
 	c.jwks.GracefulStop()
+	c.mysqlDbPersister.GracefulStop()
+	c.bcReplication.GracefulStop()
 	return nil
 }
 
@@ -205,13 +266,13 @@ func (c *Controller) ApplyStandbyReplicationConfig(ctx context.Context, bt *sql.
 
 type IterSessions func(func(sql.Session) (bool, error)) error
 
-func (c *Controller) ManageDatabaseProvider(p dbProvider) {
+func (c *Controller) SetIsStandbyCallback(callback IsStandbyCallback) {
 	if c == nil {
 		return
 	}
 	c.mu.Lock()
 	defer c.mu.Unlock()
-	c.provider = p
+	c.standbyCallback = callback
 	c.setProviderIsStandby(c.role != RolePrimary)
 }
 
@@ -266,6 +327,113 @@ func (c *Controller) RegisterStoredProcedures(store procedurestore) {
 	store.Register(newTransitionToStandbyProcedure(c))
 }
 
+// Incoming drop database replication requests need a way to drop a database in
+// the sqle.DatabaseProvider. This is our callback for that functionality.
+func (c *Controller) SetDropDatabase(dropDatabase func(*sql.Context, string) error) {
+	if c == nil {
+		return
+	}
+	c.mu.Lock()
+	defer c.mu.Unlock()
+	c.dropDatabase = dropDatabase
+}
+
+// Our DropDatabaseHook gets called when the database provider drops a
+// database. This is how we learn that we need to replicate a drop database.
+func (c *Controller) DropDatabaseHook() func(string) {
+	if c == nil {
+		return nil
+	}
+	return c.dropDatabaseHook
+}
+
+func (c *Controller) dropDatabaseHook(dbname string) {
+	c.mu.Lock()
+	defer c.mu.Unlock()
+
+	// We always cleanup the commithooks associated with that database.
+
+	j := 0
+	for i := 0; i < len(c.commithooks); i++ {
+		if c.commithooks[i].dbname == dbname {
+			c.commithooks[i].databaseWasDropped()
+			continue
+		}
+		if j != i {
+			c.commithooks[j] = c.commithooks[i]
+		}
+		j += 1
+	}
+	c.commithooks = c.commithooks[:j]
+
+	if c.role != RolePrimary {
+		return
+	}
+
+	// If we are the primary, we will replicate the drop to our standby replicas.
+
+	ctx, cancel := context.WithCancel(context.Background())
+	wg := &sync.WaitGroup{}
+	wg.Add(len(c.replicationClients))
+	state := &databaseDropReplication{
+		ctx:    ctx,
+		cancel: cancel,
+		wg:     wg,
+	}
+	c.outstandingDropDatabases[dbname] = state
+
+	for _, client := range c.replicationClients {
+		client := client
+		go c.replicateDropDatabase(state, client, dbname)
+	}
+}
+
+func (c *Controller) cancelDropDatabaseReplication(dbname string) {
+	c.mu.Lock()
+	defer c.mu.Unlock()
+	if s := c.outstandingDropDatabases[dbname]; s != nil {
+		s.cancel()
+		s.wg.Wait()
+	}
+}
+
+func (c *Controller) replicateDropDatabase(s *databaseDropReplication, client *replicationServiceClient, dbname string) {
+	defer s.wg.Done()
+	bo := backoff.NewExponentialBackOff()
+	bo.InitialInterval = time.Millisecond
+	bo.MaxInterval = time.Minute
+	bo.MaxElapsedTime = 0
+	for {
+		if s.ctx.Err() != nil {
+			return
+		}
+		ctx, cancel := context.WithTimeout(s.ctx, 15*time.Second)
+		_, err := client.client.DropDatabase(ctx, &replicationapi.DropDatabaseRequest{
+			Name: dbname,
+		})
+		cancel()
+		if err == nil {
+			c.lgr.Tracef("successfully replicated drop of [%s] to %s", dbname, client.remote)
+			return
+		}
+		if status.Code(err) == codes.FailedPrecondition {
+			c.lgr.Warnf("drop of [%s] to %s will note be replicated; FailedPrecondition", dbname, client.remote)
+			return
+		}
+		c.lgr.Warnf("failed to replicate drop of [%s] to %s: %v", dbname, client.remote, err)
+		if s.ctx.Err() != nil {
+			return
+		}
+		d := bo.NextBackOff()
+		c.lgr.Tracef("sleeping %v before next drop attempt for database [%s] at %s", d, dbname, client.remote)
+		select {
+		case <-time.After(d):
+		case <-s.ctx.Done():
+			return
+		}
+	}
+}
+
 func (c *Controller) ClusterDatabase() sql.Database {
 	if c == nil {
 		return nil
@@ -448,6 +616,8 @@ func (c *Controller) setRoleAndEpoch(role string, epoch int, opts roleTransition
 		for _, h := range c.commithooks {
 			h.setRole(c.role)
 		}
+		c.mysqlDbPersister.setRole(c.role)
+		c.bcReplication.setRole(c.role)
 	}
 	_ = c.persistVariables()
 	return roleTransitionResult{
@@ -506,19 +676,83 @@ func (c *Controller) recordSuccessfulRemoteSrvCommit(name string) {
 	}
 }
 
-func (c *Controller) RemoteSrvServerArgs(ctx *sql.Context, args remotesrv.ServerArgs) remotesrv.ServerArgs {
+func (c *Controller) RemoteSrvServerArgs(ctxFactory func(context.Context) (*sql.Context, error), args remotesrv.ServerArgs) (remotesrv.ServerArgs, error) {
+	c.mu.Lock()
+	defer c.mu.Unlock()
 	listenaddr := c.RemoteSrvListenAddr()
 	args.HttpListenAddr = listenaddr
 	args.GrpcListenAddr = listenaddr
 	args.Options = c.ServerOptions()
-	args = sqle.RemoteSrvServerArgs(ctx, args)
+	var err error
+	args, err = sqle.RemoteSrvServerArgs(ctxFactory, args)
+	if err != nil {
+		return remotesrv.ServerArgs{}, err
+	}
 	args.DBCache = remotesrvStoreCache{args.DBCache, c}
+	c.remoteSrvDBCache = args.DBCache
 
 	keyID := creds.PubKeyToKID(c.pub)
 	keyIDStr := creds.B32CredsEncoding.EncodeToString(keyID)
 	args.HttpInterceptor = JWKSHandlerInterceptor(keyIDStr, c.pub)
 
-	return args
+	return args, nil
+}
+
+func (c *Controller) HookMySQLDbPersister(persister MySQLDbPersister, mysqlDb *mysql_db.MySQLDb) MySQLDbPersister {
+	if c != nil {
+		c.mysqlDb = mysqlDb
+		c.mysqlDbPersister = &replicatingMySQLDbPersister{
+			base:     persister,
+			replicas: c.mysqlDbReplicas,
+		}
+		c.mysqlDbPersister.setRole(c.role)
+		persister = c.mysqlDbPersister
+	}
+	return persister
+}
+
+func (c *Controller) HookBranchControlPersistence(controller *branch_control.Controller, fs filesys.Filesys) {
+	if c != nil {
+		c.branchControlController = controller
+		c.branchControlFilesys = fs
+
+		replicas := make([]*branchControlReplica, len(c.replicationClients))
+		for i := range replicas {
+			bo := backoff.NewExponentialBackOff()
+			bo.InitialInterval = time.Second
+			bo.MaxInterval = time.Minute
+			bo.MaxElapsedTime = 0
+			replicas[i] = &branchControlReplica{
+				backoff: bo,
+				client:  c.replicationClients[i],
+				lgr:     c.lgr.WithFields(logrus.Fields{}),
+			}
+			replicas[i].cond = sync.NewCond(&replicas[i].mu)
+		}
+		c.bcReplication = &branchControlReplication{
+			replicas:     replicas,
+			bcController: controller,
+		}
+		c.bcReplication.setRole(c.role)
+
+		controller.SavedCallback = func() {
+			contents := controller.Serialized.Load()
+			if contents != nil {
+				c.bcReplication.UpdateBranchControlContents(*contents)
+			}
+		}
+	}
+}
+
+func (c *Controller) RegisterGrpcServices(ctxFactory func(context.Context) (*sql.Context, error), srv *grpc.Server) {
+	replicationapi.RegisterReplicationServiceServer(srv, &replicationServiceServer{
+		ctxFactory:           ctxFactory,
+		mysqlDb:              c.mysqlDb,
+		branchControl:        c.branchControlController,
+		branchControlFilesys: c.branchControlFilesys,
+		dropDatabase:         c.dropDatabase,
+		lgr:                  c.lgr.WithFields(logrus.Fields{}),
+	})
 }
 
 // TODO: make the deadline here configurable or something.
@@ -607,6 +841,13 @@ func (c *Controller) gracefulTransitionToStandby(saveConnID, minCaughtUpStandbys
 		c.lgr.Tracef("cluster/controller: successfully replicated all databases to %d out of %d standbys; transitioning to standby.", numCaughtUp, len(replicas))
 	}
 
+	if !c.mysqlDbPersister.waitForReplication(waitForHooksToReplicateTimeout) {
+		c.lgr.Warnf("cluster/controller: when transitioning to standby, did not successfully replicate users and grants to all standbys.")
+	}
+	if !c.bcReplication.waitForReplication(waitForHooksToReplicateTimeout) {
+		c.lgr.Warnf("cluster/controller: when transitioning to standby, did not successfully replicate branch control data to all standbys.")
+	}
+
 	return res, nil
 }
 
@@ -653,8 +894,8 @@ func (c *Controller) killRunningQueries(saveConnID int) {
 
 // called with c.mu held
 func (c *Controller) setProviderIsStandby(standby bool) {
-	if c.provider != nil {
-		c.provider.SetIsStandby(standby)
+	if c.standbyCallback != nil {
+		c.standbyCallback(standby)
 	}
 }
 
@@ -843,3 +1084,48 @@ func (c *Controller) standbyRemotesJWKS() *jwtauth.MultiJWKS {
 	}
 	return jwtauth.NewMultiJWKS(c.lgr.WithFields(logrus.Fields{"component": "jwks-key-provider"}), urls, client)
 }
+
+type replicationServiceClient struct {
+	remote string
+	url    string
+	client replicationapi.ReplicationServiceClient
+}
+
+func (c *Controller) replicationServiceDialOptions() []grpc.DialOption {
+	var ret []grpc.DialOption
+	if c.tlsCfg == nil {
+		ret = append(ret, grpc.WithInsecure())
+	} else {
+		ret = append(ret, grpc.WithTransportCredentials(credentials.NewTLS(c.tlsCfg)))
+	}
+
+	ret = append(ret, grpc.WithStreamInterceptor(c.cinterceptor.Stream()))
+	ret = append(ret, grpc.WithUnaryInterceptor(c.cinterceptor.Unary()))
+
+	ret = append(ret, grpc.WithPerRPCCredentials(c.grpcCreds))
+
+	return ret
+}
+
+func (c *Controller) replicationServiceClients(ctx context.Context) ([]*replicationServiceClient, error) {
+	var ret []*replicationServiceClient
+	for _, r := range c.cfg.StandbyRemotes() {
+		urlStr := strings.Replace(r.RemoteURLTemplate(), dsess.URLTemplateDatabasePlaceholder, "", -1)
+		url, err := url.Parse(urlStr)
+		if err != nil {
+			return nil, fmt.Errorf("could not parse remote url template [%s] for remote %s: %w", r.RemoteURLTemplate(), r.Name(), err)
+		}
+		grpcTarget := "dns:" + url.Hostname() + ":" + url.Port()
+		cc, err := grpc.DialContext(ctx, grpcTarget, c.replicationServiceDialOptions()...)
+		if err != nil {
+			return nil, fmt.Errorf("could not dial grpc endpoint [%s] for remote %s: %w", grpcTarget, r.Name(), err)
+		}
+		client := replicationapi.NewReplicationServiceClient(cc)
+		ret = append(ret, &replicationServiceClient{
+			remote: r.Name(),
+			url:    grpcTarget,
+			client: client,
+		})
+	}
+	return ret, nil
+}
