@@ -22,10 +22,14 @@ import (
 	"strings"
 	"time"
 
+	sqle "github.com/dolthub/go-mysql-server"
 	"github.com/dolthub/go-mysql-server/sql"
-	"github.com/dolthub/go-mysql-server/sql/parse"
+	"github.com/dolthub/go-mysql-server/sql/analyzer"
+	"github.com/dolthub/go-mysql-server/sql/fulltext"
 	"github.com/dolthub/go-mysql-server/sql/plan"
+	"github.com/dolthub/go-mysql-server/sql/planbuilder"
 	"github.com/dolthub/go-mysql-server/sql/types"
+	"github.com/dolthub/vitess/go/vt/sqlparser"
 	"gopkg.in/src-d/go-errors.v1"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/branch_control"
@@ -53,7 +57,7 @@ type Database struct {
 	ddb           *doltdb.DoltDB
 	rsr           env.RepoStateReader
 	rsw           env.RepoStateWriter
-	gs            globalstate.GlobalState
+	gs            dsess.GlobalStateImpl
 	editOpts      editor.Options
 	revision      string
 	revType       dsess.RevisionType
@@ -61,7 +65,7 @@ type Database struct {
 
 var _ dsess.SqlDatabase = Database{}
 var _ dsess.RevisionDatabase = Database{}
-var _ globalstate.StateProvider = Database{}
+var _ globalstate.GlobalStateProvider = Database{}
 var _ sql.CollatedDatabase = Database{}
 var _ sql.Database = Database{}
 var _ sql.StoredProcedureDatabase = Database{}
@@ -76,6 +80,7 @@ var _ sql.VersionedDatabase = Database{}
 var _ sql.ViewDatabase = Database{}
 var _ sql.EventDatabase = Database{}
 var _ sql.AliasedDatabase = Database{}
+var _ fulltext.Database = Database{}
 
 type ReadOnlyDatabase struct {
 	Database
@@ -134,7 +139,7 @@ func (db Database) DoltDatabases() []*doltdb.DoltDB {
 
 // NewDatabase returns a new dolt database to use in queries.
 func NewDatabase(ctx context.Context, name string, dbData env.DbData, editOpts editor.Options) (Database, error) {
-	globalState, err := globalstate.NewGlobalStateStoreForDb(ctx, dbData.Ddb)
+	globalState, err := dsess.NewGlobalStateStoreForDb(ctx, name, dbData.Ddb)
 	if err != nil {
 		return Database{}, err
 	}
@@ -244,6 +249,9 @@ func (db Database) GetTableInsensitive(ctx *sql.Context, tblName string) (sql.Ta
 
 // GetTableInsensitiveAsOf implements sql.VersionedDatabase
 func (db Database) GetTableInsensitiveAsOf(ctx *sql.Context, tableName string, asOf interface{}) (sql.Table, bool, error) {
+	if asOf == nil {
+		return db.GetTableInsensitive(ctx, tableName)
+	}
 	head, root, err := resolveAsOf(ctx, db, asOf)
 	if err != nil {
 		return nil, false, err
@@ -622,25 +630,34 @@ func (db Database) getTable(ctx *sql.Context, root *doltdb.RootValue, tableName
 		return nil, false, err
 	}
 
-	var table sql.Table
-
-	readonlyTable, err := NewDoltTable(tableName, sch, tbl, db, db.editOpts)
+	table, err := db.newDoltTable(tableName, sch, tbl)
 	if err != nil {
 		return nil, false, err
 	}
-	if doltdb.IsReadOnlySystemTable(tableName) {
-		table = readonlyTable
-	} else if doltdb.HasDoltPrefix(tableName) {
-		table = &WritableDoltTable{DoltTable: readonlyTable, db: db}
-	} else {
-		table = &AlterableDoltTable{WritableDoltTable{DoltTable: readonlyTable, db: db}}
-	}
 
 	dbState.SessionCache().CacheTable(key, tableName, table)
 
 	return table, true, nil
 }
 
+// newDoltTable returns a sql.Table wrapping the given underlying dolt table
+func (db Database) newDoltTable(tableName string, sch schema.Schema, tbl *doltdb.Table) (sql.Table, error) {
+	readonlyTable, err := NewDoltTable(tableName, sch, tbl, db, db.editOpts)
+	if err != nil {
+		return nil, err
+	}
+	var table sql.Table
+	if doltdb.IsReadOnlySystemTable(tableName) {
+		table = readonlyTable
+	} else if doltdb.HasDoltPrefix(tableName) && !doltdb.IsFullTextTable(tableName) {
+		table = &WritableDoltTable{DoltTable: readonlyTable, db: db}
+	} else {
+		table = &AlterableDoltTable{WritableDoltTable{DoltTable: readonlyTable, db: db}}
+	}
+
+	return table, nil
+}
+
 // GetTableNames returns the names of all user tables. System tables in user space (e.g. dolt_docs, dolt_query_catalog)
 // are filtered out. This method is used for queries that examine the schema of the database, e.g. show tables. Table
 // name resolution in queries is handled by GetTableInsensitive. Use GetAllTableNames for an unfiltered list of all
@@ -826,7 +843,7 @@ func (db Database) removeTableFromAutoIncrementTracker(
 		wses = append(wses, ws)
 	}
 
-	ait, err := db.gs.GetAutoIncrementTracker(ctx)
+	ait, err := db.gs.AutoIncrementTracker(ctx)
 	if err != nil {
 		return err
 	}
@@ -849,7 +866,7 @@ func (db Database) CreateTable(ctx *sql.Context, tableName string, sch sql.Prima
 		if !dtables.DoltDocsSqlSchema.Equals(sch.Schema) && !dtables.OldDoltDocsSqlSchema.Equals(sch.Schema) {
 			return fmt.Errorf("incorrect schema for dolt_docs table")
 		}
-	} else if doltdb.HasDoltPrefix(tableName) {
+	} else if doltdb.HasDoltPrefix(tableName) && !doltdb.IsFullTextTable(tableName) {
 		return ErrReservedTableName.New(tableName)
 	}
 
@@ -881,6 +898,32 @@ func (db Database) CreateIndexedTable(ctx *sql.Context, tableName string, sch sq
 	return db.createIndexedSqlTable(ctx, tableName, sch, idxDef, collation)
 }
 
+// CreateFulltextTableNames returns a set of names that will be used to create Full-Text pseudo-index tables.
+func (db Database) CreateFulltextTableNames(ctx *sql.Context, parentTableName string, parentIndexName string) (fulltext.IndexTableNames, error) {
+	allTableNames, err := db.GetAllTableNames(ctx)
+	if err != nil {
+		return fulltext.IndexTableNames{}, err
+	}
+	var tablePrefix string
+OuterLoop:
+	for i := uint64(0); true; i++ {
+		tablePrefix = strings.ToLower(fmt.Sprintf("dolt_%s_%s_%d", parentTableName, parentIndexName, i))
+		for _, tableName := range allTableNames {
+			if strings.HasPrefix(strings.ToLower(tableName), tablePrefix) {
+				continue OuterLoop
+			}
+		}
+		break
+	}
+	return fulltext.IndexTableNames{
+		Config:      fmt.Sprintf("dolt_%s_fts_config", parentTableName),
+		Position:    fmt.Sprintf("%s_fts_position", tablePrefix),
+		DocCount:    fmt.Sprintf("%s_fts_doc_count", tablePrefix),
+		GlobalCount: fmt.Sprintf("%s_fts_global_count", tablePrefix),
+		RowCount:    fmt.Sprintf("%s_fts_row_count", tablePrefix),
+	}, nil
+}
+
 // createSqlTable is the private version of CreateTable. It doesn't enforce any table name checks.
 func (db Database) createSqlTable(ctx *sql.Context, tableName string, sch sql.PrimaryKeySchema, collation sql.CollationID) error {
 	ws, err := db.GetWorkingSet(ctx)
@@ -913,7 +956,7 @@ func (db Database) createSqlTable(ctx *sql.Context, tableName string, sch sql.Pr
 	// Prevent any tables that use BINARY, CHAR, VARBINARY, VARCHAR prefixes
 
 	if schema.HasAutoIncrement(doltSch) {
-		ait, err := db.gs.GetAutoIncrementTracker(ctx)
+		ait, err := db.gs.AutoIncrementTracker(ctx)
 		if err != nil {
 			return err
 		}
@@ -961,7 +1004,7 @@ func (db Database) createIndexedSqlTable(ctx *sql.Context, tableName string, sch
 	}
 
 	if schema.HasAutoIncrement(doltSch) {
-		ait, err := db.gs.GetAutoIncrementTracker(ctx)
+		ait, err := db.gs.AutoIncrementTracker(ctx)
 		if err != nil {
 			return err
 		}
@@ -1124,14 +1167,17 @@ func getViewDefinitionFromSchemaFragmentsOfView(ctx *sql.Context, tbl *WritableD
 	var viewDef sql.ViewDefinition
 	var views = make([]sql.ViewDefinition, len(fragments))
 	for i, fragment := range fragments {
-		cv, err := parse.Parse(ctx, fragments[i].fragment)
+		cv, err := sqlparser.ParseWithOptions(fragments[i].fragment,
+			sql.NewSqlModeFromString(fragment.sqlMode).ParserOptions())
 		if err != nil {
 			return nil, sql.ViewDefinition{}, false, err
 		}
 
-		createView, ok := cv.(*plan.CreateView)
+		createView, ok := cv.(*sqlparser.DDL)
 		if ok {
-			views[i] = sql.ViewDefinition{Name: fragments[i].name, TextDefinition: createView.Definition.TextDefinition, CreateViewStatement: fragments[i].fragment}
+			selectStr := fragments[i].fragment[createView.SubStatementPositionStart:createView.SubStatementPositionEnd]
+			views[i] = sql.ViewDefinition{Name: fragments[i].name, TextDefinition: selectStr,
+				CreateViewStatement: fragments[i].fragment, SqlMode: fragment.sqlMode}
 		} else {
 			views[i] = sql.ViewDefinition{Name: fragments[i].name, TextDefinition: fragments[i].fragment, CreateViewStatement: fmt.Sprintf("CREATE VIEW %s AS %s", fragments[i].name, fragments[i].fragment)}
 		}
@@ -1200,6 +1246,7 @@ func (db Database) GetTriggers(ctx *sql.Context) ([]sql.TriggerDefinition, error
 			Name:            frag.name,
 			CreateStatement: frag.fragment,
 			CreatedAt:       frag.created,
+			SqlMode:         frag.sqlMode,
 		})
 	}
 	if err != nil {
@@ -1243,50 +1290,133 @@ func (db Database) GetEvent(ctx *sql.Context, name string) (sql.EventDefinition,
 
 	for _, frag := range frags {
 		if strings.ToLower(frag.name) == strings.ToLower(name) {
-			return sql.EventDefinition{
-				Name:            frag.name,
-				CreateStatement: frag.fragment,
-				CreatedAt:       frag.created,
-			}, true, nil
+			event, err := db.createEventDefinitionFromFragment(ctx, frag)
+			if err != nil {
+				return sql.EventDefinition{}, false, err
+			}
+			return *event, true, nil
 		}
 	}
 	return sql.EventDefinition{}, false, nil
 }
 
 // GetEvents implements sql.EventDatabase.
-func (db Database) GetEvents(ctx *sql.Context) ([]sql.EventDefinition, error) {
+func (db Database) GetEvents(ctx *sql.Context) (events []sql.EventDefinition, token interface{}, err error) {
 	tbl, ok, err := db.GetTableInsensitive(ctx, doltdb.SchemasTableName)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	if !ok {
-		return nil, nil
+		// If the dolt_schemas table doesn't exist, it's not an error, just no events
+		return nil, nil, nil
 	}
 
 	frags, err := getSchemaFragmentsOfType(ctx, tbl.(*WritableDoltTable), eventFragment)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	for _, frag := range frags {
+		event, err := db.createEventDefinitionFromFragment(ctx, frag)
+		if err != nil {
+			return nil, nil, err
+		}
+		events = append(events, *event)
+	}
+
+	// Grab a hash of the dolt_schemas table to use as the identifying token
+	// to track if events need to be reloaded.
+	tableHash, err := db.doltSchemaTableHash(ctx)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	return events, tableHash, nil
+}
+
+// NeedsToReloadEvents implements sql.EventDatabase.
+func (db Database) NeedsToReloadEvents(ctx *sql.Context, token interface{}) (bool, error) {
+	hash, ok := token.(hash.Hash)
+	if !ok {
+		return false, fmt.Errorf("expected token to be hash.Hash, but received %T", token)
+	}
+
+	tableHash, err := db.doltSchemaTableHash(ctx)
+	if err != nil {
+		return false, err
+	}
+
+	// If the current hash doesn't match what we last loaded, then we
+	// need to reload event definitions
+	return !tableHash.Equal(hash), nil
+}
+
+// doltSchemaTableHash returns the hash of the dolt_schemas table, or any error encountered along the way.
+func (db Database) doltSchemaTableHash(ctx *sql.Context) (hash.Hash, error) {
+	root, err := db.GetRoot(ctx)
+	if err != nil {
+		return hash.Hash{}, err
+	}
+
+	tableHash, _, err := root.GetTableHash(ctx, doltdb.SchemasTableName)
+	return tableHash, err
+}
+
+// createEventDefinitionFromFragment creates an EventDefinition instance from the schema fragment |frag|.
+func (db Database) createEventDefinitionFromFragment(ctx *sql.Context, frag schemaFragment) (*sql.EventDefinition, error) {
+	catalog := db.getCatalog(ctx)
+	sqlMode := sql.NewSqlModeFromString(frag.sqlMode)
+	parsed, err := planbuilder.ParseWithOptions(ctx, catalog, updateEventStatusTemporarilyForNonDefaultBranch(db.revision, frag.fragment), sqlMode.ParserOptions())
 	if err != nil {
 		return nil, err
 	}
 
-	var events []sql.EventDefinition
-	for _, frag := range frags {
-		events = append(events, sql.EventDefinition{
-			Name:            frag.name,
-			CreateStatement: frag.fragment,
-			CreatedAt:       frag.created,
-		})
+	eventPlan, ok := parsed.(*plan.CreateEvent)
+	if !ok {
+		return nil, fmt.Errorf("unexpected type %T for create event statement", eventPlan)
 	}
-	return events, nil
+
+	// NOTE: Time fields for events are assumed to be specified in the session's timezone, which defaults to the
+	//       system's timezone. When we store them, we store them at UTC, and when we send them back to a caller
+	//       we convert them back to the caller's session timezone.
+	//       Here we are loading the events from disk, so they are already in UTC and don't need any other
+	//       timezone applied, so we specify "+00:00".
+	event, err := eventPlan.GetEventDefinition(ctx, frag.created, frag.created, frag.created, "+00:00")
+	if err != nil {
+		return nil, err
+	}
+	event.SqlMode = frag.sqlMode
+
+	return &event, nil
+}
+
+// getCatalog creates and returns the analyzer.Catalog instance for this database.
+func (db Database) getCatalog(ctx *sql.Context) *analyzer.Catalog {
+	doltSession := dsess.DSessFromSess(ctx.Session)
+	return sqle.NewDefault(doltSession.Provider()).Analyzer.Catalog
 }
 
 // SaveEvent implements sql.EventDatabase.
-func (db Database) SaveEvent(ctx *sql.Context, ed sql.EventDefinition) error {
-	return db.addFragToSchemasTable(ctx,
+func (db Database) SaveEvent(ctx *sql.Context, event sql.EventDefinition) (bool, error) {
+	// If the database is not the default branch database, then the event is disabled.
+	// TODO: need better way to determine the default branch; currently it checks only 'main'
+	if db.revision != env.DefaultInitBranch && event.Status == sql.EventStatus_Enable.String() {
+		// using revision database name
+		event.Status = sql.EventStatus_Disable.String()
+		ctx.Session.Warn(&sql.Warning{
+			Level:   "Warning",
+			Code:    1105,
+			Message: fmt.Sprintf("Event status cannot be enabled for revision database."),
+		})
+	}
+
+	// TODO: store LastAltered, LastExecuted and TimezoneOffset in appropriate place
+	return event.Status == sql.EventStatus_Enable.String(), db.addFragToSchemasTable(ctx,
 		eventFragment,
-		ed.Name,
-		ed.CreateStatement,
-		ed.CreatedAt,
-		sql.ErrEventAlreadyExists.New(ed.Name),
+		event.Name,
+		event.CreateEventStatement(),
+		event.CreatedAt,
+		sql.ErrEventAlreadyExists.New(event.Name),
 	)
 }
 
@@ -1296,13 +1426,31 @@ func (db Database) DropEvent(ctx *sql.Context, name string) error {
 }
 
 // UpdateEvent implements sql.EventDatabase.
-func (db Database) UpdateEvent(ctx *sql.Context, originalName string, ed sql.EventDefinition) error {
+func (db Database) UpdateEvent(ctx *sql.Context, originalName string, event sql.EventDefinition) (bool, error) {
 	// TODO: any EVENT STATUS change should also update the branch-specific event scheduling
 	err := db.DropEvent(ctx, originalName)
 	if err != nil {
-		return err
+		return false, err
 	}
-	return db.SaveEvent(ctx, ed)
+	return db.SaveEvent(ctx, event)
+}
+
+// UpdateLastExecuted implements sql.EventDatabase
+func (db Database) UpdateLastExecuted(ctx *sql.Context, eventName string, lastExecuted time.Time) error {
+	// TODO: update LastExecuted in appropriate place
+	return nil
+}
+
+// updateEventStatusTemporarilyForNonDefaultBranch updates the event status from ENABLE to DISABLE if it's not default branch.
+// The event status metadata is not updated in storage, but only for display purposes we return event status as 'DISABLE'.
+// This function is used temporarily to implement logic of only allowing enabled events to be executed on default branch.
+func updateEventStatusTemporarilyForNonDefaultBranch(revision, createStmt string) string {
+	// TODO: need better way to determine the default branch; currently it checks only 'main'
+
+	if revision == "" || revision == env.DefaultInitBranch {
+		return createStmt
+	}
+	return strings.Replace(createStmt, "ENABLE", "DISABLE", 1)
 }
 
 // GetStoredProcedure implements sql.StoredProcedureDatabase.
@@ -1372,7 +1520,9 @@ func (db Database) addFragToSchemasTable(ctx *sql.Context, fragType, name, defin
 		return err
 	}
 
-	return inserter.Insert(ctx, sql.Row{fragType, name, definition, extraJSON})
+	sqlMode := sql.LoadSqlMode(ctx)
+
+	return inserter.Insert(ctx, sql.Row{fragType, name, definition, extraJSON, sqlMode.String()})
 }
 
 func (db Database) dropFragFromSchemasTable(ctx *sql.Context, fragType, name string, missingErr error) error {
