@@ -15,6 +15,7 @@
 package sqle
 
 import (
+	"bytes"
 	"fmt"
 	"io"
 	"sort"
@@ -25,6 +26,7 @@ import (
 	"github.com/dolthub/go-mysql-server/sql/rowexec"
 	sqltypes "github.com/dolthub/go-mysql-server/sql/types"
 	"github.com/dolthub/vitess/go/mysql"
+	"golang.org/x/exp/slices"
 
 	"github.com/dolthub/dolt/go/cmd/dolt/errhand"
 	"github.com/dolthub/dolt/go/libraries/doltcore/diff"
@@ -33,12 +35,35 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dtables"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlfmt"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
+	"github.com/dolthub/dolt/go/store/types"
 )
 
 var _ sql.TableFunction = (*PatchTableFunction)(nil)
 var _ sql.ExecSourceRel = (*PatchTableFunction)(nil)
+var _ sql.IndexAddressable = (*PatchTableFunction)(nil)
+var _ sql.IndexedTable = (*PatchTableFunction)(nil)
+var _ sql.TableNode = (*PatchTableFunction)(nil)
+
+const (
+	diffTypeSchema = "schema"
+	diffTypeData   = "data"
+)
+
+var schemaChangePartitionKey = []byte(diffTypeSchema)
+var dataChangePartitionKey = []byte(diffTypeData)
+var schemaAndDataChangePartitionKey = []byte("all")
+
+const (
+	orderColumnName     = "statement_order"
+	fromColumnName      = "from_commit_hash"
+	toColumnName        = "to_commit_hash"
+	tableNameColumnName = "table_name"
+	diffTypeColumnName  = "diff_type"
+	statementColumnName = "statement"
+)
 
 type PatchTableFunction struct {
 	ctx *sql.Context
@@ -50,13 +75,143 @@ type PatchTableFunction struct {
 	database       sql.Database
 }
 
+func (p *PatchTableFunction) CollationCoercibility(ctx *sql.Context) (collation sql.CollationID, coercibility byte) {
+	return sql.Collation_binary, 7
+}
+
+type Partition struct {
+	key []byte
+}
+
+func (p *Partition) Key() []byte { return p.key }
+
+// UnderlyingTable implements the plan.TableNode interface
+func (p *PatchTableFunction) UnderlyingTable() sql.Table {
+	return p
+}
+
+// Collation implements the sql.Table interface.
+func (p *PatchTableFunction) Collation() sql.CollationID {
+	return sql.Collation_Default
+}
+
+// Partitions is a sql.Table interface function that returns a partition of the data. This data has a single partition.
+func (p *PatchTableFunction) Partitions(ctx *sql.Context) (sql.PartitionIter, error) {
+	return dtables.NewSliceOfPartitionsItr([]sql.Partition{
+		&Partition{key: schemaAndDataChangePartitionKey},
+	}), nil
+}
+
+// PartitionRows is a sql.Table interface function that takes a partition and returns all rows in that partition.
+// This table has a partition for just schema changes, one for just data changes, and one for both.
+func (p *PatchTableFunction) PartitionRows(ctx *sql.Context, partition sql.Partition) (sql.RowIter, error) {
+	fromCommitVal, toCommitVal, dotCommitVal, tableName, err := p.evaluateArguments()
+	if err != nil {
+		return nil, err
+	}
+
+	sqledb, ok := p.database.(dsess.SqlDatabase)
+	if !ok {
+		return nil, fmt.Errorf("unable to get dolt database")
+	}
+
+	fromRefDetails, toRefDetails, err := loadDetailsForRefs(ctx, fromCommitVal, toCommitVal, dotCommitVal, sqledb)
+	if err != nil {
+		return nil, err
+	}
+
+	tableDeltas, err := diff.GetTableDeltas(ctx, fromRefDetails.root, toRefDetails.root)
+	if err != nil {
+		return nil, err
+	}
+
+	sort.Slice(tableDeltas, func(i, j int) bool {
+		return strings.Compare(tableDeltas[i].ToName, tableDeltas[j].ToName) < 0
+	})
+
+	// If tableNameExpr defined, return a single table patch result
+	if p.tableNameExpr != nil {
+		fromTblExists, err := fromRefDetails.root.HasTable(ctx, tableName)
+		if err != nil {
+			return nil, err
+		}
+		toTblExists, err := toRefDetails.root.HasTable(ctx, tableName)
+		if err != nil {
+			return nil, err
+		}
+		if !fromTblExists && !toTblExists {
+			return nil, sql.ErrTableNotFound.New(tableName)
+		}
+
+		delta := findMatchingDelta(tableDeltas, tableName)
+		tableDeltas = []diff.TableDelta{delta}
+	}
+
+	includeSchemaDiff := bytes.Equal(partition.Key(), schemaAndDataChangePartitionKey) || bytes.Equal(partition.Key(), schemaChangePartitionKey)
+	includeDataDiff := bytes.Equal(partition.Key(), schemaAndDataChangePartitionKey) || bytes.Equal(partition.Key(), dataChangePartitionKey)
+
+	patches, err := getPatchNodes(ctx, sqledb.DbData(), tableDeltas, fromRefDetails, toRefDetails, includeSchemaDiff, includeDataDiff)
+	if err != nil {
+		return nil, err
+	}
+
+	return newPatchTableFunctionRowIter(patches, fromRefDetails.hashStr, toRefDetails.hashStr), nil
+}
+
+// LookupPartitions is a sql.IndexedTable interface function that takes an index lookup and returns the set of corresponding partitions.
+func (p *PatchTableFunction) LookupPartitions(context *sql.Context, lookup sql.IndexLookup) (sql.PartitionIter, error) {
+	if lookup.Index.ID() == diffTypeColumnName {
+		diffTypes, ok := index.LookupToPointSelectStr(lookup)
+		if !ok {
+			return nil, fmt.Errorf("failed to parse commit lookup ranges: %s", sql.DebugString(lookup.Ranges))
+		}
+
+		includeSchemaDiff := slices.Contains(diffTypes, diffTypeSchema)
+		includeDataDiff := slices.Contains(diffTypes, diffTypeData)
+
+		if includeSchemaDiff && includeDataDiff {
+			return dtables.NewSliceOfPartitionsItr([]sql.Partition{
+				&Partition{key: schemaAndDataChangePartitionKey},
+			}), nil
+		}
+
+		if includeSchemaDiff {
+			return dtables.NewSliceOfPartitionsItr([]sql.Partition{
+				&Partition{key: schemaChangePartitionKey},
+			}), nil
+		}
+
+		if includeDataDiff {
+			return dtables.NewSliceOfPartitionsItr([]sql.Partition{
+				&Partition{key: dataChangePartitionKey},
+			}), nil
+		}
+
+		return dtables.NewSliceOfPartitionsItr([]sql.Partition{}), nil
+	}
+
+	return dtables.NewSliceOfPartitionsItr([]sql.Partition{
+		&Partition{key: schemaAndDataChangePartitionKey},
+	}), nil
+}
+
+func (p *PatchTableFunction) IndexedAccess(lookup sql.IndexLookup) sql.IndexedTable {
+	return p
+}
+
+func (p *PatchTableFunction) GetIndexes(ctx *sql.Context) ([]sql.Index, error) {
+	return []sql.Index{
+		index.MockIndex(diffTypeColumnName, p.Name(), types.StringKind, false),
+	}, nil
+}
+
 var patchTableSchema = sql.Schema{
-	&sql.Column{Name: "statement_order", Type: sqltypes.Uint64, PrimaryKey: true, Nullable: false},
-	&sql.Column{Name: "from_commit_hash", Type: sqltypes.LongText, Nullable: false},
-	&sql.Column{Name: "to_commit_hash", Type: sqltypes.LongText, Nullable: false},
-	&sql.Column{Name: "table_name", Type: sqltypes.LongText, Nullable: false},
-	&sql.Column{Name: "diff_type", Type: sqltypes.LongText, Nullable: false},
-	&sql.Column{Name: "statement", Type: sqltypes.LongText, Nullable: false},
+	&sql.Column{Name: orderColumnName, Type: sqltypes.Uint64, PrimaryKey: true, Nullable: false},
+	&sql.Column{Name: fromColumnName, Type: sqltypes.LongText, Nullable: false},
+	&sql.Column{Name: toColumnName, Type: sqltypes.LongText, Nullable: false},
+	&sql.Column{Name: tableNameColumnName, Type: sqltypes.LongText, Nullable: false},
+	&sql.Column{Name: diffTypeColumnName, Type: sqltypes.LongText, Nullable: false},
+	&sql.Column{Name: statementColumnName, Type: sqltypes.LongText, Nullable: false},
 }
 
 // NewInstance creates a new instance of TableFunction interface
@@ -82,6 +237,10 @@ func (p *PatchTableFunction) Resolved() bool {
 	return p.commitsResolved()
 }
 
+func (p *PatchTableFunction) IsReadOnly() bool {
+	return true
+}
+
 func (p *PatchTableFunction) commitsResolved() bool {
 	if p.dotCommitExpr != nil {
 		return p.dotCommitExpr.Resolved()
@@ -100,7 +259,10 @@ func (p *PatchTableFunction) String() string {
 	if p.tableNameExpr != nil {
 		return fmt.Sprintf("DOLT_PATCH(%s, %s, %s)", p.fromCommitExpr.String(), p.toCommitExpr.String(), p.tableNameExpr.String())
 	}
-	return fmt.Sprintf("DOLT_PATCH(%s, %s)", p.fromCommitExpr.String(), p.toCommitExpr.String())
+	if p.fromCommitExpr != nil && p.toCommitExpr != nil {
+		return fmt.Sprintf("DOLT_PATCH(%s, %s)", p.fromCommitExpr.String(), p.toCommitExpr.String())
+	}
+	return fmt.Sprintf("DOLT_PATCH(<INVALID>)")
 }
 
 // Schema implements the sql.Node interface.
@@ -241,59 +403,17 @@ func (p *PatchTableFunction) WithDatabase(database sql.Database) (sql.Node, erro
 
 // Name implements the sql.TableFunction interface
 func (p *PatchTableFunction) Name() string {
-	return "dolt_patch"
+	return p.String()
 }
 
-// RowIter implements the sql.Node interface
+// RowIter implements the sql.ExecSourceRel interface
 func (p *PatchTableFunction) RowIter(ctx *sql.Context, row sql.Row) (sql.RowIter, error) {
-	fromCommitVal, toCommitVal, dotCommitVal, tableName, err := p.evaluateArguments()
+	partitions, err := p.Partitions(ctx)
 	if err != nil {
 		return nil, err
 	}
 
-	sqledb, ok := p.database.(dsess.SqlDatabase)
-	if !ok {
-		return nil, fmt.Errorf("unable to get dolt database")
-	}
-
-	fromRefDetails, toRefDetails, err := loadDetailsForRefs(ctx, fromCommitVal, toCommitVal, dotCommitVal, sqledb)
-	if err != nil {
-		return nil, err
-	}
-
-	tableDeltas, err := diff.GetTableDeltas(ctx, fromRefDetails.root, toRefDetails.root)
-	if err != nil {
-		return nil, err
-	}
-
-	sort.Slice(tableDeltas, func(i, j int) bool {
-		return strings.Compare(tableDeltas[i].ToName, tableDeltas[j].ToName) < 0
-	})
-
-	// If tableNameExpr defined, return a single table patch result
-	if p.tableNameExpr != nil {
-		fromTblExists, err := fromRefDetails.root.HasTable(ctx, tableName)
-		if err != nil {
-			return nil, err
-		}
-		toTblExists, err := toRefDetails.root.HasTable(ctx, tableName)
-		if err != nil {
-			return nil, err
-		}
-		if !fromTblExists && !toTblExists {
-			return nil, sql.ErrTableNotFound.New(tableName)
-		}
-
-		delta := findMatchingDelta(tableDeltas, tableName)
-		tableDeltas = []diff.TableDelta{delta}
-	}
-
-	patches, err := getPatchNodes(ctx, sqledb.DbData(), tableDeltas, fromRefDetails, toRefDetails)
-	if err != nil {
-		return nil, err
-	}
-
-	return newPatchTableFunctionRowIter(patches, fromRefDetails.hashStr, toRefDetails.hashStr), nil
+	return sql.NewTableRowIter(ctx, p, partitions), nil
 }
 
 // evaluateArguments returns fromCommitVal, toCommitVal, dotCommitVal, and tableName.
@@ -341,8 +461,7 @@ type patchNode struct {
 	dataPatchStmts   []string
 }
 
-func getPatchNodes(ctx *sql.Context, dbData env.DbData, tableDeltas []diff.TableDelta, fromRefDetails, toRefDetails *refDetails) ([]*patchNode, error) {
-	var patches []*patchNode
+func getPatchNodes(ctx *sql.Context, dbData env.DbData, tableDeltas []diff.TableDelta, fromRefDetails, toRefDetails *refDetails, includeSchemaDiff, includeDataDiff bool) (patches []*patchNode, err error) {
 	for _, td := range tableDeltas {
 		// no diff
 		if td.FromTable == nil && td.ToTable == nil {
@@ -355,14 +474,18 @@ func getPatchNodes(ctx *sql.Context, dbData env.DbData, tableDeltas []diff.Table
 		}
 
 		// Get SCHEMA DIFF
-		schemaStmts, err := getSchemaSqlPatch(ctx, toRefDetails.root, td)
-		if err != nil {
-			return nil, err
+		var schemaStmts []string
+		if includeSchemaDiff {
+
+			schemaStmts, err = getSchemaSqlPatch(ctx, toRefDetails.root, td)
+			if err != nil {
+				return nil, err
+			}
 		}
 
 		// Get DATA DIFF
 		var dataStmts []string
-		if canGetDataDiff(ctx, td) {
+		if includeDataDiff && canGetDataDiff(ctx, td) {
 			dataStmts, err = getUserTableDataSqlPatch(ctx, dbData, td, fromRefDetails, toRefDetails)
 			if err != nil {
 				return nil, err
@@ -400,7 +523,7 @@ func getSchemaSqlPatch(ctx *sql.Context, toRoot *doltdb.RootValue, td diff.Table
 		}
 		ddlStatements = append(ddlStatements, stmt)
 	} else {
-		stmts, err := diff.GetNonCreateNonDropTableSqlSchemaDiff(td, toSchemas, fromSch, toSch)
+		stmts, err := GetNonCreateNonDropTableSqlSchemaDiff(td, toSchemas, fromSch, toSch)
 		if err != nil {
 			return nil, err
 		}
@@ -424,16 +547,7 @@ func canGetDataDiff(ctx *sql.Context, td diff.TableDelta) bool {
 		})
 		return false
 	}
-	// cannot sql diff
-	if td.ToSch == nil || (td.FromSch != nil && !schema.SchemasAreEqual(td.FromSch, td.ToSch)) {
-		// TODO(8/24/22 Zach): this is overly broad, we can absolutely do better
-		ctx.Session.Warn(&sql.Warning{
-			Level:   "Warning",
-			Code:    mysql.ERNotSupportedYet,
-			Message: fmt.Sprintf("Incompatible schema change, skipping data diff for table '%s'", td.ToName),
-		})
-		return false
-	}
+
 	return true
 }
 
@@ -498,6 +612,92 @@ func getDataSqlPatchResults(ctx *sql.Context, diffQuerySch, targetSch sql.Schema
 	}
 }
 
+// GetNonCreateNonDropTableSqlSchemaDiff returns any schema diff in SQL statements that is NEITHER 'CREATE TABLE' NOR 'DROP TABLE' statements.
+func GetNonCreateNonDropTableSqlSchemaDiff(td diff.TableDelta, toSchemas map[string]schema.Schema, fromSch, toSch schema.Schema) ([]string, error) {
+	if td.IsAdd() || td.IsDrop() {
+		// use add and drop specific methods
+		return nil, nil
+	}
+
+	var ddlStatements []string
+	if td.FromName != td.ToName {
+		ddlStatements = append(ddlStatements, sqlfmt.RenameTableStmt(td.FromName, td.ToName))
+	}
+
+	eq := schema.SchemasAreEqual(fromSch, toSch)
+	if eq && !td.HasFKChanges() {
+		return ddlStatements, nil
+	}
+
+	colDiffs, unionTags := diff.DiffSchColumns(fromSch, toSch)
+	for _, tag := range unionTags {
+		cd := colDiffs[tag]
+		switch cd.DiffType {
+		case diff.SchDiffNone:
+		case diff.SchDiffAdded:
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableAddColStmt(td.ToName, sqlfmt.GenerateCreateTableColumnDefinition(*cd.New, sql.CollationID(td.ToSch.GetCollation()))))
+		case diff.SchDiffRemoved:
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableDropColStmt(td.ToName, cd.Old.Name))
+		case diff.SchDiffModified:
+			// Ignore any primary key set changes here
+			if cd.Old.IsPartOfPK != cd.New.IsPartOfPK {
+				continue
+			}
+			if cd.Old.Name != cd.New.Name {
+				ddlStatements = append(ddlStatements, sqlfmt.AlterTableRenameColStmt(td.ToName, cd.Old.Name, cd.New.Name))
+			}
+		}
+	}
+
+	// Print changes between a primary key set change. It contains an ALTER TABLE DROP and an ALTER TABLE ADD
+	if !schema.ColCollsAreEqual(fromSch.GetPKCols(), toSch.GetPKCols()) {
+		ddlStatements = append(ddlStatements, sqlfmt.AlterTableDropPks(td.ToName))
+		if toSch.GetPKCols().Size() > 0 {
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableAddPrimaryKeys(td.ToName, toSch.GetPKCols().GetColumnNames()))
+		}
+	}
+
+	for _, idxDiff := range diff.DiffSchIndexes(fromSch, toSch) {
+		switch idxDiff.DiffType {
+		case diff.SchDiffNone:
+		case diff.SchDiffAdded:
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableAddIndexStmt(td.ToName, idxDiff.To))
+		case diff.SchDiffRemoved:
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableDropIndexStmt(td.FromName, idxDiff.From))
+		case diff.SchDiffModified:
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableDropIndexStmt(td.FromName, idxDiff.From))
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableAddIndexStmt(td.ToName, idxDiff.To))
+		}
+	}
+
+	for _, fkDiff := range diff.DiffForeignKeys(td.FromFks, td.ToFks) {
+		switch fkDiff.DiffType {
+		case diff.SchDiffNone:
+		case diff.SchDiffAdded:
+			parentSch := toSchemas[fkDiff.To.ReferencedTableName]
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableAddForeignKeyStmt(fkDiff.To, toSch, parentSch))
+		case diff.SchDiffRemoved:
+			from := fkDiff.From
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableDropForeignKeyStmt(from.TableName, from.Name))
+		case diff.SchDiffModified:
+			from := fkDiff.From
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableDropForeignKeyStmt(from.TableName, from.Name))
+
+			parentSch := toSchemas[fkDiff.To.ReferencedTableName]
+			ddlStatements = append(ddlStatements, sqlfmt.AlterTableAddForeignKeyStmt(fkDiff.To, toSch, parentSch))
+		}
+	}
+
+	// Handle charset/collation changes
+	toCollation := toSch.GetCollation()
+	fromCollation := fromSch.GetCollation()
+	if toCollation != fromCollation {
+		ddlStatements = append(ddlStatements, sqlfmt.AlterTableCollateStmt(td.ToName, fromCollation, toCollation))
+	}
+
+	return ddlStatements, nil
+}
+
 // getDiffQuery returns diff schema for specified columns and array of sql.Expression as projection to be used
 // on diff table function row iter. This function attempts to imitate running a query
 // fmt.Sprintf("select %s, %s from dolt_diff('%s', '%s', '%s')", columnsWithDiff, "diff_type", fromRef, toRef, tableName)
@@ -547,7 +747,7 @@ func getDiffQuerySqlSchemaAndProjections(diffTableSch sql.Schema, columns []stri
 		idx    int
 	}
 
-	columns = append(columns, "diff_type")
+	columns = append(columns, diffTypeColumnName)
 	colMap := make(map[string]*column)
 	for _, c := range columns {
 		colMap[c] = nil
@@ -668,9 +868,9 @@ func (p *patchStatementsRowIter) Next(ctx *sql.Context) (sql.Row, error) {
 	}
 
 	stmt := p.stmts[p.idx]
-	diffType := "schema"
+	diffType := diffTypeSchema
 	if p.idx >= p.ddlLen {
-		diffType = "data"
+		diffType = diffTypeData
 	}
 
 	return sql.Row{diffType, stmt}, nil
