@@ -344,6 +344,22 @@ func ResolveSchemaConflicts(ctx *sql.Context, ddb *doltdb.DoltDB, ws *doltdb.Wor
 		return ws, nil // no schema conflicts
 	}
 
+	// TODO: There's an issue with using `dolt conflicts resolve` for schema conflicts, since having
+	//       schema conflicts reported means that we haven't yet merged the table data. In some case,
+	//       such as when there have ONLY been schema changes and no data changes that need to be
+	//       merged, it is safe to use `dolt conflicts resolve`, but there are many other cases where the
+	//       data changes would not be merged and could surprise customers. So, we are being cautious to
+	//       prevent auto-resolution of schema changes with `dolt conflicts resolve` until we have a fix
+	//       for resolving schema changes AND merging data (including dealing with any data conflicts).
+	//       For more details, see: https://github.com/dolthub/dolt/issues/6616
+	if ws.MergeState().HasSchemaConflicts() {
+		return nil, fmt.Errorf("Unable to automatically resolve schema conflicts since data changes may " +
+			"not have been fully merged yet. " +
+			"To continue, abort this merge (dolt merge --abort) then apply ALTER TABLE statements to one " +
+			"side of this merge to get the two schemas in sync with the desired schema, then rerun the merge. " +
+			"To track resolution of this limitation, follow https://github.com/dolthub/dolt/issues/6616")
+	}
+
 	tblSet := set.NewStrSet(tables)
 	updates := make(map[string]*doltdb.Table)
 	err := ws.MergeState().IterSchemaConflicts(ctx, ddb, func(table string, conflict doltdb.SchemaConflict) error {
@@ -362,11 +378,13 @@ func ResolveSchemaConflicts(ctx *sql.Context, ddb *doltdb.DoltDB, ws *doltdb.Wor
 		return nil, err
 	}
 
+	var merged []string
 	root := ws.WorkingRoot()
 	for name, tbl := range updates {
 		if root, err = root.PutTable(ctx, name, tbl); err != nil {
 			return nil, err
 		}
+		merged = append(merged, name)
 	}
 
 	// clear resolved schema conflicts
@@ -377,7 +395,8 @@ func ResolveSchemaConflicts(ctx *sql.Context, ddb *doltdb.DoltDB, ws *doltdb.Wor
 		}
 		unmerged = append(unmerged, tbl)
 	}
-	return ws.WithWorkingRoot(root).WithUnmergableTables(unmerged), nil
+
+	return ws.WithWorkingRoot(root).WithUnmergableTables(unmerged).WithMergedTables(merged), nil
 }
 
 func ResolveDataConflicts(ctx *sql.Context, dSess *dsess.DoltSession, root *doltdb.RootValue, dbName string, ours bool, tblNames []string) error {
