@@ -26,8 +26,6 @@ import (
 	"github.com/dolthub/dolt/go/store/hash"
 )
 
-var _ sql.Table = (*CommitsTable)(nil)
-
 // CommitsTable is a sql.Table that implements a system table which
 // shows the combined commit log for all branches in the repo.
 type CommitsTable struct {
@@ -35,6 +33,9 @@ type CommitsTable struct {
 	ddb    *doltdb.DoltDB
 }
 
+var _ sql.Table = (*CommitsTable)(nil)
+var _ sql.IndexAddressable = (*CommitsTable)(nil)
+
 // NewCommitsTable creates a CommitsTable
 func NewCommitsTable(_ *sql.Context, ddb *doltdb.DoltDB) sql.Table {
 	return &CommitsTable{ddb: ddb}
@@ -88,7 +89,7 @@ func (dt *CommitsTable) GetIndexes(ctx *sql.Context) ([]sql.Index, error) {
 }
 
 // IndexedAccess implements sql.IndexAddressable
-func (dt *CommitsTable) IndexedAccess(_ sql.IndexLookup) sql.IndexedTable {
+func (dt *CommitsTable) IndexedAccess(lookup sql.IndexLookup) sql.IndexedTable {
 	nt := *dt
 	return &nt
 }
