@@ -52,6 +52,7 @@ const (
 
 var _ sql.Table = (*DiffTable)(nil)
 var _ sql.IndexedTable = (*DiffTable)(nil)
+var _ sql.IndexAddressable = (*DiffTable)(nil)
 
 type DiffTable struct {
 	name        string
@@ -152,7 +153,7 @@ func (dt *DiffTable) Partitions(ctx *sql.Context) (sql.PartitionIter, error) {
 	}
 
 	if !ok {
-		return nil, errors.New(fmt.Sprintf("table: %s does not exist", dt.name))
+		return nil, fmt.Errorf("table: %s does not exist", dt.name)
 	}
 
 	wrTblHash, _, err := dt.workingRoot.GetTableHash(ctx, exactName)
@@ -268,7 +269,7 @@ func (dt *DiffTable) fromCommitLookupPartitions(ctx *sql.Context, hashes []hash.
 	if err != nil {
 		return nil, err
 	} else if !ok {
-		return nil, errors.New(fmt.Sprintf("table: %s does not exist", dt.name))
+		return nil, fmt.Errorf("table: %s does not exist", dt.name)
 	}
 
 	var parentHashes []hash.Hash
@@ -434,7 +435,7 @@ func (dt *DiffTable) toCommitLookupPartitions(ctx *sql.Context, hashes []hash.Ha
 	if err != nil {
 		return nil, err
 	} else if !ok {
-		return nil, errors.New(fmt.Sprintf("table: %s does not exist", dt.name))
+		return nil, fmt.Errorf("table: %s does not exist", dt.name)
 	}
 
 	working, err := dt.head.HashOf()
@@ -706,17 +707,6 @@ type DiffPartitions struct {
 	fromSch         schema.Schema
 }
 
-func NewDiffPartitions(tblName string, cmItr doltdb.CommitItr, cmHashToTblInfo map[hash.Hash]TblInfoAtCommit, selectFunc partitionSelectFunc, toSch, fromSch schema.Schema) *DiffPartitions {
-	return &DiffPartitions{
-		tblName:         tblName,
-		cmItr:           cmItr,
-		cmHashToTblInfo: cmHashToTblInfo,
-		selectFunc:      selectFunc,
-		toSch:           toSch,
-		fromSch:         fromSch,
-	}
-}
-
 // processCommit is called in a commit iteration loop. Adds partitions when it finds a commit and its parent that have
 // different values for the hash of the table being looked at.
 func (dps *DiffPartitions) processCommit(ctx *sql.Context, cmHash hash.Hash, cm *doltdb.Commit, root *doltdb.RootValue, tbl *doltdb.Table) (*DiffPartition, error) {
