@@ -18,10 +18,10 @@ import (
 	"context"
 	"fmt"
 	"os"
+	"runtime"
 	"testing"
 	"time"
 
-	gms "github.com/dolthub/go-mysql-server"
 	"github.com/dolthub/go-mysql-server/enginetest"
 	"github.com/dolthub/go-mysql-server/enginetest/queries"
 	"github.com/dolthub/go-mysql-server/enginetest/scriptgen/setup"
@@ -49,7 +49,7 @@ var skipPrepared bool
 // SkipPreparedsCount is used by the "ci-check-repo CI workflow
 // as a reminder to consider prepareds when adding a new
 // enginetest suite.
-const SkipPreparedsCount = 82
+const SkipPreparedsCount = 83
 
 const skipPreparedFlag = "DOLT_SKIP_PREPARED_ENGINETESTS"
 
@@ -91,8 +91,8 @@ func TestSingleQuery(t *testing.T) {
 		enginetest.RunQuery(t, engine, harness, q)
 	}
 
-	engine.Analyzer.Debug = true
-	engine.Analyzer.Verbose = true
+	engine.EngineAnalyzer().Debug = true
+	engine.EngineAnalyzer().Verbose = true
 
 	var test queries.QueryTest
 	test = queries.QueryTest{
@@ -115,135 +115,36 @@ func TestSingleQuery(t *testing.T) {
 
 // Convenience test for debugging a single query. Unskip and set to the desired query.
 func TestSingleScript(t *testing.T) {
-	// t.Skip()
+	t.Skip()
 
 	var scripts = []queries.ScriptTest{
 		{
-			Name: "dolt_checkout and base name resolution",
+			Name: "insert duplicate key doesn't prevent other updates, autocommit off",
 			SetUpScript: []string{
-				"create table t (a int primary key, b int);",
-				"call dolt_commit('-Am', 'creating table t');",
-				"call dolt_branch('b2');",
-				"call dolt_branch('b3');",
-				"insert into t values (1, 1);",
-				"call dolt_commit('-Am', 'added values on main');",
-				"call dolt_checkout('b2');",
-				"insert into t values (2, 2);",
-				"call dolt_commit('-am', 'added values on b2');",
-				"call dolt_checkout('b3');",
-				"insert into t values (3, 3);",
-				"call dolt_commit('-am', 'added values on b3');",
-				"call dolt_checkout('main');",
+				"CREATE TABLE t1 (pk BIGINT PRIMARY KEY, v1 VARCHAR(3));",
+				"INSERT INTO t1 VALUES (1, 'abc');",
+				"SET autocommit = 0;",
 			},
 			Assertions: []queries.ScriptTestAssertion{
 				{
-					Query:    "select active_branch();",
-					Expected: []sql.Row{{"main"}},
+					Query:    "select * from t1 order by pk",
+					Expected: []sql.Row{{1, "abc"}},
 				},
 				{
-					Query:    "select * from t;",
-					Expected: []sql.Row{{1, 1}},
+					Query:       "INSERT INTO t1 VALUES (1, 'abc');",
+					ExpectedErr: sql.ErrPrimaryKeyViolation,
 				},
 				{
-					Query:            "use `mydb/b2`;",
+					Query:    "INSERT INTO t1 VALUES (2, 'def');",
+					Expected: []sql.Row{{gmstypes.NewOkResult(1)}},
+				},
+				{
+					Query:            "commit",
 					SkipResultsCheck: true,
 				},
 				{
-					Query:    "select active_branch();",
-					Expected: []sql.Row{{"b2"}},
-				},
-				{
-					Query:    "select * from t;",
-					Expected: []sql.Row{{2, 2}},
-				},
-				{
-					Query:    "select * from mydb.t;",
-					Expected: []sql.Row{{1, 1}},
-				},
-				{
-					Query:            "use `mydb/b3`;",
-					SkipResultsCheck: true,
-				},
-				{
-					Query:    "select active_branch();",
-					Expected: []sql.Row{{"b3"}},
-				},
-				{
-					Query:    "select * from t;",
-					Expected: []sql.Row{{3, 3}},
-				},
-				{
-					Query:    "select * from mydb.t;",
-					Expected: []sql.Row{{1, 1}},
-				},
-				{
-					Query:    "select * from `mydb/b2`.t;",
-					Expected: []sql.Row{{2, 2}},
-				},
-				{
-					Query:            "use `mydb/main`",
-					SkipResultsCheck: true,
-				},
-				{
-					Query:    "select active_branch();",
-					Expected: []sql.Row{{"main"}},
-				},
-				{
-					Query:    "select * from t;",
-					Expected: []sql.Row{{1, 1}},
-				},
-				{
-					Query:    "select * from mydb.t;",
-					Expected: []sql.Row{{1, 1}},
-				},
-				{
-					Query:    "select * from `mydb/b3`.t;",
-					Expected: []sql.Row{{3, 3}},
-				},
-				{
-					Query:            "use `mydb`",
-					SkipResultsCheck: true,
-				},
-				{
-					Query:    "select active_branch();",
-					Expected: []sql.Row{{"main"}},
-				},
-				{
-					Query:    "select * from t;",
-					Expected: []sql.Row{{1, 1}},
-				},
-				{
-					Query:    "select * from `mydb/main`.t;",
-					Expected: []sql.Row{{1, 1}},
-				},
-				{
-					Query:            "call dolt_checkout('b2');",
-					SkipResultsCheck: true,
-				},
-				{
-					Query:            "use `mydb/b3`",
-					SkipResultsCheck: true,
-				},
-				{
-					Query:    "select active_branch();",
-					Expected: []sql.Row{{"b3"}},
-				},
-				// Since b2 was the last branch checked out with dolt_checkout, it's what mydb resolves to
-				{
-					Query:    "select * from `mydb`.t;",
-					Expected: []sql.Row{{2, 2}},
-				},
-				{
-					Query:            "use `mydb`",
-					SkipResultsCheck: true,
-				},
-				{
-					Query:    "select active_branch();",
-					Expected: []sql.Row{{"b2"}},
-				},
-				{
-					Query:    "select * from t;",
-					Expected: []sql.Row{{2, 2}},
+					Query:    "select * from t1 order by pk",
+					Expected: []sql.Row{{1, "abc"}, {2, "def"}},
 				},
 			},
 		},
@@ -254,7 +155,7 @@ func TestSingleScript(t *testing.T) {
 	defer cleanup()
 
 	harness := newDoltHarness(t)
-	harness.Setup(setup.MydbData, setup.Parent_childData)
+	harness.Setup(setup.MydbData)
 	for _, script := range scripts {
 		sql.RunWithNowFunc(tcc.Now, func() error {
 			enginetest.TestScript(t, harness, script)
@@ -409,6 +310,14 @@ func TestVersionedQueries(t *testing.T) {
 	enginetest.TestVersionedQueries(t, h)
 }
 
+func TestAnsiQuotesSqlMode(t *testing.T) {
+	enginetest.TestAnsiQuotesSqlMode(t, newDoltHarness(t))
+}
+
+func TestAnsiQuotesSqlModePrepared(t *testing.T) {
+	enginetest.TestAnsiQuotesSqlModePrepared(t, newDoltHarness(t))
+}
+
 // Tests of choosing the correct execution plan independent of result correctness. Mostly useful for confirming that
 // the right indexes are being used for joining tables.
 func TestQueryPlans(t *testing.T) {
@@ -664,11 +573,6 @@ func TestSpatialIndexPlans(t *testing.T) {
 	enginetest.TestSpatialIndexPlans(t, newDoltHarness(t))
 }
 
-func TestSpatialIndexPlansPrepared(t *testing.T) {
-	skipOldFormat(t)
-	enginetest.TestSpatialIndexPlansPrepared(t, newDoltHarness(t))
-}
-
 func TestTruncate(t *testing.T) {
 	h := newDoltHarness(t)
 	defer h.Close()
@@ -719,8 +623,8 @@ func TestDoltUserPrivileges(t *testing.T) {
 				Address: "localhost",
 			})
 
-			engine.Analyzer.Catalog.MySQLDb.AddRootAccount()
-			engine.Analyzer.Catalog.MySQLDb.SetPersister(&mysql_db.NoopPersister{})
+			engine.EngineAnalyzer().Catalog.MySQLDb.AddRootAccount()
+			engine.EngineAnalyzer().Catalog.MySQLDb.SetPersister(&mysql_db.NoopPersister{})
 
 			for _, statement := range script.SetUpScript {
 				if sh, ok := interface{}(harness).(enginetest.SkippingHarness); ok {
@@ -778,15 +682,6 @@ func TestJoinOps(t *testing.T) {
 	enginetest.TestJoinOps(t, h)
 }
 
-func TestJoinPlanningPrepared(t *testing.T) {
-	if types.IsFormat_LD(types.Format_Default) {
-		t.Skip("DOLT_LD keyless indexes are not sorted")
-	}
-	h := newDoltHarness(t).WithParallelism(1)
-	defer h.Close()
-	enginetest.TestJoinPlanningPrepared(t, h)
-}
-
 func TestJoinPlanning(t *testing.T) {
 	if types.IsFormat_LD(types.Format_Default) {
 		t.Skip("DOLT_LD keyless indexes are not sorted")
@@ -796,16 +691,6 @@ func TestJoinPlanning(t *testing.T) {
 	enginetest.TestJoinPlanning(t, h)
 }
 
-func TestJoinOpsPrepared(t *testing.T) {
-	if types.IsFormat_LD(types.Format_Default) {
-		t.Skip("DOLT_LD keyless indexes are not sorted")
-	}
-
-	h := newDoltHarness(t)
-	defer h.Close()
-	enginetest.TestJoinOpsPrepared(t, h)
-}
-
 func TestJoinQueries(t *testing.T) {
 	h := newDoltHarness(t)
 	defer h.Close()
@@ -1096,6 +981,18 @@ func TestForeignKeyBranchesPrepared(t *testing.T) {
 	}
 }
 
+func TestFulltextIndexes(t *testing.T) {
+	if !types.IsFormat_DOLT(types.Format_Default) {
+		t.Skip("FULLTEXT is not supported on the old format")
+	}
+	if runtime.GOOS == "windows" && os.Getenv("CI") != "" {
+		t.Skip("For some reason, this is flaky only on Windows CI.")
+	}
+	h := newDoltHarness(t)
+	defer h.Close()
+	enginetest.TestFulltextIndexes(t, h)
+}
+
 func TestCreateCheckConstraints(t *testing.T) {
 	h := newDoltHarness(t)
 	defer h.Close()
@@ -1129,7 +1026,7 @@ func TestDropCheckConstraints(t *testing.T) {
 func TestReadOnly(t *testing.T) {
 	h := newDoltHarness(t)
 	defer h.Close()
-	enginetest.TestReadOnly(t, h)
+	enginetest.TestReadOnly(t, h, false /* testStoredProcedures */)
 }
 
 func TestViews(t *testing.T) {
@@ -1219,7 +1116,8 @@ func TestColumnDefaults(t *testing.T) {
 }
 
 func TestAlterTable(t *testing.T) {
-	h := newDoltHarness(t)
+	// This is a newly added test in GMS that dolt doesn't support yet
+	h := newDoltHarness(t).WithSkippedQueries([]string{"ALTER TABLE t42 ADD COLUMN s varchar(20), drop check check1"})
 	defer h.Close()
 	enginetest.TestAlterTable(t, h)
 }
@@ -1291,6 +1189,26 @@ func TestStoredProcedures(t *testing.T) {
 	enginetest.TestStoredProcedures(t, h)
 }
 
+func TestDoltStoredProcedures(t *testing.T) {
+	for _, script := range DoltProcedureTests {
+		func() {
+			h := newDoltHarness(t)
+			defer h.Close()
+			enginetest.TestScript(t, h, script)
+		}()
+	}
+}
+
+func TestDoltStoredProceduresPrepared(t *testing.T) {
+	for _, script := range DoltProcedureTests {
+		func() {
+			h := newDoltHarness(t)
+			defer h.Close()
+			enginetest.TestScriptPrepared(t, h, script)
+		}()
+	}
+}
+
 func TestEvents(t *testing.T) {
 	doltHarness := newDoltHarness(t)
 	defer doltHarness.Close()
@@ -1484,7 +1402,7 @@ func TestDoltRevisionDbScripts(t *testing.T) {
 			},
 			{
 				Query:    "call dolt_checkout('main');",
-				Expected: []sql.Row{{0}},
+				Expected: []sql.Row{{0, "Switched to branch 'main'"}},
 			},
 			{
 				Query:    "select database();",
@@ -1613,19 +1531,10 @@ func TestDoltMerge(t *testing.T) {
 		func() {
 			h := newDoltHarness(t).WithParallelism(1)
 			defer h.Close()
+			h.Setup(setup.MydbData)
 			enginetest.TestScript(t, h, script)
 		}()
 	}
-
-	if types.IsFormat_DOLT(types.Format_Default) {
-		for _, script := range Dolt1MergeScripts {
-			func() {
-				h := newDoltHarness(t).WithParallelism(1)
-				defer h.Close()
-				enginetest.TestScript(t, h, script)
-			}()
-		}
-	}
 }
 
 func TestDoltMergePrepared(t *testing.T) {
@@ -1638,15 +1547,27 @@ func TestDoltMergePrepared(t *testing.T) {
 			enginetest.TestScriptPrepared(t, h, script)
 		}()
 	}
+}
 
-	if types.IsFormat_DOLT(types.Format_Default) {
-		for _, script := range Dolt1MergeScripts {
-			func() {
-				h := newDoltHarness(t).WithParallelism(1)
-				defer h.Close()
-				enginetest.TestScriptPrepared(t, h, script)
-			}()
-		}
+func TestDoltRevert(t *testing.T) {
+	for _, script := range RevertScripts {
+		// harness can't reset effectively. Use a new harness for each script
+		func() {
+			h := newDoltHarness(t).WithParallelism(1)
+			defer h.Close()
+			enginetest.TestScript(t, h, script)
+		}()
+	}
+}
+
+func TestDoltRevertPrepared(t *testing.T) {
+	for _, script := range RevertScripts {
+		// harness can't reset effectively. Use a new harness for each script
+		func() {
+			h := newDoltHarness(t).WithParallelism(1)
+			defer h.Close()
+			enginetest.TestScriptPrepared(t, h, script)
+		}()
 	}
 }
 
@@ -1659,15 +1580,6 @@ func TestDoltAutoIncrement(t *testing.T) {
 			enginetest.TestScript(t, h, script)
 		}()
 	}
-
-	for _, script := range BrokenAutoIncrementTests {
-		t.Run(script.Name, func(t *testing.T) {
-			t.Skip()
-			h := newDoltHarness(t)
-			defer h.Close()
-			enginetest.TestScript(t, h, script)
-		})
-	}
 }
 
 func TestDoltAutoIncrementPrepared(t *testing.T) {
@@ -1679,15 +1591,6 @@ func TestDoltAutoIncrementPrepared(t *testing.T) {
 			enginetest.TestScriptPrepared(t, h, script)
 		}()
 	}
-
-	for _, script := range BrokenAutoIncrementTests {
-		t.Run(script.Name, func(t *testing.T) {
-			t.Skip()
-			h := newDoltHarness(t)
-			defer h.Close()
-			enginetest.TestScriptPrepared(t, h, script)
-		})
-	}
 }
 
 func TestDoltConflictsTableNameTable(t *testing.T) {
@@ -1726,9 +1629,6 @@ func TestKeylessDoltMergeCVsAndConflicts(t *testing.T) {
 
 // eventually this will be part of TestDoltMerge
 func TestDoltMergeArtifacts(t *testing.T) {
-	if !types.IsFormat_DOLT(types.Format_Default) {
-		t.Skip()
-	}
 	for _, script := range MergeArtifactsScripts {
 		func() {
 			h := newDoltHarness(t)
@@ -1841,12 +1741,12 @@ func (tcc *testCommitClock) Now() time.Time {
 }
 
 func installTestCommitClock(tcc *testCommitClock) func() {
-	oldNowFunc := datas.CommitNowFunc
+	oldNowFunc := datas.CommitterDate
 	oldCommitLoc := datas.CommitLoc
-	datas.CommitNowFunc = tcc.Now
+	datas.CommitterDate = tcc.Now
 	datas.CommitLoc = time.UTC
 	return func() {
-		datas.CommitNowFunc = oldNowFunc
+		datas.CommitterDate = oldNowFunc
 		datas.CommitLoc = oldCommitLoc
 	}
 }
@@ -1861,67 +1761,56 @@ func TestSingleTransactionScript(t *testing.T) {
 	defer cleanup()
 
 	sql.RunWithNowFunc(tcc.Now, func() error {
-
 		script := queries.TransactionTest{
-			Name: "READ ONLY Transactions",
+			Name: "Insert error with auto commit off",
 			SetUpScript: []string{
-				"create table t2 (pk int primary key, val int)",
-				"insert into t2 values (0,0)",
-				"commit",
+				"create table t1 (pk int primary key, val int)",
+				"insert into t1 values (0,0)",
 			},
 			Assertions: []queries.ScriptTestAssertion{
 				{
-					Query:    "/* client a */ set autocommit = off",
-					Expected: []sql.Row{{}},
+					Query:            "/* client a */ set autocommit = off",
+					SkipResultsCheck: true,
 				},
 				{
-					Query:    "/* client a */ create temporary table tmp(pk int primary key)",
-					Expected: []sql.Row{{gmstypes.NewOkResult(0)}},
+					Query:            "/* client b */ set autocommit = off",
+					SkipResultsCheck: true,
 				},
 				{
-					Query:    "/* client a */  START TRANSACTION READ ONLY",
-					Expected: []sql.Row{},
-				},
-				{
-					Query:    "/* client a */ INSERT INTO tmp VALUES (1)",
+					Query:    "/* client a */ insert into t1 values (1, 1)",
 					Expected: []sql.Row{{gmstypes.NewOkResult(1)}},
 				},
 				{
-					Query:       "/* client a */ insert into t2 values (1, 1)",
-					ExpectedErr: sql.ErrReadOnlyTransaction,
+					Query:       "/* client a */ insert into t1 values (1, 2)",
+					ExpectedErr: sql.ErrPrimaryKeyViolation,
 				},
 				{
-					Query:       "/* client a */ insert into t2 values (2, 2)",
-					ExpectedErr: sql.ErrReadOnlyTransaction,
+					Query:    "/* client a */ insert into t1 values (2, 2)",
+					Expected: []sql.Row{{gmstypes.NewOkResult(1)}},
 				},
 				{
-					Query:       "/* client a */ delete from t2 where pk = 0",
-					ExpectedErr: sql.ErrReadOnlyTransaction,
+					Query:    "/* client a */ select * from t1 order by pk",
+					Expected: []sql.Row{{0, 0}, {1, 1}, {2, 2}},
 				},
 				{
-
-					Query:    "/* client a */ alter table t2 add val2 int",
-					Expected: []sql.Row{{gmstypes.NewOkResult(0)}},
+					Query:    "/* client b */ select * from t1 order by pk",
+					Expected: []sql.Row{{0, 0}},
 				},
 				{
-					Query:    "/* client a */ select * from t2",
-					Expected: []sql.Row{{0, 0, nil}},
+					Query:            "/* client a */ commit",
+					SkipResultsCheck: true,
 				},
 				{
-					Query:       "/* client a */ create temporary table tmp2(pk int primary key)",
-					ExpectedErr: sql.ErrReadOnlyTransaction,
+					Query:            "/* client b */ start transaction",
+					SkipResultsCheck: true,
 				},
 				{
-					Query:    "/* client a */ COMMIT",
-					Expected: []sql.Row{},
+					Query:    "/* client b */ select * from t1 order by pk",
+					Expected: []sql.Row{{0, 0}, {1, 1}, {2, 2}},
 				},
 				{
-					Query:    "/* client b */ START TRANSACTION READ ONLY",
-					Expected: []sql.Row{},
-				},
-				{
-					Query:    "/* client b */ SELECT * FROM t2",
-					Expected: []sql.Row{{0, 0, nil}},
+					Query:    "/* client a */ select * from t1 order by pk",
+					Expected: []sql.Row{{0, 0}, {1, 1}, {2, 2}},
 				},
 			},
 		}
@@ -2119,6 +2008,7 @@ func TestLogTableFunction(t *testing.T) {
 	harness.Setup(setup.MydbData)
 	for _, test := range LogTableFunctionScriptTests {
 		harness.engine = nil
+		harness.skipSetupCommit = true
 		t.Run(test.Name, func(t *testing.T) {
 			enginetest.TestScript(t, harness, test)
 		})
@@ -2131,6 +2021,7 @@ func TestLogTableFunctionPrepared(t *testing.T) {
 	harness.Setup(setup.MydbData)
 	for _, test := range LogTableFunctionScriptTests {
 		harness.engine = nil
+		harness.skipSetupCommit = true
 		t.Run(test.Name, func(t *testing.T) {
 			enginetest.TestScriptPrepared(t, harness, test)
 		})
@@ -2237,7 +2128,19 @@ func TestSchemaDiffSystemTablePrepared(t *testing.T) {
 	}
 }
 
-func mustNewEngine(t *testing.T, h enginetest.Harness) *gms.Engine {
+func TestQueryDiff(t *testing.T) {
+	harness := newDoltHarness(t)
+	defer harness.Close()
+	harness.Setup(setup.MydbData)
+	for _, test := range QueryDiffTableScriptTests {
+		harness.engine = nil
+		t.Run(test.Name, func(t *testing.T) {
+			enginetest.TestScript(t, harness, test)
+		})
+	}
+}
+
+func mustNewEngine(t *testing.T, h enginetest.Harness) enginetest.QueryEngine {
 	e, err := h.NewEngine(t)
 	if err != nil {
 		require.NoError(t, err)
@@ -2263,7 +2166,7 @@ func TestSystemTableIndexes(t *testing.T) {
 		harness.SkipSetupCommit()
 		e := mustNewEngine(t, harness)
 		defer e.Close()
-		e.Analyzer.Coster = memo.NewMergeBiasedCoster()
+		e.EngineAnalyzer().Coster = memo.NewMergeBiasedCoster()
 
 		ctx := enginetest.NewContext(harness)
 		for _, q := range stt.setup {
@@ -2271,7 +2174,7 @@ func TestSystemTableIndexes(t *testing.T) {
 		}
 
 		for i, c := range []string{"inner", "lookup", "hash", "merge"} {
-			e.Analyzer.Coster = biasedCosters[i]
+			e.EngineAnalyzer().Coster = biasedCosters[i]
 			for _, tt := range stt.queries {
 				t.Run(fmt.Sprintf("%s(%s): %s", stt.name, c, tt.query), func(t *testing.T) {
 					if tt.skip {
@@ -2313,13 +2216,35 @@ func TestSystemTableIndexesPrepared(t *testing.T) {
 
 				ctx = ctx.WithQuery(tt.query)
 				if tt.exp != nil {
-					enginetest.TestPreparedQueryWithContext(t, ctx, e, harness, tt.query, tt.exp, nil)
+					enginetest.TestPreparedQueryWithContext(t, ctx, e, harness, tt.query, tt.exp, nil, nil)
 				}
 			})
 		}
 	}
 }
 
+func TestSystemTableFunctionIndexes(t *testing.T) {
+	harness := newDoltHarness(t)
+	harness.Setup(setup.MydbData)
+	for _, test := range SystemTableFunctionIndexTests {
+		harness.engine = nil
+		t.Run(test.Name, func(t *testing.T) {
+			enginetest.TestScript(t, harness, test)
+		})
+	}
+}
+
+func TestSystemTableFunctionIndexesPrepared(t *testing.T) {
+	harness := newDoltHarness(t)
+	harness.Setup(setup.MydbData)
+	for _, test := range SystemTableFunctionIndexTests {
+		harness.engine = nil
+		t.Run(test.Name, func(t *testing.T) {
+			enginetest.TestScriptPrepared(t, harness, test)
+		})
+	}
+}
+
 func TestReadOnlyDatabases(t *testing.T) {
 	h := newDoltHarness(t)
 	defer h.Close()
@@ -2335,7 +2260,10 @@ func TestAddDropPks(t *testing.T) {
 func TestAddAutoIncrementColumn(t *testing.T) {
 	h := newDoltHarness(t)
 	defer h.Close()
-	enginetest.TestAddAutoIncrementColumn(t, h)
+
+	for _, script := range queries.AlterTableAddAutoIncrementScripts {
+		enginetest.TestScript(t, h, script)
+	}
 }
 
 func TestNullRanges(t *testing.T) {
@@ -2406,12 +2334,6 @@ func TestQueriesPrepared(t *testing.T) {
 	enginetest.TestQueriesPrepared(t, h)
 }
 
-func TestPreparedStaticIndexQuery(t *testing.T) {
-	h := newDoltHarness(t)
-	defer h.Close()
-	enginetest.TestPreparedStaticIndexQuery(t, h)
-}
-
 func TestStatistics(t *testing.T) {
 	h := newDoltHarness(t)
 	defer h.Close()
@@ -2824,8 +2746,16 @@ func TestDoltStorageFormatPrepared(t *testing.T) {
 
 func TestThreeWayMergeWithSchemaChangeScripts(t *testing.T) {
 	skipOldFormat(t)
-	t.Run("right to left merges", func(t *testing.T) {
-		for _, script := range ThreeWayMergeWithSchemaChangeTestScripts {
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsBasicCases, "basic cases", false)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsForDataConflicts, "data conflicts", false)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsCollations, "collation changes", false)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsConstraints, "constraint changes", false)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsSchemaConflicts, "schema conflicts", false)
+
+	// Run non-symmetric schema merge tests in just one direction
+	t.Run("type changes", func(t *testing.T) {
+		for _, script := range SchemaChangeTestsTypeChanges {
+			// run in a func() so we can cleanly defer closing the harness
 			func() {
 				h := newDoltHarness(t)
 				defer h.Close()
@@ -2833,36 +2763,62 @@ func TestThreeWayMergeWithSchemaChangeScripts(t *testing.T) {
 			}()
 		}
 	})
-	t.Run("left to right merges", func(t *testing.T) {
-		for _, script := range ThreeWayMergeWithSchemaChangeTestScripts {
-			func() {
-				h := newDoltHarness(t)
-				defer h.Close()
-				enginetest.TestScript(t, h, convertMergeScriptTest(script, true))
-			}()
-		}
-	})
 }
 
 func TestThreeWayMergeWithSchemaChangeScriptsPrepared(t *testing.T) {
 	skipOldFormat(t)
-	t.Run("right to left merges", func(t *testing.T) {
-		for _, script := range ThreeWayMergeWithSchemaChangeTestScripts {
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsBasicCases, "basic cases", true)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsForDataConflicts, "data conflicts", true)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsCollations, "collation changes", true)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsConstraints, "constraint changes", true)
+	runMergeScriptTestsInBothDirections(t, SchemaChangeTestsSchemaConflicts, "schema conflicts", true)
+
+	// Run non-symmetric schema merge tests in just one direction
+	t.Run("type changes", func(t *testing.T) {
+		for _, script := range SchemaChangeTestsTypeChanges {
+			// run in a func() so we can cleanly defer closing the harness
 			func() {
 				h := newDoltHarness(t)
 				defer h.Close()
-				enginetest.TestScript(t, h, convertMergeScriptTest(script, false))
+				enginetest.TestScriptPrepared(t, h, convertMergeScriptTest(script, false))
 			}()
 		}
 	})
-	t.Run("left to right merges", func(t *testing.T) {
-		for _, script := range ThreeWayMergeWithSchemaChangeTestScripts {
-			func() {
-				h := newDoltHarness(t)
-				defer h.Close()
-				enginetest.TestScript(t, h, convertMergeScriptTest(script, true))
-			}()
-		}
+}
+
+// runMergeScriptTestsInBothDirections creates a new test run, named |name|, and runs the specified merge |tests|
+// in both directions (right to left merge, and left to right merge). If
+// |runAsPrepared| is true then the test scripts will be run using the prepared
+// statement test code.
+func runMergeScriptTestsInBothDirections(t *testing.T, tests []MergeScriptTest, name string, runAsPrepared bool) {
+	t.Run(name, func(t *testing.T) {
+		t.Run("right to left merges", func(t *testing.T) {
+			for _, script := range tests {
+				// run in a func() so we can cleanly defer closing the harness
+				func() {
+					h := newDoltHarness(t)
+					defer h.Close()
+					if runAsPrepared {
+						enginetest.TestScriptPrepared(t, h, convertMergeScriptTest(script, false))
+					} else {
+						enginetest.TestScript(t, h, convertMergeScriptTest(script, false))
+					}
+				}()
+			}
+		})
+		t.Run("left to right merges", func(t *testing.T) {
+			for _, script := range tests {
+				func() {
+					h := newDoltHarness(t)
+					defer h.Close()
+					if runAsPrepared {
+						enginetest.TestScriptPrepared(t, h, convertMergeScriptTest(script, true))
+					} else {
+						enginetest.TestScript(t, h, convertMergeScriptTest(script, true))
+					}
+				}()
+			}
+		})
 	})
 }
 
