@@ -22,6 +22,7 @@ import (
 
 	"github.com/dolthub/go-mysql-server/sql"
 	"github.com/dolthub/go-mysql-server/sql/expression"
+	"github.com/dolthub/go-mysql-server/sql/fulltext"
 	sqltypes "github.com/dolthub/go-mysql-server/sql/types"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
@@ -51,6 +52,7 @@ type DoltTableable interface {
 type DoltIndex interface {
 	sql.FilteredIndex
 	sql.OrderedIndex
+	fulltext.Index
 	Schema() schema.Schema
 	IndexSchema() schema.Schema
 	Format() *types.NomsBinFormat
@@ -197,28 +199,35 @@ func DoltToFromCommitIndex(tbl string) sql.Index {
 		constrainedToLookupExpression: false,
 	}
 }
+
+// MockIndex returns a sql.Index that is not backed by an actual datastore. It's useful for system tables and
+// system table functions provide indexes but produce their rows at execution time based on the provided `IndexLookup`
+func MockIndex(columnName, tableName string, columnType types.NomsKind, unique bool) (index *doltIndex) {
+	return &doltIndex{
+		id:      columnName,
+		tblName: tableName,
+		dbName:  "",
+		columns: []schema.Column{
+			schema.NewColumn(columnName, 0, columnType, false),
+		},
+		indexSch:                      nil,
+		tableSch:                      nil,
+		unique:                        unique,
+		comment:                       "",
+		vrw:                           nil,
+		ns:                            nil,
+		order:                         sql.IndexOrderNone,
+		constrainedToLookupExpression: false,
+	}
+}
+
 func DoltCommitIndexes(tab string, db *doltdb.DoltDB, unique bool) (indexes []sql.Index, err error) {
 	if !types.IsFormat_DOLT(db.Format()) {
 		return nil, nil
 	}
 
 	return []sql.Index{
-		NewCommitIndex(&doltIndex{
-			id:      CommitHashIndexId,
-			tblName: tab,
-			dbName:  "",
-			columns: []schema.Column{
-				schema.NewColumn(CommitHashIndexId, 0, types.StringKind, false),
-			},
-			indexSch:                      nil,
-			tableSch:                      nil,
-			unique:                        unique,
-			comment:                       "",
-			vrw:                           db.ValueReadWriter(),
-			ns:                            db.NodeStore(),
-			order:                         sql.IndexOrderNone,
-			constrainedToLookupExpression: false,
-		}),
+		NewCommitIndex(MockIndex(CommitHashIndexId, tab, types.StringKind, unique)),
 	}, nil
 }
 
@@ -373,6 +382,7 @@ func getSecondaryIndex(ctx context.Context, db, tbl string, t *doltdb.Table, sch
 		tableSch:                      sch,
 		unique:                        idx.IsUnique(),
 		spatial:                       idx.IsSpatial(),
+		fulltext:                      idx.IsFullText(),
 		isPk:                          false,
 		comment:                       idx.Comment(),
 		vrw:                           vrw,
@@ -382,6 +392,38 @@ func getSecondaryIndex(ctx context.Context, db, tbl string, t *doltdb.Table, sch
 		constrainedToLookupExpression: true,
 		doltBinFormat:                 types.IsFormat_DOLT(vrw.Format()),
 		prefixLengths:                 idx.PrefixLengths(),
+		fullTextProps:                 idx.FullTextProperties(),
+	}, nil
+}
+
+// ConvertFullTextToSql converts a given Full-Text schema.Index into a sql.Index. As we do not need to write to a
+// Full-Text index, we can omit all such fields. This must not be used in any other circumstance.
+func ConvertFullTextToSql(ctx context.Context, db, tbl string, sch schema.Schema, idx schema.Index) (sql.Index, error) {
+	cols := make([]schema.Column, idx.Count())
+	for i, tag := range idx.IndexedColumnTags() {
+		cols[i], _ = idx.GetColumn(tag)
+	}
+
+	return &doltIndex{
+		id:                            idx.Name(),
+		tblName:                       tbl,
+		dbName:                        db,
+		columns:                       cols,
+		indexSch:                      idx.Schema(),
+		tableSch:                      sch,
+		unique:                        idx.IsUnique(),
+		spatial:                       idx.IsSpatial(),
+		fulltext:                      idx.IsFullText(),
+		isPk:                          false,
+		comment:                       idx.Comment(),
+		vrw:                           nil,
+		ns:                            nil,
+		keyBld:                        nil,
+		order:                         sql.IndexOrderAsc,
+		constrainedToLookupExpression: true,
+		doltBinFormat:                 true,
+		prefixLengths:                 idx.PrefixLengths(),
+		fullTextProps:                 idx.FullTextProperties(),
 	}, nil
 }
 
@@ -502,6 +544,7 @@ type doltIndex struct {
 	tableSch schema.Schema
 	unique   bool
 	spatial  bool
+	fulltext bool
 	isPk     bool
 	comment  string
 	order    sql.IndexOrder
@@ -516,6 +559,7 @@ type doltIndex struct {
 	doltBinFormat bool
 
 	prefixLengths []uint16
+	fullTextProps schema.FullTextProperties
 }
 
 var _ DoltIndex = (*doltIndex)(nil)
@@ -846,6 +890,11 @@ func (di *doltIndex) IsSpatial() bool {
 	return di.spatial
 }
 
+// IsFullText implements sql.Index
+func (di *doltIndex) IsFullText() bool {
+	return di.fulltext
+}
+
 // IsPrimaryKey implements DoltIndex.
 func (di *doltIndex) IsPrimaryKey() bool {
 	return di.isPk
@@ -890,6 +939,34 @@ func (di *doltIndex) Format() *types.NomsBinFormat {
 	return di.vrw.Format()
 }
 
+// FullTextTableNames implements sql.Index
+func (di *doltIndex) FullTextTableNames(ctx *sql.Context) (fulltext.IndexTableNames, error) {
+	return fulltext.IndexTableNames{
+		Config:      di.fullTextProps.ConfigTable,
+		Position:    di.fullTextProps.PositionTable,
+		DocCount:    di.fullTextProps.DocCountTable,
+		GlobalCount: di.fullTextProps.GlobalCountTable,
+		RowCount:    di.fullTextProps.RowCountTable,
+	}, nil
+}
+
+// FullTextKeyColumns implements sql.Index
+func (di *doltIndex) FullTextKeyColumns(ctx *sql.Context) (fulltext.KeyColumns, error) {
+	var positions []int
+	if len(di.fullTextProps.KeyPositions) > 0 {
+		positions = make([]int, len(di.fullTextProps.KeyPositions))
+		for i := range positions {
+			positions[i] = int(di.fullTextProps.KeyPositions[i])
+		}
+	}
+
+	return fulltext.KeyColumns{
+		Type:      fulltext.KeyType(di.fullTextProps.KeyType),
+		Name:      di.fullTextProps.KeyName,
+		Positions: positions,
+	}, nil
+}
+
 // keysToTuple returns a tuple that indicates the starting point for an index. The empty tuple will cause the index to
 // start at the very beginning.
 func (di *doltIndex) keysToTuple(ctx *sql.Context, keys []interface{}) (types.Tuple, error) {
