@@ -20,8 +20,8 @@ import (
 
 	sqle "github.com/dolthub/go-mysql-server"
 	"github.com/dolthub/go-mysql-server/sql"
-	"github.com/dolthub/go-mysql-server/sql/parse"
 	"github.com/dolthub/go-mysql-server/sql/plan"
+	"github.com/dolthub/go-mysql-server/sql/planbuilder"
 	"github.com/dolthub/go-mysql-server/sql/transform"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -1351,7 +1351,7 @@ func TestMergeableIndexes(t *testing.T) {
 					}
 				}
 			} else {
-				t.Log(fmt.Sprintf("%v != %v", test.finalRanges, finalRanges))
+				t.Logf("%v != %v", test.finalRanges, finalRanges)
 			}
 		})
 	}
@@ -1557,14 +1557,14 @@ func TestMergeableIndexesNulls(t *testing.T) {
 					}
 				}
 			} else {
-				t.Log(fmt.Sprintf("%v != %v", test.finalRanges, finalRanges))
+				t.Logf("%v != %v", test.finalRanges, finalRanges)
 			}
 		})
 	}
 }
 
 func ReadRangesFromQuery(ctx *sql.Context, eng *sqle.Engine, query string) ([]*noms.ReadRange, error) {
-	parsed, err := parse.Parse(ctx, query)
+	parsed, err := planbuilder.Parse(ctx, eng.Analyzer.Catalog, query)
 	if err != nil {
 		return nil, err
 	}
