@@ -15,8 +15,8 @@
 package mysql_file_handler
 
 import (
+	"context"
 	"errors"
-	"io/ioutil"
 	"os"
 	"sync"
 
@@ -53,11 +53,11 @@ func (p *Persister) Persist(ctx *sql.Context, data []byte) error {
 		}
 	}
 
-	return ioutil.WriteFile(p.privsFilePath, data, 0777)
+	return os.WriteFile(p.privsFilePath, data, 0777)
 }
 
 // LoadData reads the mysql.db file, returns nil if empty or not found
-func (p Persister) LoadData() ([]byte, error) {
+func (p Persister) LoadData(context.Context) ([]byte, error) {
 	// do nothing if no filepath specified
 	if len(p.privsFilePath) == 0 {
 		return nil, nil
@@ -67,7 +67,7 @@ func (p Persister) LoadData() ([]byte, error) {
 	defer p.fileMutex.Unlock()
 
 	// read from mysqldbFilePath, error if something other than not-exists
-	buf, err := ioutil.ReadFile(p.privsFilePath)
+	buf, err := os.ReadFile(p.privsFilePath)
 	if err != nil && !errors.Is(err, os.ErrNotExist) {
 		return nil, err
 	}
