@@ -61,6 +61,7 @@ var schemasTableCols = schema.NewColCollection(
 	mustNewColWithTypeInfo(doltdb.SchemasTablesNameCol, schema.DoltSchemasNameTag, typeinfo.CreateVarStringTypeFromSqlType(mustCreateStringType(query.Type_VARCHAR, 64, sql.Collation_utf8mb4_0900_ai_ci)), true, "", false, ""),
 	mustNewColWithTypeInfo(doltdb.SchemasTablesFragmentCol, schema.DoltSchemasFragmentTag, typeinfo.CreateVarStringTypeFromSqlType(gmstypes.LongText), false, "", false, ""),
 	mustNewColWithTypeInfo(doltdb.SchemasTablesExtraCol, schema.DoltSchemasExtraTag, typeinfo.JSONType, false, "", false, ""),
+	mustNewColWithTypeInfo(doltdb.SchemasTablesSqlModeCol, schema.DoltSchemasSqlModeTag, typeinfo.CreateVarStringTypeFromSqlType(mustCreateStringType(query.Type_VARCHAR, 256, sql.Collation_utf8mb4_0900_ai_ci)), false, "", false, ""),
 )
 
 var schemaTableSchema = schema.MustSchemaFromCols(schemasTableCols)
@@ -117,6 +118,7 @@ func migrateOldSchemasTableToNew(ctx *sql.Context, db Database, schemasTable *Wr
 	typeIdx := schemasTable.sqlSchema().IndexOfColName(doltdb.SchemasTablesTypeCol)
 	fragmentIdx := schemasTable.sqlSchema().IndexOfColName(doltdb.SchemasTablesFragmentCol)
 	extraIdx := schemasTable.sqlSchema().IndexOfColName(doltdb.SchemasTablesExtraCol)
+	sqlModeIdx := schemasTable.sqlSchema().IndexOfColName(doltdb.SchemasTablesSqlModeCol)
 
 	defer func(iter sql.RowIter, ctx *sql.Context) {
 		err := iter.Close(ctx)
@@ -142,6 +144,9 @@ func migrateOldSchemasTableToNew(ctx *sql.Context, db Database, schemasTable *Wr
 		if extraIdx >= 0 {
 			newRow[3] = sqlRow[extraIdx]
 		}
+		if sqlModeIdx >= 0 {
+			newRow[4] = sqlRow[sqlModeIdx]
+		}
 
 		newRows = append(newRows, newRow)
 	}
@@ -229,6 +234,9 @@ type schemaFragment struct {
 	name     string
 	fragment string
 	created  time.Time
+	// sqlMode indicates the SQL_MODE that was used when this schema fragment was initially parsed. SQL_MODE settings
+	// such as ANSI_QUOTES control customized parsing behavior needed for some schema fragments.
+	sqlMode string
 }
 
 func getSchemaFragmentsOfType(ctx *sql.Context, tbl *WritableDoltTable, fragType string) (sf []schemaFragment, rerr error) {
@@ -243,6 +251,7 @@ func getSchemaFragmentsOfType(ctx *sql.Context, tbl *WritableDoltTable, fragType
 	typeIdx := tbl.sqlSchema().IndexOfColName(doltdb.SchemasTablesTypeCol)
 	fragmentIdx := tbl.sqlSchema().IndexOfColName(doltdb.SchemasTablesFragmentCol)
 	extraIdx := tbl.sqlSchema().IndexOfColName(doltdb.SchemasTablesExtraCol)
+	sqlModeIdx := tbl.sqlSchema().IndexOfColName(doltdb.SchemasTablesSqlModeCol)
 
 	defer func(iter sql.RowIter, ctx *sql.Context) {
 		err := iter.Close(ctx)
@@ -265,12 +274,26 @@ func getSchemaFragmentsOfType(ctx *sql.Context, tbl *WritableDoltTable, fragType
 			continue
 		}
 
+		sqlModeString := ""
+		if sqlModeIdx >= 0 {
+			if s, ok := sqlRow[sqlModeIdx].(string); ok {
+				sqlModeString = s
+			}
+		} else {
+			defaultSqlMode, err := loadDefaultSqlMode()
+			if err != nil {
+				return nil, err
+			}
+			sqlModeString = defaultSqlMode
+		}
+
 		// For older tables, use 1 as the trigger creation time
 		if extraIdx < 0 || sqlRow[extraIdx] == nil {
 			frags = append(frags, schemaFragment{
 				name:     sqlRow[nameIdx].(string),
 				fragment: sqlRow[fragmentIdx].(string),
 				created:  time.Unix(1, 0).UTC(), // TablePlus editor thinks 0 is out of range
+				sqlMode:  sqlModeString,
 			})
 			continue
 		}
@@ -282,12 +305,27 @@ func getSchemaFragmentsOfType(ctx *sql.Context, tbl *WritableDoltTable, fragType
 			name:     sqlRow[nameIdx].(string),
 			fragment: sqlRow[fragmentIdx].(string),
 			created:  time.Unix(createdTime, 0).UTC(),
+			sqlMode:  sqlModeString,
 		})
 	}
 
 	return frags, nil
 }
 
+// loadDefaultSqlMode loads the default value for the @@SQL_MODE system variable and returns it, along
+// with any unexpected errors encountered while reading the default value.
+func loadDefaultSqlMode() (string, error) {
+	global, _, ok := sql.SystemVariables.GetGlobal("SQL_MODE")
+	if !ok {
+		return "", fmt.Errorf("unable to load default @@SQL_MODE")
+	}
+	s, ok := global.Default.(string)
+	if !ok {
+		return "", fmt.Errorf("unexpected type for @@SQL_MODE default value: %T", global.Default)
+	}
+	return s, nil
+}
+
 func getCreatedTime(ctx *sql.Context, extraCol gmstypes.JSONValue) (int64, error) {
 	doc, err := extraCol.Unmarshall(ctx)
 	if err != nil {
