@@ -15,13 +15,13 @@
 package sqlutil
 
 import (
-	"context"
+	"fmt"
 	"strings"
 
+	sqle "github.com/dolthub/go-mysql-server"
 	"github.com/dolthub/go-mysql-server/sql"
-	"github.com/dolthub/go-mysql-server/sql/parse"
 	"github.com/dolthub/go-mysql-server/sql/plan"
-	"github.com/dolthub/vitess/go/vt/sqlparser"
+	"github.com/dolthub/go-mysql-server/sql/planbuilder"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
@@ -29,51 +29,31 @@ import (
 
 // ParseCreateTableStatement will parse a CREATE TABLE ddl statement and use it to create a Dolt Schema. A RootValue
 // is used to generate unique tags for the Schema
-func ParseCreateTableStatement(ctx context.Context, root *doltdb.RootValue, query string) (string, schema.Schema, error) {
-	// todo: verify create table statement
-	p, err := sqlparser.Parse(query)
+func ParseCreateTableStatement(ctx *sql.Context, root *doltdb.RootValue, engine *sqle.Engine, query string) (string, schema.Schema, error) {
+	parsed, err := planbuilder.Parse(ctx, engine.Analyzer.Catalog, query)
 	if err != nil {
 		return "", nil, err
 	}
-	ddl := p.(*sqlparser.DDL)
+	create, ok := parsed.(*plan.CreateTable)
+	if !ok {
+		return "", nil, fmt.Errorf("expected create table, found %T", create)
+	}
 
-	sctx := sql.NewContext(ctx)
-	s, collation, err := parse.TableSpecToSchema(sctx, ddl.TableSpec, false)
+	sch, err := ToDoltSchema(ctx, root, create.Name(), create.CreateSchema, nil, create.Collation)
 	if err != nil {
 		return "", nil, err
 	}
 
-	for _, col := range s.Schema {
-		if collatedType, ok := col.Type.(sql.TypeWithCollation); ok {
-			col.Type, err = collatedType.WithNewCollation(sql.Collation_Default)
-			if err != nil {
-				return "", nil, err
-			}
-		}
-	}
-
-	buf := sqlparser.NewTrackedBuffer(nil)
-	ddl.Table.Format(buf)
-	tableName := buf.String()
-	sch, err := ToDoltSchema(ctx, root, tableName, s, nil, collation)
-	if err != nil {
-		return "", nil, err
-	}
-
-	indexes, err := parse.ConvertIndexDefs(sctx, ddl.TableSpec)
-	if err != nil {
-		return "", nil, err
-	}
-
-	for _, idx := range indexes {
+	for _, idx := range create.IdxDefs {
 		var prefixes []uint16
 		for _, c := range idx.Columns {
 			prefixes = append(prefixes, uint16(c.Length))
 		}
 		props := schema.IndexProperties{
-			IsUnique:  idx.IsUnique(),
-			IsSpatial: idx.IsSpatial(),
-			Comment:   idx.Comment,
+			IsUnique:   idx.IsUnique(),
+			IsSpatial:  idx.IsSpatial(),
+			IsFullText: idx.IsFullText(),
+			Comment:    idx.Comment,
 		}
 		name := getIndexName(idx)
 		_, err = sch.Indexes().AddIndexByColNames(name, idx.ColumnNames(), prefixes, props)
@@ -83,18 +63,14 @@ func ParseCreateTableStatement(ctx context.Context, root *doltdb.RootValue, quer
 	}
 
 	// foreign keys are stored on the *doltdb.Table object, ignore them here
-	_, checks, err := parse.ConvertConstraintsDefs(sctx, ddl.Table, ddl.TableSpec)
-	if err != nil {
-		return "", nil, err
-	}
-	for _, chk := range checks {
+	for _, chk := range create.Checks() {
 		name := getCheckConstraintName(chk)
 		_, err = sch.Checks().AddCheck(name, chk.Expr.String(), chk.Enforced)
 		if err != nil {
 			return "", nil, err
 		}
 	}
-	return tableName, sch, err
+	return create.Name(), sch, err
 }
 
 func getIndexName(def *plan.IndexDefinition) string {
