@@ -29,15 +29,16 @@ import (
 	"sync"
 
 	"github.com/dolthub/go-mysql-server/sql"
+	"github.com/dolthub/go-mysql-server/sql/fulltext"
 	sqltypes "github.com/dolthub/go-mysql-server/sql/types"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/branch_control"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb/durable"
+	"github.com/dolthub/dolt/go/libraries/doltcore/ref"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema/typeinfo"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
-	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/globalstate"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/writer"
@@ -461,8 +462,8 @@ func partitionRows(ctx *sql.Context, t *doltdb.Table, sqlSch sql.Schema, projCol
 // WritableDoltTable allows updating, deleting, and inserting new rows. It implements sql.UpdatableTable and friends.
 type WritableDoltTable struct {
 	*DoltTable
-	db Database
-	ed writer.TableWriter
+	db                 Database
+	pinnedWriteSession writer.WriteSession
 }
 
 var _ doltTableInterface = (*WritableDoltTable)(nil)
@@ -491,7 +492,6 @@ func (t *WritableDoltTable) WithProjections(colNames []string) sql.Table {
 	return &WritableDoltTable{
 		DoltTable: t.DoltTable.WithProjections(colNames).(*DoltTable),
 		db:        t.db,
-		ed:        t.ed,
 	}
 }
 
@@ -510,19 +510,248 @@ func (t *WritableDoltTable) Inserter(ctx *sql.Context) sql.RowInserter {
 func (t *WritableDoltTable) getTableEditor(ctx *sql.Context) (ed writer.TableWriter, err error) {
 	ds := dsess.DSessFromSess(ctx.Session)
 
-	state, _, err := ds.LookupDbState(ctx, t.db.RevisionQualifiedName())
-	if err != nil {
-		return nil, err
+	var writeSession writer.WriteSession
+	if t.pinnedWriteSession != nil {
+		writeSession = t.pinnedWriteSession
+	} else {
+		state, _, err := ds.LookupDbState(ctx, t.db.RevisionQualifiedName())
+		if err != nil {
+			return nil, err
+		}
+		writeSession = state.WriteSession()
 	}
 
 	setter := ds.SetRoot
-	ed, err = state.WriteSession().GetTableWriter(ctx, t.tableName, t.db.RevisionQualifiedName(), setter)
-
+	ed, err = writeSession.GetTableWriter(ctx, t.tableName, t.db.RevisionQualifiedName(), setter)
 	if err != nil {
 		return nil, err
 	}
 
-	return ed, nil
+	if t.sch.Indexes().ContainsFullTextIndex() {
+		ftEditor, err := t.getFullTextEditor(ctx)
+		if err != nil {
+			return nil, err
+		}
+		multiEditor, err := fulltext.CreateMultiTableEditor(ctx, ed, ftEditor)
+		if err != nil {
+			return nil, err
+		}
+		return multiEditor.(writer.TableWriter), nil
+	} else {
+		return ed, nil
+	}
+}
+
+// getFullTextEditor gathers all pseudo-index tables for a Full-Text index and returns an editor that will write
+// to all of them. This assumes that there are Full-Text indexes in the schema.
+func (t *WritableDoltTable) getFullTextEditor(ctx *sql.Context) (fulltext.TableEditor, error) {
+	workingRoot, err := t.workingRoot(ctx)
+	if err != nil {
+		return fulltext.TableEditor{}, err
+	}
+
+	configTable, sets, err := t.fulltextTableSets(ctx, workingRoot)
+	if err != nil {
+		return fulltext.TableEditor{}, err
+	}
+
+	return fulltext.CreateEditor(ctx, t, configTable, sets...)
+}
+
+func (t *WritableDoltTable) fulltextTableSets(ctx *sql.Context, workingRoot *doltdb.RootValue) (fulltext.EditableTable, []fulltext.TableSet, error) {
+	var configTable fulltext.EditableTable
+	var sets []fulltext.TableSet
+	for _, idx := range t.sch.Indexes().AllIndexes() {
+		if !idx.IsFullText() {
+			continue
+		}
+		props := idx.FullTextProperties()
+		// We only load the config table once since it's shared by all indexes
+		// TODO: should this load directly from the root, bypassing the session?
+		if configTable == nil {
+			tbl, ok, err := t.db.getTable(ctx, workingRoot, props.ConfigTable)
+			if err != nil {
+				return nil, nil, err
+			} else if !ok {
+				return nil, nil, fmt.Errorf("missing Full-Text table: %s", props.ConfigTable)
+			}
+			configTable = tbl.(fulltext.EditableTable)
+		}
+		// Load the rest of the tables
+		positionTable, ok, err := t.db.getTable(ctx, workingRoot, props.PositionTable)
+		if err != nil {
+			return nil, nil, err
+		} else if !ok {
+			return nil, nil, fmt.Errorf("missing Full-Text table: %s", props.PositionTable)
+		}
+		docCountTable, ok, err := t.db.getTable(ctx, workingRoot, props.DocCountTable)
+		if err != nil {
+			return nil, nil, err
+		} else if !ok {
+			return nil, nil, fmt.Errorf("missing Full-Text table: %s", props.DocCountTable)
+		}
+		globalCountTable, ok, err := t.db.getTable(ctx, workingRoot, props.GlobalCountTable)
+		if err != nil {
+			return nil, nil, err
+		} else if !ok {
+			return nil, nil, fmt.Errorf("missing Full-Text table: %s", props.GlobalCountTable)
+		}
+		rowCountTable, ok, err := t.db.getTable(ctx, workingRoot, props.RowCountTable)
+		if err != nil {
+			return nil, nil, err
+		} else if !ok {
+			return nil, nil, fmt.Errorf("missing Full-Text table: %s", props.RowCountTable)
+		}
+		// Convert the index into a sql.Index
+		sqlIdx, err := index.ConvertFullTextToSql(ctx, t.db.RevisionQualifiedName(), t.tableName, t.sch, idx)
+		if err != nil {
+			return nil, nil, err
+		}
+
+		sets = append(sets, fulltext.TableSet{
+			Index:       sqlIdx.(fulltext.Index),
+			Position:    positionTable.(fulltext.EditableTable),
+			DocCount:    docCountTable.(fulltext.EditableTable),
+			GlobalCount: globalCountTable.(fulltext.EditableTable),
+			RowCount:    rowCountTable.(fulltext.EditableTable),
+		})
+	}
+	return configTable, sets, nil
+}
+
+// tableSetsForRewrite returns the fulltext.TableSet for each Full-Text index in the table, truncated and modified
+// for a table rewrite operation. Returns the root given with all full-text pseudo tables updated with their new
+// truncated value.
+func (t *WritableDoltTable) tableSetsForRewrite(
+	ctx *sql.Context,
+	workingRoot *doltdb.RootValue,
+) (*doltdb.RootValue, fulltext.EditableTable, []fulltext.TableSet, error) {
+	configTable, sets, err := t.fulltextTableSets(ctx, workingRoot)
+	if err != nil {
+		return nil, nil, nil, err
+	}
+
+	// truncate each of the fulltext tables in each set before returning them
+	_, insertCols, err := fulltext.GetKeyColumns(ctx, t)
+	if err != nil {
+		return nil, nil, nil, err
+	}
+
+	newSets := make([]fulltext.TableSet, len(sets))
+	for i := range sets {
+		set := sets[i]
+
+		positionSch, err := fulltext.NewSchema(fulltext.SchemaPosition, insertCols, set.Position.Name(), t.Collation())
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		posTableDolt, posTable, err := emptyFulltextTable(ctx, t, workingRoot, set.Position, positionSch)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		docCountSch, err := fulltext.NewSchema(fulltext.SchemaDocCount, insertCols, set.DocCount.Name(), t.Collation())
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		dcTableDolt, dcTable, err := emptyFulltextTable(ctx, t, workingRoot, set.DocCount, docCountSch)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		globalCountSch, err := fulltext.NewSchema(fulltext.SchemaGlobalCount, nil, set.GlobalCount.Name(), t.Collation())
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		gcTableDolt, gcTable, err := emptyFulltextTable(ctx, t, workingRoot, set.GlobalCount, globalCountSch)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		rowCountSch, err := fulltext.NewSchema(fulltext.SchemaRowCount, nil, set.RowCount.Name(), t.Collation())
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		rcTableDolt, rcTable, err := emptyFulltextTable(ctx, t, workingRoot, set.RowCount, rowCountSch)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		set.Position = posTable
+		set.DocCount = dcTable
+		set.GlobalCount = gcTable
+		set.RowCount = rcTable
+
+		workingRoot, err = workingRoot.PutTable(ctx, posTable.Name(), posTableDolt)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		workingRoot, err = workingRoot.PutTable(ctx, dcTable.Name(), dcTableDolt)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		workingRoot, err = workingRoot.PutTable(ctx, gcTable.Name(), gcTableDolt)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		workingRoot, err = workingRoot.PutTable(ctx, rcTable.Name(), rcTableDolt)
+		if err != nil {
+			return nil, nil, nil, err
+		}
+
+		newSets[i] = set
+	}
+
+	return workingRoot, configTable, newSets, nil
+}
+
+// emptyFulltextTable returns a new empty fulltext table with the given schema, and the underlying dolt table.
+func emptyFulltextTable(
+	ctx *sql.Context,
+	parentTable *WritableDoltTable,
+	workingRoot *doltdb.RootValue,
+	fulltextTable fulltext.EditableTable,
+	fulltextSch sql.Schema,
+) (*doltdb.Table, fulltext.EditableTable, error) {
+	doltTable, ok := fulltextTable.(*AlterableDoltTable)
+	if !ok {
+		return nil, nil, fmt.Errorf("unexpected row count table type: %T", fulltextTable)
+	}
+
+	// TODO: this should be the head root, not working root
+	doltSchema, err := sqlutil.ToDoltSchema(ctx, workingRoot, doltTable.tableName, sql.NewPrimaryKeySchema(fulltextSch), workingRoot, parentTable.Collation())
+	if err != nil {
+		return nil, nil, err
+	}
+
+	dt, err := doltTable.DoltTable.DoltTable(ctx)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	empty, err := durable.NewEmptyIndex(ctx, dt.ValueReadWriter(), dt.NodeStore(), doltSchema)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	dt, err = doltdb.NewTable(ctx, dt.ValueReadWriter(), dt.NodeStore(), doltSchema, empty, nil, nil)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	newTable, err := parentTable.db.newDoltTable(fulltextTable.Name(), doltSchema, dt)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	return dt, newTable.(fulltext.EditableTable), nil
 }
 
 // Deleter implements sql.DeletableTable
@@ -730,7 +959,11 @@ func (t *WritableDoltTable) GetNextAutoIncrementValue(ctx *sql.Context, potentia
 		return 0, err
 	}
 
-	return ed.GetNextAutoIncrementValue(ctx, potentialVal)
+	if multiTableEditor, ok := ed.(fulltext.MultiTableEditor); ok {
+		return multiTableEditor.PrimaryEditor().(writer.AutoIncrementGetter).GetNextAutoIncrementValue(ctx, potentialVal)
+	} else {
+		return ed.(writer.AutoIncrementGetter).GetNextAutoIncrementValue(ctx, potentialVal)
+	}
 }
 
 func (t *DoltTable) GetChecks(ctx *sql.Context) ([]sql.CheckDefinition, error) {
@@ -1076,6 +1309,7 @@ type doltAlterableTableInterface interface {
 	sql.PrimaryKeyAlterableTable
 	sql.ProjectedTable
 	sql.CollationAlterableTable
+	fulltext.IndexAlterableTable
 }
 
 var _ doltAlterableTableInterface = (*AlterableDoltTable)(nil)
@@ -1129,7 +1363,7 @@ func (t *AlterableDoltTable) AddColumn(ctx *sql.Context, column *sql.Column, ord
 	}
 
 	if column.AutoIncrement {
-		ait, err := t.db.gs.GetAutoIncrementTracker(ctx)
+		ait, err := t.db.gs.AutoIncrementTracker(ctx)
 		if err != nil {
 			return err
 		}
@@ -1275,63 +1509,34 @@ func (t *AlterableDoltTable) RewriteInserter(
 	}
 	newSch = schema.CopyChecksConstraints(oldSch, newSch)
 
+	isModifyColumn := newColumn != nil && oldColumn != nil
 	if isColumnDrop(oldSchema, newSchema) {
-		newSch = schema.CopyIndexes(oldSch, newSch)
-		droppedCol := getDroppedColumn(oldSchema, newSchema)
-		for _, index := range newSch.Indexes().IndexesWithColumn(droppedCol.Name) {
-			_, err = newSch.Indexes().RemoveIndex(index.Name())
-			if err != nil {
-				return nil, err
-			}
+		newSch, err = dropIndexesOnDroppedColumn(newSch, oldSch, oldSchema, newSchema, err)
+		if err != nil {
+			return nil, err
 		}
-	} else if newColumn != nil && oldColumn != nil { // modify column
-		// It may be possible to optimize this and not always rewrite every index, but since we're already truncating the
-		// table to rewrite it we also truncate all the indexes. Much easier to get right.
-		for _, index := range oldSch.Indexes().AllIndexes() {
-			var colNames []string
-			prefixLengths := index.PrefixLengths()
-			for i, colName := range index.ColumnNames() {
-				if strings.ToLower(oldColumn.Name) == strings.ToLower(colName) {
-					colNames = append(colNames, newColumn.Name)
-					if len(prefixLengths) > 0 {
-						if !sqltypes.IsText(newColumn.Type) {
-							// drop prefix lengths if column is not a string type
-							prefixLengths[i] = 0
-						} else if uint32(prefixLengths[i]) > newColumn.Type.MaxTextResponseByteLength(ctx) {
-							// drop prefix length if prefixLength is too long
-							prefixLengths[i] = 0
-						}
-					}
-				} else {
-					colNames = append(colNames, colName)
-				}
-			}
-
-			// check if prefixLengths should be dropped entirely
-			var nonZeroPrefixLength bool
-			for _, prefixLength := range prefixLengths {
-				if prefixLength > 0 {
-					nonZeroPrefixLength = true
-					break
-				}
-			}
-			if !nonZeroPrefixLength {
-				prefixLengths = nil
-			}
-
-			newSch.Indexes().AddIndexByColNames(
-				index.Name(),
-				colNames,
-				prefixLengths,
-				schema.IndexProperties{
-					IsUnique:      index.IsUnique(),
-					IsSpatial:     index.IsSpatial(),
-					IsUserDefined: index.IsUserDefined(),
-					Comment:       index.Comment(),
-				})
+	} else if isModifyColumn {
+		newSch, err = modifyIndexesForTableRewrite(ctx, oldSch, oldColumn, newColumn, newSch)
+		if err != nil {
+			return nil, err
 		}
 	} else {
-		newSch = schema.CopyIndexes(oldSch, newSch)
+		// we need a temp version of a sql.Table here to get key columns
+		newTbl, err := t.db.newDoltTable(t.Name(), newSch, dt)
+		if err != nil {
+			return nil, err
+		}
+
+		keyCols, _, err := fulltext.GetKeyColumns(ctx, newTbl)
+		if err != nil {
+			return nil, err
+		}
+
+		// this copies over all non-full-text indexes in place
+		newSch, err = modifyFulltextIndexesForRewrite(ctx, keyCols, oldSch, newSch)
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	// If we have an auto increment column, we need to set it here before we begin the rewrite process (it may have changed)
@@ -1351,33 +1556,38 @@ func (t *AlterableDoltTable) RewriteInserter(
 		return nil, err
 	}
 
-	// We can't just call getTableEditor here because it uses the session state, which we can't update until after the
-	// rewrite operation
-	opts := dbState.WriteSession().GetOptions()
-	opts.ForeignKeyChecksDisabled = true
-
 	newRoot, err := ws.WorkingRoot().PutTable(ctx, t.Name(), dt)
 	if err != nil {
 		return nil, err
 	}
 
-	if len(oldSchema.PkOrdinals) > 0 && len(newSchema.PkOrdinals) == 0 {
+	isPrimaryKeyDrop := len(oldSchema.PkOrdinals) > 0 && len(newSchema.PkOrdinals) == 0
+	if isPrimaryKeyDrop {
 		newRoot, err = t.adjustForeignKeysForDroppedPk(ctx, t.Name(), newRoot)
 		if err != nil {
 			return nil, err
 		}
 	}
 
-	newWs := ws.WithWorkingRoot(newRoot)
+	// We can't just call getTableEditor for this operation because it uses the session state, which we can't update
+	// until after the rewrite operation
+	if newSch.Indexes().ContainsFullTextIndex() {
+		return fullTextRewriteEditor(ctx, t, newSch, dt, ws, sess, dbState, newRoot)
+	}
 
 	// TODO: figure out locking. Other DBs automatically lock a table during this kind of operation, we should probably
 	//  do the same. We're messing with global auto-increment values here and it's not safe.
-	ait, err := t.db.gs.GetAutoIncrementTracker(ctx)
+	ait, err := t.db.gs.AutoIncrementTracker(ctx)
 	if err != nil {
 		return nil, err
 	}
 
+	newWs := ws.WithWorkingRoot(newRoot)
+
+	opts := dbState.WriteSession().GetOptions()
+	opts.ForeignKeyChecksDisabled = true
 	writeSession := writer.NewWriteSession(dt.Format(), newWs, ait, opts)
+
 	ed, err := writeSession.GetTableWriter(ctx, t.Name(), t.db.RevisionQualifiedName(), sess.SetRoot)
 	if err != nil {
 		return nil, err
@@ -1386,6 +1596,229 @@ func (t *AlterableDoltTable) RewriteInserter(
 	return ed, nil
 }
 
+func fullTextRewriteEditor(
+	ctx *sql.Context,
+	t *AlterableDoltTable,
+	newSch schema.Schema,
+	dt *doltdb.Table,
+	ws *doltdb.WorkingSet,
+	sess *dsess.DoltSession,
+	dbState dsess.SessionState,
+	workingRoot *doltdb.RootValue,
+) (sql.RowInserter, error) {
+
+	newTable, err := t.db.newDoltTable(t.Name(), newSch, dt)
+	if err != nil {
+		return nil, err
+	}
+
+	updatedRoot, configTable, tableSets, err := newTable.(*AlterableDoltTable).tableSetsForRewrite(ctx, workingRoot)
+	if err != nil {
+		return nil, err
+	}
+
+	// TODO: figure out locking. Other DBs automatically lock a table during this kind of operation, we should probably
+	//  do the same. We're messing with global auto-increment values here and it's not safe.
+	ait, err := t.db.gs.AutoIncrementTracker(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	newWs := ws.WithWorkingRoot(updatedRoot)
+
+	// We need our own write session for the rewrite operation. The connection's session must continue to return rows of
+	// the table as it existed before the rewrite operation began until it completes, at which point we update the
+	// session with the rewritten table.
+	opts := dbState.WriteSession().GetOptions()
+	opts.ForeignKeyChecksDisabled = true
+	writeSession := writer.NewWriteSession(dt.Format(), newWs, ait, opts)
+
+	parentEditor, err := writeSession.GetTableWriter(ctx, t.Name(), t.db.RevisionQualifiedName(), sess.SetRoot)
+	if err != nil {
+		return nil, err
+	}
+
+	// There's a layer of indirection here: the call to fulltext.CreateEditor is going to in turn ask each of these
+	// tables for an Inserter, and we need to return the one we're using to do the rewrite, not a fresh one from the
+	// session's data (which still has the tables as they existed before the rewrite began). To get around this,
+	// we manually set the writeSession in these tables before passing control back to the engine. Then in Inserter(),
+	// we check for a pinned writeSession and return that one, not the session one.
+	for i := range tableSets {
+		tableSets[i].Position.(*AlterableDoltTable).SetWriteSession(writeSession)
+		tableSets[i].DocCount.(*AlterableDoltTable).SetWriteSession(writeSession)
+		tableSets[i].GlobalCount.(*AlterableDoltTable).SetWriteSession(writeSession)
+		tableSets[i].RowCount.(*AlterableDoltTable).SetWriteSession(writeSession)
+	}
+
+	ftEditor, err := fulltext.CreateEditor(ctx, newTable, configTable, tableSets...)
+	if err != nil {
+		return nil, err
+	}
+
+	multiEditor, err := fulltext.CreateMultiTableEditor(ctx, parentEditor, ftEditor)
+	if err != nil {
+		return nil, err
+	}
+
+	return multiEditor, nil
+}
+
+// modifyFulltextIndexesForRewrite modifies the fulltext indexes of a table to correspond to the new schema before
+// a table rewrite. All non-full-text indexes are copied from the old schema directly.
+func modifyFulltextIndexesForRewrite(ctx *sql.Context, keyCols fulltext.KeyColumns, oldSch schema.Schema, newSch schema.Schema) (schema.Schema, error) {
+	for _, idx := range oldSch.Indexes().AllIndexes() {
+		if !idx.IsFullText() {
+			newSch.Indexes().AddIndex(idx)
+			continue
+		}
+
+		ft := idx.FullTextProperties()
+		keyColPositions := make([]uint16, len(keyCols.Positions))
+		for i, pos := range keyCols.Positions {
+			keyColPositions[i] = uint16(pos)
+		}
+
+		ft.KeyPositions = keyColPositions
+		ft.KeyType = uint8(keyCols.Type)
+
+		props := schema.IndexProperties{
+			IsUnique:           idx.IsUnique(),
+			IsSpatial:          idx.IsSpatial(),
+			IsFullText:         true,
+			IsUserDefined:      true,
+			Comment:            idx.Comment(),
+			FullTextProperties: ft,
+		}
+
+		newSch.Indexes().AddIndexByColNames(idx.Name(), idx.ColumnNames(), idx.PrefixLengths(), props)
+	}
+
+	return newSch, nil
+}
+
+// dropIndexesOnDroppedColumn removes from the schema any indexes which contain a dropped column.
+func dropIndexesOnDroppedColumn(newSch schema.Schema, oldSch schema.Schema, oldSchema sql.PrimaryKeySchema, newSchema sql.PrimaryKeySchema, err error) (schema.Schema, error) {
+	newSch = schema.CopyIndexes(oldSch, newSch)
+	droppedCol := getDroppedColumn(oldSchema, newSchema)
+	for _, index := range newSch.Indexes().IndexesWithColumn(droppedCol.Name) {
+		_, err = newSch.Indexes().RemoveIndex(index.Name())
+		if err != nil {
+			return nil, err
+		}
+
+		// For fulltext indexes, we don't just remove them entirely on a column drop, we modify them to contain only the
+		// remaining columns
+		if index.IsFullText() {
+			modifyFulltextIndexForColumnDrop(index, newSch, droppedCol)
+		}
+	}
+
+	return newSch, nil
+}
+
+// modifyFulltextIndexForColumnDrop modifies a fulltext index to remove a column that was dropped, adding it to the
+// schema's indexes only if there are still remaining columns in the index after the drop
+func modifyFulltextIndexForColumnDrop(index schema.Index, newSch schema.Schema, droppedCol *sql.Column) {
+	if len(index.ColumnNames()) == 1 {
+		// if there was only one column left in the index, we remove it entirely
+		return
+	}
+
+	var i int
+	colNames := make([]string, len(index.ColumnNames())-1)
+	for _, col := range index.ColumnNames() {
+		if col == droppedCol.Name {
+			continue
+		}
+		colNames[i] = col
+		i++
+	}
+
+	newSch.Indexes().AddIndexByColNames(
+		index.Name(),
+		colNames,
+		index.PrefixLengths(),
+		schema.IndexProperties{
+			IsUnique:           index.IsUnique(),
+			IsSpatial:          false,
+			IsFullText:         true,
+			IsUserDefined:      index.IsUserDefined(),
+			Comment:            index.Comment(),
+			FullTextProperties: index.FullTextProperties(),
+		})
+}
+
+func modifyIndexesForTableRewrite(ctx *sql.Context, oldSch schema.Schema, oldColumn *sql.Column, newColumn *sql.Column, newSch schema.Schema) (schema.Schema, error) {
+	for _, index := range oldSch.Indexes().AllIndexes() {
+		if index.IsFullText() {
+			err := validateFullTextColumnChange(ctx, index, oldColumn, newColumn)
+			if err != nil {
+				return nil, err
+			}
+		}
+
+		var colNames []string
+		prefixLengths := index.PrefixLengths()
+		for i, colName := range index.ColumnNames() {
+			if strings.ToLower(oldColumn.Name) == strings.ToLower(colName) {
+				colNames = append(colNames, newColumn.Name)
+				if len(prefixLengths) > 0 {
+					if !sqltypes.IsText(newColumn.Type) {
+						// drop prefix lengths if column is not a string type
+						prefixLengths[i] = 0
+					} else if uint32(prefixLengths[i]) > newColumn.Type.MaxTextResponseByteLength(ctx) {
+						// drop prefix length if prefixLength is too long
+						prefixLengths[i] = 0
+					}
+				}
+			} else {
+				colNames = append(colNames, colName)
+			}
+		}
+
+		// check if prefixLengths should be dropped entirely
+		var nonZeroPrefixLength bool
+		for _, prefixLength := range prefixLengths {
+			if prefixLength > 0 {
+				nonZeroPrefixLength = true
+				break
+			}
+		}
+		if !nonZeroPrefixLength {
+			prefixLengths = nil
+		}
+
+		newSch.Indexes().AddIndexByColNames(
+			index.Name(),
+			colNames,
+			prefixLengths,
+			schema.IndexProperties{
+				IsUnique:           index.IsUnique(),
+				IsSpatial:          index.IsSpatial(),
+				IsFullText:         index.IsFullText(),
+				IsUserDefined:      index.IsUserDefined(),
+				Comment:            index.Comment(),
+				FullTextProperties: index.FullTextProperties(),
+			})
+	}
+
+	return newSch, nil
+}
+
+// validateFullTextColumnChange returns an error if the column change given violates this full text index.
+func validateFullTextColumnChange(ctx *sql.Context, idx schema.Index, oldColumn *sql.Column, newColumn *sql.Column) error {
+	colNames := idx.ColumnNames()
+	for _, colName := range colNames {
+		if oldColumn.Name != colName && newColumn.Name != colName {
+			continue
+		}
+		if !sqltypes.IsTextOnly(newColumn.Type) {
+			return sql.ErrFullTextInvalidColumnType.New()
+		}
+	}
+	return nil
+}
+
 func (t *AlterableDoltTable) getNewSch(ctx context.Context, oldColumn, newColumn *sql.Column, oldSch schema.Schema, newSchema sql.PrimaryKeySchema, root, headRoot *doltdb.RootValue) (schema.Schema, error) {
 	if oldColumn == nil || newColumn == nil {
 		// Adding or dropping a column
@@ -1644,6 +2077,7 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 
 	// For auto columns modified to be auto increment, we have more work to do
 	if !existingCol.AutoIncrement && col.AutoIncrement {
+		// TODO: delegate this to tracker?
 		seq, err := t.getFirstAutoIncrementValue(ctx, columnName, column.Type, updatedTable)
 		if err != nil {
 			return err
@@ -1654,14 +2088,19 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 			return err
 		}
 
-		ait, err := t.db.gs.GetAutoIncrementTracker(ctx)
+		ait, err := t.db.gs.AutoIncrementTracker(ctx)
 		if err != nil {
 			return err
 		}
 
-		// TODO: this isn't transactional, and it should be
+		// TODO: this isn't transactional, and it should be (but none of the auto increment tracking is)
 		ait.AddNewTable(t.tableName)
-		ait.Set(t.tableName, seq)
+		// Since this is a new auto increment table, we don't need to exclude the current working set from consideration
+		// when computing its new sequence value, hence the empty ref
+		_, err = ait.Set(ctx, t.tableName, updatedTable, ref.WorkingSetRef{}, seq)
+		if err != nil {
+			return err
+		}
 	}
 
 	// If we're removing an auto inc property, we just need to update global auto increment tracking
@@ -1734,7 +2173,7 @@ func (t *AlterableDoltTable) getFirstAutoIncrementValue(
 		}
 	}
 
-	seq, err := globalstate.CoerceAutoIncrementValue(initialValue)
+	seq, err := dsess.CoerceAutoIncrementValue(initialValue)
 	if err != nil {
 		return 0, err
 	}
@@ -1775,70 +2214,7 @@ func (t *AlterableDoltTable) CreateIndex(ctx *sql.Context, idx sql.IndexDef) err
 		return fmt.Errorf("only the following types of index constraints are supported: none, unique, spatial")
 	}
 
-	columns := make([]string, len(idx.Columns))
-	for i, indexCol := range idx.Columns {
-		columns[i] = indexCol.Name
-	}
-
-	table, err := t.DoltTable.DoltTable(ctx)
-	if err != nil {
-		return err
-	}
-
-	ret, err := creation.CreateIndex(
-		ctx,
-		table,
-		idx.Name,
-		columns,
-		allocatePrefixLengths(idx.Columns),
-		idx.Constraint == sql.IndexConstraint_Unique,
-		idx.Constraint == sql.IndexConstraint_Spatial,
-		true,
-		idx.Comment,
-		t.opts,
-	)
-	if err != nil {
-		return err
-	}
-	root, err := t.getRoot(ctx)
-	if err != nil {
-		return err
-	}
-	if ret.OldIndex != nil && ret.OldIndex != ret.NewIndex { // old index was replaced, so we update foreign keys
-		fkc, err := root.GetForeignKeyCollection(ctx)
-		if err != nil {
-			return err
-		}
-		for _, fk := range fkc.AllKeys() {
-			newFk := fk
-			if t.tableName == fk.TableName && fk.TableIndex == ret.OldIndex.Name() {
-				newFk.TableIndex = ret.NewIndex.Name()
-			}
-			if t.tableName == fk.ReferencedTableName && fk.ReferencedTableIndex == ret.OldIndex.Name() {
-				newFk.ReferencedTableIndex = ret.NewIndex.Name()
-			}
-			fkc.RemoveKeys(fk)
-			err = fkc.AddKeys(newFk)
-			if err != nil {
-				return err
-			}
-		}
-		root, err = root.PutForeignKeyCollection(ctx, fkc)
-		if err != nil {
-			return err
-		}
-	}
-	newRoot, err := root.PutTable(ctx, t.tableName, ret.NewTable)
-	if err != nil {
-		return err
-	}
-
-	err = t.setRoot(ctx, newRoot)
-
-	if err != nil {
-		return err
-	}
-	return t.updateFromRoot(ctx, newRoot)
+	return t.createIndex(ctx, idx, fulltext.KeyColumns{}, fulltext.IndexTableNames{})
 }
 
 // DropIndex implements sql.IndexAlterableTable
@@ -1910,6 +2286,110 @@ func (t *AlterableDoltTable) RenameIndex(ctx *sql.Context, fromIndexName string,
 	return t.updateFromRoot(ctx, newRoot)
 }
 
+// CreateFulltextIndex implements fulltext.IndexAlterableTable
+func (t *AlterableDoltTable) CreateFulltextIndex(ctx *sql.Context, idx sql.IndexDef, keyCols fulltext.KeyColumns, tableNames fulltext.IndexTableNames) error {
+	if !types.IsFormat_DOLT(t.Format()) {
+		return fmt.Errorf("FULLTEXT is not supported on storage format %s. Run `dolt migrate` to upgrade to the latest storage format.", t.Format().VersionString())
+	}
+	if err := dsess.CheckAccessForDb(ctx, t.db, branch_control.Permissions_Write); err != nil {
+		return err
+	}
+	if idx.Constraint != sql.IndexConstraint_Fulltext {
+		return fmt.Errorf("attempted to create non-FullText index through FullText interface")
+	}
+
+	return t.createIndex(ctx, idx, keyCols, tableNames)
+}
+
+// createIndex handles the common functionality between CreateIndex and CreateFulltextIndex.
+func (t *AlterableDoltTable) createIndex(ctx *sql.Context, idx sql.IndexDef, keyCols fulltext.KeyColumns, tableNames fulltext.IndexTableNames) error {
+	columns := make([]string, len(idx.Columns))
+	for i, indexCol := range idx.Columns {
+		columns[i] = indexCol.Name
+	}
+
+	table, err := t.DoltTable.DoltTable(ctx)
+	if err != nil {
+		return err
+	}
+
+	var keyPositions []uint16
+	if len(keyCols.Positions) > 0 {
+		keyPositions = make([]uint16, len(keyCols.Positions))
+		for i := range keyPositions {
+			keyPositions[i] = uint16(keyCols.Positions[i])
+		}
+	}
+
+	ret, err := creation.CreateIndex(
+		ctx,
+		table,
+		idx.Name,
+		columns,
+		allocatePrefixLengths(idx.Columns),
+		schema.IndexProperties{
+			IsUnique:      idx.Constraint == sql.IndexConstraint_Unique,
+			IsSpatial:     idx.Constraint == sql.IndexConstraint_Spatial,
+			IsFullText:    idx.Constraint == sql.IndexConstraint_Fulltext,
+			IsUserDefined: true,
+			Comment:       idx.Comment,
+			FullTextProperties: schema.FullTextProperties{
+				ConfigTable:      tableNames.Config,
+				PositionTable:    tableNames.Position,
+				DocCountTable:    tableNames.DocCount,
+				GlobalCountTable: tableNames.GlobalCount,
+				RowCountTable:    tableNames.RowCount,
+				KeyType:          uint8(keyCols.Type),
+				KeyName:          keyCols.Name,
+				KeyPositions:     keyPositions,
+			},
+		},
+		t.opts,
+	)
+	if err != nil {
+		return err
+	}
+	root, err := t.getRoot(ctx)
+	if err != nil {
+		return err
+	}
+	if ret.OldIndex != nil && ret.OldIndex != ret.NewIndex { // old index was replaced, so we update foreign keys
+		fkc, err := root.GetForeignKeyCollection(ctx)
+		if err != nil {
+			return err
+		}
+		for _, fk := range fkc.AllKeys() {
+			newFk := fk
+			if t.tableName == fk.TableName && fk.TableIndex == ret.OldIndex.Name() {
+				newFk.TableIndex = ret.NewIndex.Name()
+			}
+			if t.tableName == fk.ReferencedTableName && fk.ReferencedTableIndex == ret.OldIndex.Name() {
+				newFk.ReferencedTableIndex = ret.NewIndex.Name()
+			}
+			fkc.RemoveKeys(fk)
+			err = fkc.AddKeys(newFk)
+			if err != nil {
+				return err
+			}
+		}
+		root, err = root.PutForeignKeyCollection(ctx, fkc)
+		if err != nil {
+			return err
+		}
+	}
+	newRoot, err := root.PutTable(ctx, t.tableName, ret.NewTable)
+	if err != nil {
+		return err
+	}
+
+	err = t.setRoot(ctx, newRoot)
+
+	if err != nil {
+		return err
+	}
+	return t.updateFromRoot(ctx, newRoot)
+}
+
 // createForeignKey creates a doltdb.ForeignKey from a sql.ForeignKeyConstraint
 func (t *AlterableDoltTable) createForeignKey(
 	ctx *sql.Context,
@@ -2167,10 +2647,13 @@ func (t *AlterableDoltTable) CreateIndexForForeignKey(ctx *sql.Context, idx sql.
 		idx.Name,
 		columns,
 		allocatePrefixLengths(idx.Columns),
-		idx.Constraint == sql.IndexConstraint_Unique,
-		idx.Constraint == sql.IndexConstraint_Spatial,
-		false,
-		"",
+		schema.IndexProperties{
+			IsUnique:      idx.Constraint == sql.IndexConstraint_Unique,
+			IsSpatial:     idx.Constraint == sql.IndexConstraint_Spatial,
+			IsFullText:    idx.Constraint == sql.IndexConstraint_Fulltext,
+			IsUserDefined: false,
+			Comment:       "",
+		},
 		t.opts,
 	)
 	if err != nil {
@@ -2586,6 +3069,10 @@ func (t *AlterableDoltTable) DropPrimaryKey(ctx *sql.Context) error {
 	return fmt.Errorf("not implemented: AlterableDoltTable.DropPrimaryKey()")
 }
 
+func (t *WritableDoltTable) SetWriteSession(session writer.WriteSession) {
+	t.pinnedWriteSession = session
+}
+
 func findIndexWithPrefix(sch schema.Schema, prefixCols []string) (schema.Index, bool, error) {
 	type idxWithLen struct {
 		schema.Index
