@@ -26,7 +26,6 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb/durable"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
-	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/globalstate"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/writer"
@@ -111,7 +110,7 @@ func NewTempTable(
 
 	newWs := ws.WithWorkingRoot(newRoot)
 
-	ait, err := globalstate.NewAutoIncrementTracker(ctx, newWs)
+	ait, err := dsess.NewAutoIncrementTracker(ctx, db, newWs)
 	if err != nil {
 		return nil, err
 	}
@@ -161,7 +160,7 @@ func setTempTableRoot(t *TempTable) func(ctx *sql.Context, dbName string, newRoo
 		}
 		newWs := ws.WithWorkingRoot(newRoot)
 
-		ait, err := globalstate.NewAutoIncrementTracker(ctx, newWs)
+		ait, err := dsess.NewAutoIncrementTracker(ctx, t.dbName, newWs)
 		if err != nil {
 			return err
 		}
@@ -258,7 +257,7 @@ func (t *TempTable) PartitionRows(ctx *sql.Context, partition sql.Partition) (sq
 	}
 }
 
-func (t *TempTable) IndexedAccess(_ sql.IndexLookup) sql.IndexedTable {
+func (t *TempTable) IndexedAccess(lookup sql.IndexLookup) sql.IndexedTable {
 	return t
 }
 
@@ -277,10 +276,13 @@ func (t *TempTable) CreateIndex(ctx *sql.Context, idx sql.IndexDef) error {
 		idx.Name,
 		cols,
 		allocatePrefixLengths(idx.Columns),
-		idx.Constraint == sql.IndexConstraint_Unique,
-		idx.Constraint == sql.IndexConstraint_Spatial,
-		true,
-		idx.Comment,
+		schema.IndexProperties{
+			IsUnique:      idx.Constraint == sql.IndexConstraint_Unique,
+			IsSpatial:     idx.Constraint == sql.IndexConstraint_Spatial,
+			IsFullText:    idx.Constraint == sql.IndexConstraint_Fulltext,
+			IsUserDefined: true,
+			Comment:       idx.Comment,
+		},
 		t.opts,
 	)
 	if err != nil {
