@@ -87,7 +87,7 @@ func ExecuteSql(dEnv *env.DoltEnv, root *doltdb.RootValue, statements string) (*
 			if execErr == nil {
 				execErr = drainIter(ctx, rowIter)
 			}
-		case *sqlparser.DDL, *sqlparser.MultiAlterDDL:
+		case *sqlparser.DDL, *sqlparser.AlterTable:
 			var rowIter sql.RowIter
 			_, rowIter, execErr = engine.Query(ctx, query)
 			if execErr == nil {
