@@ -76,7 +76,7 @@ func (db *UserSpaceDatabase) GetTableNames(ctx *sql.Context) ([]string, error) {
 	return resultingTblNames, nil
 }
 
-func (db *UserSpaceDatabase) InitialDBState(ctx *sql.Context, branch string) (dsess.InitialDbState, error) {
+func (db *UserSpaceDatabase) InitialDBState(ctx *sql.Context) (dsess.InitialDbState, error) {
 	return dsess.InitialDbState{
 		Db:       db,
 		ReadOnly: true,
@@ -87,6 +87,15 @@ func (db *UserSpaceDatabase) InitialDBState(ctx *sql.Context, branch string) (ds
 	}, nil
 }
 
+func (db *UserSpaceDatabase) WithBranchRevision(requestedName string, branchSpec dsess.SessionDatabaseBranchSpec) (dsess.SqlDatabase, error) {
+	// Nothing to do here, we don't support changing branch revisions
+	return db, nil
+}
+
+func (db *UserSpaceDatabase) DoltDatabases() []*doltdb.DoltDB {
+	return nil
+}
+
 func (db *UserSpaceDatabase) GetRoot(*sql.Context) (*doltdb.RootValue, error) {
 	return db.RootValue, nil
 }
@@ -99,10 +108,6 @@ func (db *UserSpaceDatabase) DbData() env.DbData {
 	return env.DbData{}
 }
 
-func (db *UserSpaceDatabase) Flush(ctx *sql.Context) error {
-	panic("UserSpaceDatabase cannot flush")
-}
-
 func (db *UserSpaceDatabase) EditOptions() editor.Options {
 	return db.editOpts
 }
@@ -111,10 +116,18 @@ func (db *UserSpaceDatabase) Revision() string {
 	return ""
 }
 
+func (db *UserSpaceDatabase) Versioned() bool {
+	return false
+}
+
 func (db *UserSpaceDatabase) RevisionType() dsess.RevisionType {
 	return dsess.RevisionTypeNone
 }
 
-func (db *UserSpaceDatabase) BaseName() string {
+func (db *UserSpaceDatabase) RevisionQualifiedName() string {
+	return db.Name()
+}
+
+func (db *UserSpaceDatabase) RequestedName() string {
 	return db.Name()
 }
