@@ -17,9 +17,10 @@ package writer
 import (
 	"github.com/dolthub/go-mysql-server/sql"
 
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/globalstate"
+
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
-	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/globalstate"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
 	"github.com/dolthub/dolt/go/libraries/doltcore/table/editor"
@@ -29,7 +30,13 @@ import (
 type TableWriter interface {
 	sql.TableEditor
 	sql.ForeignKeyEditor
-	sql.AutoIncrementEditor
+	sql.AutoIncrementSetter
+}
+
+// AutoIncrementGetter is implemented by editors that support AUTO_INCREMENT to return the next value that will be
+// inserted.
+type AutoIncrementGetter interface {
+	GetNextAutoIncrementValue(ctx *sql.Context, insertVal interface{}) (uint64, error)
 }
 
 // SessionRootSetter sets the root value for the session.
@@ -55,12 +62,15 @@ type nomsTableWriter struct {
 	tableEditor editor.TableEditor
 	flusher     WriteSessionFlusher
 
-	autoInc globalstate.AutoIncrementTracker
+	autoInc                globalstate.AutoIncrementTracker
+	nextAutoIncrementValue map[string]uint64
 
-	setter SessionRootSetter
+	setter         SessionRootSetter
+	errEncountered error
 }
 
 var _ TableWriter = &nomsTableWriter{}
+var _ AutoIncrementGetter = &nomsTableWriter{}
 
 func (te *nomsTableWriter) duplicateKeyErrFunc(keyString, indexName string, k, v types.Tuple, isPk bool) error {
 	oldRow, err := te.kvToSQLRow.ConvertKVTuplesToSqlRow(k, v)
@@ -128,12 +138,13 @@ func (te *nomsTableWriter) GetNextAutoIncrementValue(ctx *sql.Context, insertVal
 }
 
 func (te *nomsTableWriter) SetAutoIncrementValue(ctx *sql.Context, val uint64) error {
-	seq, err := globalstate.CoerceAutoIncrementValue(val)
+	seq, err := te.autoInc.CoerceAutoIncrementValue(val)
 	if err != nil {
 		return err
 	}
-	te.autoInc.Set(te.tableName, seq)
-	te.tableEditor.MarkDirty()
+
+	te.nextAutoIncrementValue = make(map[string]uint64)
+	te.nextAutoIncrementValue[te.tableName] = seq
 
 	return te.flush(ctx)
 }
@@ -158,16 +169,25 @@ func (te *nomsTableWriter) GetIndexes(ctx *sql.Context) ([]sql.Index, error) {
 
 // Close implements Closer
 func (te *nomsTableWriter) Close(ctx *sql.Context) error {
-	return te.flush(ctx)
+	if te.errEncountered == nil {
+		return te.flush(ctx)
+	}
+	return nil
 }
 
 // StatementBegin implements the interface sql.TableEditor.
 func (te *nomsTableWriter) StatementBegin(ctx *sql.Context) {
+	// Table writers are reused in a session, which means we need to reset the error state resulting from previous
+	// errors on every new statement.
+	te.errEncountered = nil
 	te.tableEditor.StatementStarted(ctx)
 }
 
 // DiscardChanges implements the interface sql.TableEditor.
 func (te *nomsTableWriter) DiscardChanges(ctx *sql.Context, errorEncountered error) error {
+	if _, ignored := errorEncountered.(sql.IgnorableError); !ignored {
+		te.errEncountered = errorEncountered
+	}
 	return te.tableEditor.StatementFinished(ctx, true)
 }
 
