@@ -208,6 +208,11 @@ func (m Map) Mutate() *MutableMap {
 	return newMutableMap(m)
 }
 
+// Rewriter returns a mutator that intends to rewrite this map with the key and value descriptors provided.
+func (m Map) Rewriter(kd, vd val.TupleDesc) *MutableMap {
+	return newMutableMapWithDescriptors(m, kd, vd)
+}
+
 // Count returns the number of key-value pairs in the Map.
 func (m Map) Count() (int, error) {
 	return m.tuples.Count()
