@@ -40,7 +40,7 @@ func (tvs TupleValueSlice) Kind() NomsKind {
 	return TupleKind
 }
 
-func (tvs TupleValueSlice) Less(nbf *NomsBinFormat, other LesserValuable) (bool, error) {
+func (tvs TupleValueSlice) Less(ctx context.Context, nbf *NomsBinFormat, other LesserValuable) (bool, error) {
 	switch typedOther := other.(type) {
 	case Tuple:
 		val, err := NewTuple(nbf, tvs...)
@@ -49,7 +49,7 @@ func (tvs TupleValueSlice) Less(nbf *NomsBinFormat, other LesserValuable) (bool,
 			return false, err
 		}
 
-		return typedOther.Less(nbf, val)
+		return typedOther.Less(ctx, nbf, val)
 
 	case TupleValueSlice:
 		myLen := len(tvs)
@@ -77,7 +77,7 @@ func (tvs TupleValueSlice) Less(nbf *NomsBinFormat, other LesserValuable) (bool,
 			}
 
 			if !val.Equals(otherVal) {
-				return val.Less(nbf, otherVal)
+				return val.Less(ctx, nbf, otherVal)
 			}
 		}
 
@@ -659,7 +659,7 @@ func (t Tuple) splitFieldsAt(n uint64) (prolog, head, tail []byte, count uint64,
 	return
 }
 
-func (t Tuple) TupleCompare(nbf *NomsBinFormat, otherTuple Tuple) (int, error) {
+func (t Tuple) TupleCompare(ctx context.Context, nbf *NomsBinFormat, otherTuple Tuple) (int, error) {
 	itrs := tupItrPairPool.Get().(*tupleItrPair)
 	defer tupItrPairPool.Put(itrs)
 
@@ -824,7 +824,7 @@ func (t Tuple) TupleCompare(nbf *NomsBinFormat, otherTuple Tuple) (int, error) {
 			if err != nil {
 				return 0, err
 			}
-			res, err = blob.Compare(nbf, otherBlob)
+			res, err = blob.Compare(ctx, nbf, otherBlob)
 			if err != nil {
 				return 0, err
 			}
@@ -845,7 +845,7 @@ func (t Tuple) TupleCompare(nbf *NomsBinFormat, otherTuple Tuple) (int, error) {
 			if v.Equals(otherV) {
 				continue
 			} else {
-				isLess, err := v.Less(nbf, otherV)
+				isLess, err := v.Less(ctx, nbf, otherV)
 				if err != nil {
 					return 0, err
 				} else if isLess {
@@ -864,13 +864,13 @@ func (t Tuple) TupleCompare(nbf *NomsBinFormat, otherTuple Tuple) (int, error) {
 	return int(itr.Len()) - int(otherItr.Len()), nil
 }
 
-func (t Tuple) Less(nbf *NomsBinFormat, other LesserValuable) (bool, error) {
+func (t Tuple) Less(ctx context.Context, nbf *NomsBinFormat, other LesserValuable) (bool, error) {
 	otherTuple, ok := other.(Tuple)
 	if !ok {
 		return TupleKind < other.Kind(), nil
 	}
 
-	res, err := t.TupleCompare(nbf, otherTuple)
+	res, err := t.TupleCompare(ctx, nbf, otherTuple)
 	if err != nil {
 		return false, err
 	}
@@ -878,13 +878,13 @@ func (t Tuple) Less(nbf *NomsBinFormat, other LesserValuable) (bool, error) {
 	return res < 0, err
 }
 
-func (t Tuple) Compare(nbf *NomsBinFormat, other LesserValuable) (int, error) {
+func (t Tuple) Compare(ctx context.Context, nbf *NomsBinFormat, other LesserValuable) (int, error) {
 	otherTuple, ok := other.(Tuple)
 	if !ok {
 		return int(TupleKind) - int(other.Kind()), nil
 	}
 
-	res, err := t.TupleCompare(nbf, otherTuple)
+	res, err := t.TupleCompare(ctx, nbf, otherTuple)
 	if err != nil {
 		return 0, err
 	}
