@@ -28,10 +28,18 @@ func TestCluster(t *testing.T) {
 	RunTestsFile(t, "tests/sql-server-cluster.yaml")
 }
 
+func TestClusterUsersAndGrants(t *testing.T) {
+	RunTestsFile(t, "tests/sql-server-cluster-users-and-grants.yaml")
+}
+
+func TestRemotesAPI(t *testing.T) {
+	RunTestsFile(t, "tests/sql-server-remotesapi.yaml")
+}
+
 // TestSingle is a convenience method for running a single test from within an IDE. Unskip and set to the file and name
 // of the test you want to debug. See README.md in the `tests` directory for more debugging info.
 func TestSingle(t *testing.T) {
-	// t.Skip()
+	t.Skip()
 	RunSingleTest(t, "tests/sql-server-cluster.yaml", "primary comes up and replicates to standby")
 }
 
@@ -46,3 +54,7 @@ func TestOriginal(t *testing.T) {
 func TestTLS(t *testing.T) {
 	RunTestsFile(t, "tests/sql-server-tls.yaml")
 }
+
+func TestClusterReadOnly(t *testing.T) {
+	RunTestsFile(t, "tests/sql-server-cluster-read-only.yaml")
+}
