@@ -6,7 +6,7 @@ export const viewsTests = [
   {
     q: "CALL DOLT_CHECKOUT('-b', :branchName)",
     p: { branchName: "more-updates" },
-    res: [{ status: 0 }],
+    res: [{ status: 0, message: "Switched to branch 'more-updates'" }],
   },
   {
     q: "SELECT * FROM ::tableName ::col0 LIMIT :limit OFFSET :offset",
@@ -34,6 +34,7 @@ export const viewsTests = [
         name: "myview",
         fragment: "CREATE VIEW `myview` AS SELECT * FROM test",
         extra: { CreatedAt: 0 },
+        sql_mode: 'STRICT_TRANS_TABLES,NO_ENGINE_SUBSTITUTION,ONLY_FULL_GROUP_BY',
       },
     ],
   },
